package com.bleujin.lore.core;

import junit.framework.TestCase;

import com.bleujin.framework.db.DBController;
import com.bleujin.framework.db.servant.NoneServant;
import com.bleujin.framework.db.servant.StdOutServant;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.dimpl.AbstractNode;
import com.bleujin.lore.core.dimpl.NodeTypeManager;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.workspace.AnotherLore;
import com.bleujin.lore.core.workspace.TestCredential;
import com.bleujin.lore.core.workspace.Workspace;
import com.bleujin.lore.core.workspace.config.InitLore;

public abstract class TestCaseParent extends TestCase {

	protected AnotherLore anotherLore = null;
	protected UserSession session = null;
	protected Workspace workspace ;
	protected NodeType objectType = null;
	protected NodeTypeManager ntManager ;
	private DBController dc = null ;
	
	public void setUp() throws Exception {
		this.setUp("anony");
	}
	public void tearDown() throws Exception {
		this.tearDown("anony");
	}
	
	public void setUp(String name) throws Exception {
		// dbSetUp() ;
		transientSetUp() ;
	}
	
	public void tearDown(String name) throws Exception {
		// dbTearDown() ;
		transientTearDown() ;
	}
	
	
	private void transientSetUp() throws Exception{
		anotherLore = InitLore.initTest() ;
		// deleteIndexDir(); 

		session = anotherLore.login(new TestCredential("bleujin"));
		session.getWorkspace().clearStore();
		anotherLore.getGlobalStore().clearStore() ;

		workspace = session.getWorkspace() ;
		ntManager = session.getNodeTypeManager() ;
		NewNodeType root = createNewNodeType(NodeType.NO_SUPERTYPE, "_object", new PropertyDefinition[0]);
		objectType = registerNodeType(root) ;
	}
	
	
	private void dbSetUp() throws Exception {
		
		dc = DBController.getTestInstance() ;
		dc.init() ;
		
		anotherLore = InitLore.initTest(dc) ;
		// deleteIndexDir(); 

		session = anotherLore.login(new TestCredential("bleujin"));
		session.getWorkspace().clearStore() ;
		anotherLore.getGlobalStore().clearStore() ;
		
		workspace = session.getWorkspace() ;
		ntManager = session.getNodeTypeManager() ;
		NewNodeType root = createNewNodeType(NodeType.NO_SUPERTYPE, "_object", new PropertyDefinition[0]);
		objectType = registerNodeType(root) ;
	}
	
	protected void traceOn(){
		dc.changeServant(new StdOutServant(StdOutServant.All)) ;
	}
	
	protected void traceOff(){
		dc.changeServant(new NoneServant()) ;
	}
	
	
	private void dbTearDown() throws Exception {
		anotherLore.destroyWorld();
	}

	public void transientTearDown() throws Exception {
		if (anotherLore != null) anotherLore.destroyWorld();
	}

	protected NodeType getRobotType() throws ALRepositoryException {
		if (ntManager.hasNodeType("robot")) {
			return ntManager.getNodeType("robot");
		} else {
			return registerNodeType(getRobotTypeTemplate()) ;
		}
	}

	protected NewNodeType getRobotTypeTemplate() throws ALRepositoryException {
		PropertyDefinition explain = createPropertyDefinition("explain", Property.Type.STRING);
		NewNodeType newType = createNewNodeType(objectType, "robot", new PropertyDefinition[] { explain });
		newType.setConstraint("explain", new RequireConstraint());
		return newType;
	}

	protected Node saveNode(NodeType nodetype, String name) throws ALRepositoryException {
		return createNodeSave(session.getRootNode(), nodetype, name).save();
	}

	protected Node createNodeSave(NodeType nodetype, String name, String explain) throws ALRepositoryException {
		return createNodeSave(session.getRootNode(), nodetype, name, explain).save();
	}

	protected Node createNodeSave(Node parent, NodeType nodetype, String name) throws ALRepositoryException {
		return session.createNode(parent, nodetype, name).save();
	}

	protected Node createNodeSave(Node parent, NodeType nodetype, String name, String explain) throws ALRepositoryException {
		return session.createNode(parent, nodetype, name, explain).save();
	}

	protected NewNodeType createNewNodeType(NodeType superType, String ntId, PropertyDefinition[] definitions) throws ALRepositoryException {
		return session.getNodeTypeManager().createNodeTypeTemplate(superType, ntId, definitions);
	}

	protected NodeType registerNodeType(NodeType superType, String ntId, PropertyDefinition[] definitions) throws ALRepositoryException {
		return session.getNodeTypeManager().register(superType, ntId, definitions);
	}

	protected PropertyDefinition createPropertyDefinition(String propId, Property.Type requireType) throws ALRepositoryException {
		return session.getNodeTypeManager().createPropertyDefinition(propId, requireType);
	}
	
	protected void println(Object... obj) {
		Debug.debug(this, obj);
	}

	protected void println(GenericIterator<Link> links) {
		Debug.debug(this, links) ;
	}

	protected void confirmLocal() throws ALRepositoryException{
		session.getLocalStore().confirm(session) ;
	}

	private String TEST_FIRST_NODE_NAME = "_test";
	protected Node createTestNode() throws ALRepositoryException{
		return session.createNode(session.getRootNode(), objectType, TEST_FIRST_NODE_NAME).save();
	}

	protected Node getTestFirstNode() throws ALRepositoryException{
		return session.getNode("/" + TEST_FIRST_NODE_NAME) ;
	}
	
	protected String getTestNodeName(){
		return TEST_FIRST_NODE_NAME ;
	}
	
	protected String addBrace(String uuid){
		return AbstractNode.ID_BEGIN_MARK + uuid + AbstractNode.ID_END_MARK ;
	}
	public NodeType registerNodeType(NewNodeType newType) throws ALRepositoryException {
		return ntManager.register(newType, false);
	}
}
