package com.bleujin.lore.core.common;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.ecs.xml.XML;

import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.AlreadyMovedException;

public class GenericIteratorHelper<T> implements GenericIterator<T>, IConstant, Publishable {

	static final long UNDETERMINED_SIZE = -1;

//	public static final GenericIteratorHelper EMPTY = new GenericIteratorHelper(Collections.EMPTY_LIST);

	private final Iterator<T> iter;
	private long size;
	private long pos;

	public GenericIteratorHelper(Collection<T> list) {
		this(list.iterator());
		size = list.size();
	}

	public GenericIteratorHelper(Iterator<T> iter) {
		this.iter = iter;
		pos = 0;
		size = UNDETERMINED_SIZE;
	}

	
	
	public void skip(long skipNum) {
		while (skipNum-- > 0) {
			next();
		}
	}

	public long size() {
		return size;
	}

	public long getPosition() {
		return pos;
	}

	public boolean hasNext() {
		return iter.hasNext();
	}

	public T next() {
		T obj = iter.next();
		// increment position
		pos++;
		return obj;
	}

	public void remove() {
		iter.remove();
	}
	
	public List<T> toList(){
		if (pos != 0L) throw new AlreadyMovedException() ;
		List result = new ArrayList<T>() ;
		
		while(hasNext()){
			result.add(next()) ;
		}
		
		return result ;
	}

	
	public XML toXML() throws ALRepositoryException{
		if (pos != 0L) throw new AlreadyMovedException() ;
		XML root = new XML(ROOT) ;
		while(hasNext()) {
			T obj = next() ;
			root.addElement(((Publishable)obj).toXML()) ;
		}
		
		return root ;
	}
	
	public String toString() {
		StringBuffer buffer = new StringBuffer() ;
		buffer.append("[") ;
		for( T obj : this){
			buffer.append(obj.toString()+ "', ") ;
		}
		buffer.append("]") ;
		return buffer.toString() ;
	}

	public Iterator<T> iterator() {
		return this.iter;
	}

	public boolean hasMoreElements() {
		return hasNext();
	}

	public T nextElement() {
		return next();
	}

}
