package com.bleujin.lore.core.common;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.node.Node;

public class TestGenericIterator extends TestCaseParent {

	
	public void testPage() throws Exception {
		assertEquals(1, new Double(Math.ceil(1.0 /10.0)).intValue()) ;
		assertEquals(101, Page.create(10, 1, 10).getMaxCount()) ;
		assertEquals(101, Page.create(10, 10, 10).getMaxCount()) ;
		assertEquals(201, Page.create(10, 11, 10).getMaxCount()) ;
		assertEquals(201, Page.create(10, 19, 10).getMaxCount()) ;
		
		Page p = Page.create(10, 1) ;
		assertEquals(2, p.getNextPage().getPageNo()) ;
		assertEquals(2, p.getNextPage().getPageNo()) ;
		assertEquals(3, p.getNextPage().getNextPage().getPageNo()) ;
		 
		p = p.getNextPage().getNextPage() ;
		assertEquals(2, p.getPrePage().getPageNo()) ;
		assertEquals(2, p.getPrePage().getPageNo()) ;
		assertEquals(1, p.getPrePage().getPrePage().getPageNo()) ;
		assertEquals(1, p.getPrePage().getPrePage().getPrePage().getPageNo()) ;
}
	
	public void testSlice() throws Exception {
		List<String> list  = new ArrayList<String>() ;
		assertEquals(0, ListUtil.slice(list, Page.create(10, 2)).size()) ;
		
		list.add("a");
		list.add("b");
		list.add("c");
		list.add("d");
		assertEquals(2, ListUtil.slice(list, Page.create(2, 1)).size()) ;
		assertEquals(2, ListUtil.slice(list, Page.create(2, 2)).size()) ;
		assertEquals(0, ListUtil.slice(list, Page.create(2, 3)).size()) ;
		
		Iterator iter = ListUtil.slice(list, Page.create(2, 2)).iterator() ;
		assertEquals("c", iter.next()) ;
		assertEquals("d", iter.next()) ;
		assertEquals(false, iter.hasNext()) ;
	}
	
	public void testIterator() throws Exception {
		List<String> ss = new ArrayList<String>();
		ss.add("abc") ;
		ss.add("def") ;
		
		GenericIterator<String> sl = new GenericIteratorHelper<String>(ss);
		assertEquals("abc", sl.next()) ;
		assertEquals("def", sl.next()) ;
	}
	
	public void testOverall() throws Exception{
		List<Node> nodeList = new ArrayList<Node>();
		Node mbNode = saveNode(objectType, "myeongbak");
		Node ghNode = saveNode(objectType, "geunhae");
		nodeList.add(mbNode);
		nodeList.add(ghNode);
		GenericIterator<Node> iter = new GenericIteratorHelper<Node>(nodeList);
		assertEquals(mbNode, iter.next());
		assertEquals(1, iter.getPosition());
		assertEquals(ghNode, iter.next());
		assertEquals(2, iter.getPosition());
		assertEquals(nodeList.size(), iter.size());

	}

	public void testSkip() throws Exception {
		List<Node> nodeList = new ArrayList<Node>();
		Node mbNode = saveNode(objectType, "myeongbak");
		Node ghNode = saveNode(objectType, "geunhae");
		Node gkNode = saveNode(objectType, "gokun");
		nodeList.add(mbNode);
		nodeList.add(ghNode);
		nodeList.add(gkNode);
		GenericIterator<Node> iter = new GenericIteratorHelper<Node>(nodeList);

		iter = new GenericIteratorHelper<Node>(nodeList);
		iter.skip(1);
		iter.skip(1);
		assertEquals(gkNode, iter.next());
		
		try {
			iter.next();
			fail() ;
		} catch(NoSuchElementException ignore){}
		
		iter = new GenericIteratorHelper<Node>(nodeList);
		iter.skip(-1) ;
		assertEquals(mbNode, iter.next()) ;
	}
	
	public void testArray() throws Exception {
		List<Node> nodeList = new ArrayList<Node>();
		Node mbNode = saveNode(objectType, "myeongbak");
		Node ghNode = saveNode(objectType, "geunhae");
		Node gkNode = saveNode(objectType, "gokun");
		nodeList.add(mbNode);
		nodeList.add(ghNode);
		nodeList.add(gkNode);
		GenericIterator<Node> iter = new GenericIteratorHelper<Node>(nodeList);

		
    }
}
