package com.bleujin.lore.core.dimpl;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.ecs.xml.XML;

import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.core.bean.Rating;
import com.bleujin.lore.core.bean.Register;
import com.bleujin.lore.core.bean.SourceFrom;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.IConstant;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.IllegalNodeNameException;
import com.bleujin.lore.core.exception.NotMatchePropertyTypeException;
import com.bleujin.lore.core.mapping.ValidMessage;
import com.bleujin.lore.core.node.BinaryPropertyValue;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.Property.Type;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.observation.Event.EventType;
import com.bleujin.lore.core.persitence.ILocalStore;
import com.bleujin.lore.core.persitence.NodeBean;
import com.bleujin.lore.core.value.Value;

public abstract class AbstractINode implements INode, Serializable, IConstant {

	protected abstract NodeBean getBean() ;
	public abstract void setProperty(IDString propId, Value value) throws ALRepositoryException; // realize...
	public abstract Property getProperty(IDString propId) throws ALRepositoryException;


	public String getName() {
		return getBean().getName().getString() ;
	}

	
	public void setProperty(String propId, String value) throws ALRepositoryException{
		// checkPropertyType(propId, Property.Type.STRING);
		setProperty(propId, newValue(propId, value)) ;
	}

	public void setProperty(String propId, Object value) throws ALRepositoryException{
		setProperty(propId, newValue(propId, value)) ;
	}

	public void setProperty(String propId, Value value) throws ALRepositoryException{
		setProperty(IDString.propId(propId), value) ;
	}

	public void setProperty(String propId, BinaryPropertyValue value) throws ALRepositoryException {
		checkPropertyType(propId, Property.Type.BINARY);
		this.setProperty(propId, Value.binaryValue(value));
	}

	public void setProperty(String pid, boolean value) throws ALRepositoryException {
		checkPropertyType(pid, Property.Type.BOOLEAN);
		this.setProperty(pid, Value.booleanValue(value));
	}

	public void setProperty(String pid, int value) throws ALRepositoryException {
		checkPropertyType(pid, Property.Type.LONG);
		this.setProperty(pid, Value.longValue((long)value));
	}

	public void setProperty(String pid, BigDecimal value) throws ALRepositoryException {
		checkPropertyType(pid, Property.Type.DECIMAL);
		this.setProperty(pid, Value.decimalValue(value));
	}

	public void setProperty(String pid, long value) throws ALRepositoryException {
		checkPropertyType(pid, Property.Type.LONG);
		this.setProperty(pid, Value.longValue(value));
	}

	public void setProperty(String pid, Calendar value) throws ALRepositoryException {
		checkPropertyType(pid, Property.Type.DATE);
		this.setProperty(pid, Value.dateValue(value));
	}

	public void setProperty(String pid, Date value) throws ALRepositoryException {
		checkPropertyType(pid, Property.Type.DATE);
		Calendar cal = Calendar.getInstance() ;
		cal.setTime(value) ;
		this.setProperty(pid, Value.dateValue(cal));
	}

	public void setProperty(String propId, Object[] values) throws ALRepositoryException {
		if (values == null || values.length == 0) this.setProperty(propId, (Object)null) ;
		else {
			List<Value> result = new ArrayList<Value>() ;
			for (Object v : values) {
				result.add(newValue(propId, v)) ;
			}
			this.setProperty(propId, result) ;
		}
	}
	
	public void setProperty(String[] pid, Object[] value) throws ALRepositoryException {
		for (int i = 0, last = value.length; i < last; i++) {
			setProperty(pid[i], value[i]);
		}
	}
		

	protected Value newValue(String propId, Object value) throws ALRepositoryException {
		Type requireType = getProperty(propId).getType();
		if (! Value.isMatchedType(requireType, value)){
			throw new NotMatchePropertyTypeException(requireType + " required at " + propId);
		}
		
	    return Value.newValue(requireType, value);
    }
	private void checkPropertyType(String propId, Property.Type currentType) throws ALRepositoryException {
	    Type requireType = getProperty(propId).getType();
		if (! requireType.equals(currentType)) {
			throw new NotMatchePropertyTypeException(requireType + " required. But " + currentType) ;
		}
    }
	
	public boolean isSame(Node n2) {
		if (n2 == null) return false ;
		
		return this.getIdentifier().equals(n2.getIdentifier());
	}
	
	public boolean equals(Object other) {
		if (other instanceof INode) {
			return isSame((Node) other);
		}
		return false;
	}

	public int hashCode() {
		return getIdentifier().hashCode();
	}
	
	public XML toXML() throws ALRepositoryException{
		XML result = new XML(NODE) ;
		
		result.addElement(getBean().toXML()) ;

		GenericIterator<Property> properties = getProperties() ;
		for(Property property : properties){
			result.addElement(property.getId().getString(), property.toXML()) ;
		}
		return result ;
	}

	
	
	
	
	
	
	
	public String getUUIDString(){
		return ID_BEGIN_MARK + getIdentifier() + ID_END_MARK ;
	}
	
	
	public String getIdentifier() {
		return getBean().getIdentifier();
	}
	
	public NodeTypeId getNodeTypeId(){
		return getBean().getNodeTypeId() ;
	}

	public Calendar getCreated(){
		return getBean().getRegister().getCreateDate() ;
	}
	
	public String getExplain() {
		return getBean().getExplain() ;
	}
	
	public void setExplain(String explain) {
		getBean().setExplain(explain) ;
		getSession().fireEvent(this, EventType.NODE_UPDATED) ;
	}
	
	public void setName(String name) throws IllegalNodeNameException {
		setName(NodeName.nodeName(name)) ;
	}
	
	public void setName(NodeName name){
		getBean().setName(name) ;
		getSession().fireEvent(this, EventType.NODE_UPDATED) ;
	}
	
	public void setRegister(Register register) {
		getBean().setRegister(register) ;
	}
	public Register getRegister() {
		return getBean().getRegister() ;
	}
	public void setSourceFrom(SourceFrom from) {
		getBean().setSourceFrom(from) ;
	}
	public SourceFrom getSourceFrom() {
		return getBean().getSourceFrom() ;
	}
	public void addRating(Rating rating) {
		getBean().addRating(rating) ;
	}
	public String getStatus() {
		return getBean().getStatus() ;
	}
	public String getWorkspaceName(){
		return getBean().getWorkspaceName() ;
	}
	
	public void increaseViewCount() {
		getBean().increaseViewCount() ;
	}

	public Property getProperty(String propId) throws ALRepositoryException{
		Property property = getProperty(IDString.propId(propId)) ;
		if (property == null) throw new ALRepositoryException("exception.property.notfound[" + propId + "]");
		return property ;
	}
	
	public String getString(String propId) throws ALRepositoryException{
		return getProperty(propId).getString() ;
	}

	public NodeType getNodeType() throws ALRepositoryException {
	    NodeType nodetype = getSession().getNodeTypeManager().getNodeType(getBean().getNodeTypeId().getString());
	    return nodetype ;
    }

	
	public Value getValue(String propId) throws ALRepositoryException {
		return getProperty(propId).getValue() ;
	}
	
	public boolean hasProperty(String propId) throws ALRepositoryException {
		return getNodeType().containsPropId(propId) ;
	}

	public boolean hasProperties() throws ALRepositoryException {
		return getNodeType().getMappedDefinitions().size() > 0;
	}
	
	public String getPath() throws ALRepositoryException{
		if (isRoot()) return "" ;
		String result = getName();
		for (Node p = getParent() ; p.isRoot() == false ; p = p.getParent()) {
			result = p.getName() + "/" + result;
		}
		return "/" + result ;
	}
	

	public GenericIterator<ValidMessage> getViolateConstraintMessage() throws ALRepositoryException{
		return getNodeType().getViolateConstraintMessage(this) ;
	}

	
	protected ILocalStore getLocalStore(){
		return getSession().getLocalStore() ;
	}
	
	
	public Node getNode(String nodeInfo) throws ALRepositoryException{
		if (nodeInfo == null) throw new IllegalArgumentException("node path or uuid is " + nodeInfo) ;
		if (StringUtil.isEmpty(nodeInfo)) return getSession().getRootNode() ; // root
		if (nodeInfo.startsWith(ID_BEGIN_MARK) && nodeInfo.endsWith(ID_END_MARK)) return getSession().getNode(nodeInfo) ;
		if (nodeInfo.startsWith("/")) return getSession().getNode(nodeInfo); 
		return getSession().getNode(getPath() + "/" + nodeInfo) ;
	}

	public boolean isRoot() {
		return false;
	}

	
}
