package com.bleujin.lore.core.dimpl;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.ItemVisitor;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Lock;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.VersionHistory;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.observation.Event.Mode;
import com.bleujin.lore.core.value.Value;

public abstract class AbstractNode extends AbstractINode implements Node{

	private static final long serialVersionUID = -8263227381657368078L;
	
	public GenericIterator<Node> getNodes(String linkType, int listNum, int pageNo) throws ALRepositoryException{
		return getNodes(LinkTypeImpl.load(linkType), Page.create(listNum, pageNo)) ;
	}

	public GenericIterator<Node> getMemberNode(int listNum, int pageNo) throws ALRepositoryException {
		return getMemberNode(Page.create(listNum, pageNo)) ;
	}
	public GenericIterator<Node> getChildNode() throws ALRepositoryException {
		return getChildNode(Page.HUNDRED_ONE) ;
	}
	public GenericIterator<Link> getLinks() throws ALRepositoryException  {
		return getLinks(Page.HUNDRED_ONE) ;
	}
	public GenericIterator<Link> getLinks(String linkType) throws ALRepositoryException  {
		return getLinks(LinkTypeImpl.load(linkType), Page.ALL) ;
	}
	public GenericIterator<Link> getLinks(String linkType, int listNum, int pageNo) throws ALRepositoryException  {
		return getLinks(LinkTypeImpl.load(linkType), Page.create(listNum, pageNo)) ;
	}
	
	

	public boolean isSameVersion(Node n2) {
		return isSame(n2) && (getVersionNum() == n2.getVersionNum());
	}
	
	public String toString() {
		return getClass().getName() + "[name:" + getName() + "]" + "{" + getVersionNum() + "}";
	}
	
	public int hashCode() {
		return getIdentifier().hashCode() + getVersionNum();
	}

	public Lock lock(boolean isDeep) throws ALRepositoryException{
		return getSession().lock(this, isDeep) ;
	}

	public boolean isLocked() throws ALRepositoryException {
		return getSession().isLocked(this);
    }

	public Lock getLock() throws ALRepositoryException {
		return getSession().getLock(this);
    }

	public void addMemberNode(Node member) throws ALRepositoryException {
		if (getNodeType().isAllowedMemberType(member.getNodeType())) {
			addLink(member, LinkTypeImpl.MEMBER, Position.BEFORE);
		} else {
			throw new IllegalArgumentException(this.getNodeType().getName() + " not supported member type : " + member.getNodeType().getName());
		}
	}


	public GenericIterator<Node> searchNode(String searchText) throws ALRepositoryException{
		return getLocalStore().searchNode(getSession(), this, searchText, Page.HUNDRED_ONE) ;
	}

	public GenericIterator<Node> searchNode(String searchText, Page page) throws ALRepositoryException{
		return getLocalStore().searchNode(getSession(), this, searchText, page) ;
	}
	
	public VersionHistory getVersionHistory() throws ALRepositoryException {
		return getLocalStore().getVersionHistory(this);
	}

	public long getViewCount(){
		return getBean().getViewCount() ;
	}
	
	public GenericIterator<Node> getMemberNode() throws ALRepositoryException {
		return getNodes(LinkTypeImpl.MEMBER, Page.TEN);
	}

	public GenericIterator<Node> getMemberNode(Page page) throws ALRepositoryException {
		return getNodes(LinkTypeImpl.MEMBER, page);
	}
	
	public void delete() throws ALRepositoryException {
		setMode(Mode.DELETED) ;
		getSession().delNode(this) ;
	}
	
	public INode createChild(NodeType nodetype, String name) throws ALRepositoryException{
		return getSession().createNode(this, nodetype, name) ;
	}
	
	public Node save() throws ALRepositoryException {
		getSession().save(this);
		return this ;
	}

	public void accept(ItemVisitor v) {
		v.visit(this); 
	}
	
	protected void createRefernceLink(Property.Type type, Value value) throws ALRepositoryException {
		if (value.isBlank()) return ;
		
		// TODO ...
		// require existing node ?
		if (Property.Type.REFERENCE.equals(type)) {
			this.addLink(getNode(ID_BEGIN_MARK + value.getString() + ID_END_MARK), LinkTypeImpl.REFERENCE, Position.AFTER) ;
		} else if (Property.Type.PATH.equals(type)) {
			this.addLink(getNode(value.getString()), LinkTypeImpl.PATH, Position.AFTER) ;
		}
	}

}
