package com.bleujin.lore.core.dimpl;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.NamespaceException;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.mapping.IDefaultValue;
import com.bleujin.lore.core.mapping.IValueConstraint;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.persitence.NamespaceRegistry;
import com.bleujin.lore.core.persitence.Sequence;
import com.bleujin.lore.core.persitence.URI;
import com.bleujin.lore.extend.template.Template;
import com.bleujin.lore.extend.template.TemplateId;

public class CacheGlobalStore extends AbstractGlobalStore {

	
	private AbstractGlobalStore base ;
	private Map<NodeTypeId, NodeType> nodetypes ;
	private NamespaceRegistry namespace ;
	public CacheGlobalStore(AbstractGlobalStore base){
		this.base = base ;
		this.nodetypes =  Collections.synchronizedMap(new HashMap<NodeTypeId, NodeType>()) ;
		this.namespace = new NamespaceRegistry(this) ;
	}
	
	@Override
	public void clearStore() throws ALRepositoryException {
		base.clearStore() ;
		nodetypes.clear() ;
	}

	@Override
	public void confirm() {
		base.confirm() ;
	}

	@Override
	public NewNodeType createNodeType(NodeType superType, String ntId, PropertyDefinition[] definitions) throws ALRepositoryException {
		return base.createNodeType(superType, ntId, definitions);
	}

	public GenericIterator<NodeType> register(NewNodeType[] newNodeTypes, boolean allowUpdate) throws ALRepositoryException {
		List<NodeType> created = base.register(newNodeTypes, allowUpdate).toList() ;
		for (NodeType nodeType : created) {
			NodeTypeImpl.load(this, (NodeTypeImpl)nodeType) ;
		}
		
		return new GenericIteratorHelper<NodeType>(created) ;
	}
	
	@Override
	public GenericIterator<NodeType> getAllNodeTypes(String prefixId) throws ALRepositoryException {
		return base.getAllNodeTypes(prefixId);
	}

	@Override
	public LinkType getLinkType(String linkTypeName) throws ALRepositoryException {
		return base.getLinkType(linkTypeName);
	}

	@Override
	public GenericIterator<MappedDefinition> getMappedDefinition(NodeType nodetype) throws ALRepositoryException {
		return base.getMappedDefinition(nodetype);
	}

	public NodeType getNodeType(AbstractGlobalStore globalStore, NodeTypeId ntId) throws NamespaceException, ALRepositoryException {
		if (nodetypes.containsKey(ntId)) {
			// Debug.debug("FOUND : " + ntId) ;
			return nodetypes.get(ntId) ;
		}
		NodeType nodetype = base.getNodeType(this, ntId);
		nodetypes.put(nodetype.getId(), nodetype) ;
		
		return nodetype ;
    }	

	public Sequence getSequence() {
		return base.getSequence();
	}

	public boolean includeNodeType(String ntId) throws ALRepositoryException {
		if (nodetypes.containsKey(makeNodeTypeId(ntId))) return true ;
		return base.includeNodeType(ntId);
	}

	@Override
	public void removeNodeType(String ntId) throws ALRepositoryException {
		nodetypes.remove(makeNodeTypeId(ntId)) ;
		base.removeNodeType(ntId) ;
	}

	@Override
	public void setMemberType(NodeType nodetype, NodeType memberType) throws ALRepositoryException {
		nodetypes.remove(nodetype.getId()) ;
		base.setMemberType(nodetype, memberType) ;
	}

	@Override
	public void setPropertyConstraint(NodeType nodetype, PropertyDefinition prop, IValueConstraint constraint) throws ALRepositoryException {
		nodetypes.remove(nodetype.getId()) ;
		base.setPropertyConstraint(nodetype, prop, constraint) ;
	}

	@Override
	public void setPropertyDefaultValue(NodeType nodetype, PropertyDefinition prop, IDefaultValue defaultValue) throws ALRepositoryException {
		nodetypes.remove(nodetype.getId()) ;
		base.setPropertyDefaultValue(nodetype, prop, defaultValue) ;
	}

	private NodeTypeId makeNodeTypeId(String ntId) throws NamespaceException {
		return NodeTypeIdImpl.create(getNamespace(), ntId);
	}
	
	@Override
	public void setPrimaryProperty(NodeType nodetype, PropertyDefinition pd) throws ALRepositoryException {
		nodetypes.remove(nodetype.getId()) ;
		base.setPrimaryProperty(nodetype, pd) ;
	}


	@Override
	public ShareInfo getShareInfo(String shareName) throws ALRepositoryException {
		return base.getShareInfo(shareName);
	}

	@Override
	public void shareNode(String shareName, String workspaceName, String nodePath, boolean enableWrite) {
		base.shareNode(shareName, workspaceName, nodePath, enableWrite) ;
	}

	@Override
	public Template saveTemplate(TemplateId tplId, Template template) throws ALRepositoryException {
		return base.saveTemplate(tplId, template);
	}

	@Override
	public Template getBoundTemplate(Node node) throws ALRepositoryException {
		return base.getBoundTemplate(node);
	}

	@Override
	public Template getTemplate(UserSession session, TemplateId tplId) throws ALRepositoryException {
		return base.getTemplate(session, tplId);
	}

	@Override
	public GenericIterator<TemplateId> getTemplates(UserSession session) throws ALRepositoryException {
		return base.getTemplates(session);
	}

	@Override
	public void bindTemplate(Node node, TemplateId tplId) throws ALRepositoryException {
		base.bindTemplate(node, tplId) ;
	}

	@Override
	public void bindTemplate(NodeType nodeType, TemplateId tplId) throws ALRepositoryException {
		base.bindTemplate(nodeType, tplId) ;
	}

	public void close() {
		base.close();
	}

	@Override
	public GenericIterator<ShareInfo> getShareInfos() throws ALRepositoryException {
		return base.getShareInfos();
	}

	@Override
	public NamespaceRegistry getNamespace() {
		return namespace ;
	}


	private Map<String, URI> cacheNamespace = Collections.synchronizedMap(new HashMap<String, URI>()) ;
	protected  int addURIAddress(String prefix, String address) {
		int result = base.addURIAddress(prefix, address);
		cacheNamespace.put(prefix, new URI(prefix, address)) ;
		return result ;
	}

	protected  void removeURIAddress(String prefix) {
		base.removeURIAddress(prefix) ;
		cacheNamespace.remove(prefix) ;
	}

	protected  boolean existURIAddress(String prefix) throws NamespaceException {
		if (cacheNamespace.containsKey(prefix)) return true ;
		
		boolean result = base.existURIAddress(prefix);
		if (result) {
			URI uri = getURIAddress(prefix) ;
			cacheNamespace.put(uri.getPrefix(), uri) ;
		}
		return result ;
	}

	protected  URI getURIAddress(String prefix) throws NamespaceException {
		if (cacheNamespace.containsKey(prefix)) return cacheNamespace.get(prefix) ;
		URI uri = base.getURIAddress(prefix);
		if (uri != null) cacheNamespace.put(prefix, uri) ;
		return uri ;
	}

	protected  GenericIterator<URI> listURIAddress() {
		GenericIterator<URI> result =  base.listURIAddress();
		Map<String, URI> newMap = new HashMap<String, URI>() ;
		for (URI uri : result) {
			newMap.put(uri.getPrefix(), new URI(uri.getPrefix(), uri.getAddress())) ;
		}
		cacheNamespace = newMap ;
		return new GenericIteratorHelper<URI>(cacheNamespace.values()) ;
	}



}
