package com.bleujin.lore.core.dimpl;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections15.map.LRUMap;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.SearchException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.VersionHistory;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.observation.NodeEvent;
import com.bleujin.lore.core.observation.PropertyEvent;
import com.bleujin.lore.core.observation.Event.EventType;
import com.bleujin.lore.core.persitence.ILocalStore;
import com.bleujin.lore.core.search.indexer.NodeIndexer;
import com.bleujin.lore.core.search.searcher.NodeSearcher;
import com.bleujin.lore.core.value.Value;


public class CacheLocalStore implements ILocalStore{

	
	private ILocalStore base ;
	private Map<String, Node> cacheNodes = Collections.synchronizedMap(new LRUMap<String, Node>(500)) ;
	
	public CacheLocalStore(ILocalStore base){
		this.base = base ;
	}
	
	public NodeIndexer getIndexer() {
		return base.getIndexer();
	}

	public NodeSearcher getSearcher(UserSession session, Node startNode) {
		return base.getSearcher(session, startNode);
	}
	public void clear() {
		base.clear() ;
	}

	public void save(List<Event> pendingEvent) throws ALRepositoryException {
		base.save(pendingEvent) ;
		clearCache(pendingEvent) ;
	}

	public synchronized void save(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException {
		base.save(pendingEvent, thisNode) ;
		cacheNodes.remove(thisNode.getIdentifier()) ;
	}
	
	private void clearCache(List<Event> pendingEvent) throws ALRepositoryException {
		synchronized (mutex) {
			for (Event event : pendingEvent) {
				if (EventType.PROPERTY_ADDED.equals(event.getType())){
					cacheNodes.remove(((PropertyEvent) event).getProperty().getNode().getIdentifier()) ;
				} else if (EventType.PROPERTY_CHANGED.equals(event.getType())){
					cacheNodes.remove(((PropertyEvent) event).getProperty().getNode().getIdentifier()) ;
				} else if (EventType.NODE_MOVED.equals(event.getType())) {
					// TODO : select Node
					cacheNodes.clear() ;
				} else if (EventType.NODE_REMOVED.equals(event.getType())) {
					cacheNodes.remove(((NodeEvent) event).getNode().getIdentifier()) ;
				} else if (EventType.NODE_CHECKOUTED.equals(event.getType())) {
					cacheNodes.remove(((NodeEvent) event).getNode().getIdentifier()) ;
				}
			}
		}
		// cacheNodes.clear() ;
	}

	
	public void confirm(UserSession session) throws ALRepositoryException {
		base.confirm(session) ;
	}

	public void copy(Node src, Node destParent) throws ALRepositoryException {
		base.copy(src, destParent) ;
	}

	public INode createNode(UserSession session, Node parent, NodeType nodetype, NodeName name, String explain) {
		return base.createNode(session, parent, nodetype, name, explain);
	}

	public boolean exist(UserSession session, String path) throws ALRepositoryException {
		return base.exist(session, path);
	}

	public GenericIterator<Link> getLinks(Node node, Page page) throws ALRepositoryException {
		return base.getLinks(node, page);
	}

	public GenericIterator<Link> getLinks(Node node, LinkType linkType, Page page) throws ALRepositoryException {
		return base.getLinks(node, linkType, page);
	}

	private Object mutex = new Object() ;
	public Node getNodeByPath(UserSession session, String path) throws ALRepositoryException {
		Node foundNode = null ;
		synchronized (mutex) {
			for (Node cache : cacheNodes.values()) {
				if (cache.getPath().equals(path) && isSameWorkspace(session, cache)){
					// TODO : change session..
					foundNode = cache ;
					break ;
				}
			}
		}
		if (foundNode != null && foundNode instanceof NodeDBImpl) return foundNode.newClone(session) ;
		
		Node node = base.getNodeByPath(session, path);
		loadMemory(node) ;
		return node ;
	}

	private boolean isSameWorkspace(UserSession session, Node node) {
		return node.getWorkspaceName().equals(session.getWorkspace().getName());
	}

	public Node getNodeByUUID(UserSession session, String uuid) throws ALRepositoryException {
		Node cache = cacheNodes.get(uuid) ;
		if (cache != null && isSameWorkspace(session, cache) && cache instanceof NodeDBImpl) {
			return ((NodeDBImpl)cache).newClone(session) ; 
		} 
		Node node = base.getNodeByUUID(session, uuid);
		loadMemory(node) ;
		return node ;
	}

	public GenericIterator<Node> getNodes(Node node, LinkType type, Page page) throws ALRepositoryException {
		return base.getNodes(node, type, page);
	}


	public GenericIterator<Node> getNodes(UserSession session, NodeType nodetype, boolean includeSub, Page page) throws ALRepositoryException {
		return base.getNodes(session, nodetype, includeSub, page);
	}

	public GenericIterator<Property> getProperties(Node node) throws ALRepositoryException {
		return base.getProperties(node);
	}

	public VersionHistory getVersionHistory(Node node) throws ALRepositoryException {
		return base.getVersionHistory(node);
	}

	public void move(Node src, Node destParent) throws ALRepositoryException {
		base.move(src, destParent) ;
	}

	public Value[] getValues(Node node, Property property) throws ALRepositoryException {
		return base.getValues(node, property) ;
	}

	
	public void indexNode(List<Event> pendingEvent, Node node) throws ALRepositoryException {
		base.indexNode(pendingEvent, node) ;
	}

	public void indexNode(List<Event> pendingEvent, Collection<Node> thisNodes, Map<String, Link> thisLinks) throws ALRepositoryException {
		base.indexNode(pendingEvent, thisNodes, thisLinks) ;
	}

	public GenericIterator<Node> searchNode(UserSession session, Node startNode, String searchText, Page page) throws SearchException, ALRepositoryException {
		return base.searchNode(session, startNode, searchText, page);
	}

	public Node getIternalShareNode(UserSession session, String workspaceName, ShareInfo shareInfo)  throws ALRepositoryException{
		return base.getIternalShareNode(session, workspaceName, shareInfo);
	}

	private void loadMemory(Node node){
		if (node.isRoot()) return ;
//		Debug.debug("LOAD [" + node.getMode() + "] " + node + " " + nodes.contains(node));
//		if (! node.getMode().equals(Mode.NORMAL)) return ;
		cacheNodes.put(node.getIdentifier(), node) ;
		
	}

	public void close() {
		base.close() ;
	}

	public GenericIterator<Node> getChildNodes(Node node, Page page) throws ALRepositoryException {
		return base.getChildNodes(node, page);
	}



}
