package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.List;

import com.bleujin.framework.db.DBController;
import com.bleujin.framework.db.NRows;
import com.bleujin.framework.db.Row;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.procedure.IUserCommand;
import com.bleujin.framework.db.procedure.UserProcedures;
import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.dimpl.carrier.MappingCarrier;
import com.bleujin.lore.core.dimpl.carrier.NamespaceCarrier;
import com.bleujin.lore.core.dimpl.carrier.NodeTypeCarrier;
import com.bleujin.lore.core.dimpl.carrier.ShareCarrier;
import com.bleujin.lore.core.dimpl.carrier.TemplateCarrier;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.NamespaceException;
import com.bleujin.lore.core.exception.NoSuchNodeTypeException;
import com.bleujin.lore.core.exception.ShareNodeException;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.mapping.ConstraintHelper;
import com.bleujin.lore.core.mapping.IDefaultValue;
import com.bleujin.lore.core.mapping.IValueConstraint;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeExistsException;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.persitence.NamespaceRegistry;
import com.bleujin.lore.core.persitence.Sequence;
import com.bleujin.lore.core.persitence.URI;
import com.bleujin.lore.extend.template.BindTemplate;
import com.bleujin.lore.extend.template.Template;
import com.bleujin.lore.extend.template.TemplateId;
import com.bleujin.lore.extend.template.TemplateImpl;

public class DatabaseGlobalStore extends AbstractGlobalStore{
	

	private NamespaceRegistry namespace ;
	private DBController dc ;
	private Sequence seq ;
	private DatabaseGlobalStore(DBController dc) {
		this.dc = dc ;
		this.seq = new CacheSequence(new DatabaseSequence(namespace, dc)) ;
		this.namespace = new NamespaceRegistry(this) ;
	}
	
	public final static DatabaseGlobalStore create(DBController dc){
		return new DatabaseGlobalStore(dc) ;
	}

	@Override
	
	
	public NewNodeType createNodeType(NodeType superType, String ntId, PropertyDefinition[] definitions) throws ALRepositoryException {
		return newNodeType(this, superType, ntId, definitions) ;
    }
	
	public GenericIterator<NodeType> register(NewNodeType[] newNodeTypes, boolean allowUpdate) throws ALRepositoryException {
		for (NewNodeType newNodeType : newNodeTypes) {
			if (allowUpdate == false && includeNodeType(newNodeType.getName())) throw NodeTypeExistsException.create(newNodeType.getName()) ;
		}
		
		if (allowUpdate) {
			getNodeTypeCarrier().updateType(newNodeTypes, allowUpdate) ;
		} else {
			getNodeTypeCarrier().createType(newNodeTypes, allowUpdate) ;
		}
		
		List<NodeType> result = new ArrayList<NodeType>() ;
		for (NewNodeType template : newNodeTypes) {
			result.add(NodeTypeImpl.create(this, template)) ;
		}
		
		return new GenericIteratorHelper<NodeType>(result) ;
	}
	
	@Override
	public GenericIterator<MappedDefinition> getMappedDefinition(NodeType nodetype) throws ALRepositoryException{
		
		NRows rows = getMappingCarrier().listBy(nodetype.getId()) ;
		
		List<MappedDefinition> result = new ArrayList<MappedDefinition>() ;
		while(rows.next()){
			PropertyDefinition pd = PropertyDefinitionImpl.create(IDString.propId(rows.getString("pdId")), Property.Type.fromName(rows.getString("typeCd"))) ;
			IValueConstraint constraint = ConstraintHelper.makeConstraint(rows.getString("constraints")) ;
			
			IDefaultValue defaultValue = ConstraintHelper.makeDefaultValue(rows.getString("defaultValue")) ;
			result.add(MappedDefinition.load(pd, constraint, defaultValue)) ;
		}
		
		return new GenericIteratorHelper<MappedDefinition>(result) ;		
	}
	
	
	@Override
	public boolean includeNodeType(String nodeTypeName) throws NamespaceException {
		return getNodeTypeCarrier().isIncludeType(toNodeTypeId(nodeTypeName)) ;
    }
	
	@Override
	public void setMemberType(NodeType nodetype, NodeType memberType) {
		getNodeTypeCarrier().setMemberType(nodetype.getId(), memberType.getId()) ;
	}
	
	public void setPrimaryProperty(NodeType nodetype, PropertyDefinition pd) throws ALRepositoryException{
		getNodeTypeCarrier().setPrimary(nodetype.getId(), pd) ;
	}
	
	@Override
	public GenericIterator<NodeType> getAllNodeTypes(String prefix) throws NamespaceException {
		NRows rows = getNodeTypeCarrier().listBy(prefix) ;
		List<NodeType> result = new ArrayList<NodeType>() ;
		while(rows.next()){
			NodeType type = NodeTypeImpl.load(this, rows.getString("superId"), rows.getString("memberId"), rows.getString("ntId"), rows.getString("primaryPropId")) ;
			result.add(type) ;
		}
		
		return new GenericIteratorHelper<NodeType>(result) ;
	}

	@Override
	public LinkType getLinkType(String name) {
		return LinkTypeImpl.load(name); 
    }

	@Deprecated
	private GenericIterator<LinkType> getAllLinkTypes() {
		throw new UnsupportedOperationException("getAllLinkType");
    }

	protected NodeType getNodeType(AbstractGlobalStore globalStore, NodeTypeId ntId) throws NamespaceException, NoSuchNodeTypeException {
		if (ntId == null || StringUtil.isBlank(ntId.getString()) || "*".equals(ntId.getString())) return NodeType.NO_SUPERTYPE ; // top
		
		NRows rows = getNodeTypeCarrier().getType(ntId) ;
		if (rows.next()){
			return NodeTypeImpl.load(globalStore, rows.getString("superId"), rows.getString("memberId"), rows.getString("ntId"), rows.getString("primaryPropId")) ;
		} else {
			throw NoSuchNodeTypeException.create(ntId.getString());
		}
	}


	@Override
	public void removeNodeType(String ntId) throws NamespaceException {
		getNodeTypeCarrier().removeType(toNodeTypeId(ntId)) ;
	    // TODO : 노드는 어떻게????
	    
    }

	
	@Override
	public void setPropertyConstraint(NodeType nodetype, PropertyDefinition prop, IValueConstraint constraint) throws ALRepositoryException {
		int result = getMappingCarrier().setConstraintWith(nodetype.getId(), prop.getId(), constraint.getConfigurationExpression()) ;
		if (result == 0) throw new ALRepositoryException("same constraint already exist at " + prop.getId()) ;
	}
	
	@Override
	public void setPropertyDefaultValue(NodeType nodeType, PropertyDefinition prop, IDefaultValue defaultValue) throws ALRepositoryException {
		getMappingCarrier().setDefaultValueWith(nodeType.getId(), prop.getId(), defaultValue.getConfigurationExpression()) ;
	}

	
	@Override
	public void clearStore() {
		getNodeTypeCarrier().clear() ;
		
	}
	
	@Override
	public Sequence getSequence() {
		return seq;
	}

	public NamespaceRegistry getNamespace(){
		return namespace ;
	}
	
	@Override
	public void confirm(){
		IUserCommand nt_cmd = dc.createUserCommand("select * from nodetype_tblc") ;
		IUserCommand pd_cmd = dc.createUserCommand("select x1.ntId, x2.* from nodetype_pdefinition_tblc x1, propertydefinition_tblc x2 where x1.pdId = x2.pdId");
		IUserCommand pd_count = dc.createUserCommand("select 'nodetype_pd' key, ntId, pdId from nodetype_pdefinition_tblc x1 union all select 'definition', '', pdId from propertydefinition_tblc");

		UserProcedures upts = new UserProcedures("confirm").add(nt_cmd).add(pd_cmd).add(pd_count);
    	
    	Rows rows = dc.getRows(upts) ;
    	NRows nodetype = new NRows(rows, new RuntimeException());
    	NRows pdefinition = new NRows(rows.getNextRows(), new RuntimeException());
    	NRows pdCount = new NRows(rows.getNextRows().getNextRows(), new RuntimeException());
    	
    	while(nodetype.next()){
    		Debug.debug(nodetype.getString("ntId") + "[" + nodetype.getString("superId") + "]");
    		pdefinition.beforeFirst() ;
        	while(pdefinition.next()){
        		if (nodetype.getString("ntId").equals(pdefinition.getString("ntId"))) {
        			Debug.debug("    <" + pdefinition.getString("pdId") + "> " + pdefinition.getString("typeCd"));
        		}
        	}
    	}
    	while (pdCount.next()) {
    		Debug.debug(pdCount.getString(1) + "  " + pdCount.getString(2) + "  " + pdCount.getString(3) + "  ");
		}
    	Debug.debug("-------------------------------------");
        
		
	}
	
	private MappingCarrier getMappingCarrier() {
		return new MappingCarrier(dc) ;
	}
	
	private NodeTypeCarrier getNodeTypeCarrier() {
		return new NodeTypeCarrier(dc) ;
	}

	private NamespaceCarrier getNamespaceCarrier() {
		return new NamespaceCarrier(dc) ;
	}

	
	
	@Override
	public ShareInfo getShareInfo(String shareName) throws ALRepositoryException {
		Row row = getShareCarrier().getNodeInfo(shareName);
		return ShareInfo.create(row.getString("shareName"), row.getString("workspaceName"), row.getString("nodePath"), "T".equals(row.getString("enableWrite")));
	}

	private ShareCarrier getShareCarrier() {
		return new ShareCarrier(dc);
	}

	@Override
	public void shareNode(String shareName, String workspaceName, String nodePath, boolean enableWrite) {
		int result = getShareCarrier().shareNode(shareName, workspaceName, nodePath, enableWrite) ;
		if (result == 0) throw new ShareNodeException("이미 있어요 : " + shareName) ;
	}


	private TemplateCarrier getTemplateCarrier(){
		return new TemplateCarrier(dc) ;
	}
	
	
	@Override
	public Template getBoundTemplate(Node node) throws ALRepositoryException{
		NRows rows = getTemplateCarrier().getTemplate(node.getIdentifier(), node.getNodeType().getId()) ;
		if (rows.first()){
			return BindTemplate.create(node, TemplateId.createId(rows.getString("tplId")), rows.getString("content")) ; 
		} else {
			return BindTemplate.createDefault(node) ;
		}
	}

	public GenericIterator<TemplateId> getTemplates(UserSession session)  throws ALRepositoryException {
		NRows rows = getTemplateCarrier().getTemplates() ;
		List<TemplateId> result = new ArrayList<TemplateId>() ;
		while(rows.next()){
			result.add(TemplateId.createId(rows.getString("tplId"))) ;	
		}
		return new GenericIteratorHelper<TemplateId>(result) ;
	}
	
	@Override
	public Template getTemplate(UserSession session, TemplateId tplId) throws ALRepositoryException {
		NRows rows = getTemplateCarrier().getTemplate(tplId) ;
		if (rows.first()){
			return TemplateImpl.load(session, tplId, rows.getString("content"));
		} else {
			throw new ALRepositoryException("not found template, tplId =" + tplId) ;
		}
	}

	@Override
	public Template saveTemplate(TemplateId tplId, Template template) throws ALRepositoryException {
		int result = getTemplateCarrier().create(tplId, template.getContent()) ;
		if (result == 0) throw new ALRepositoryException("already exist tplId : " + tplId) ;
		return template;
	}

	@Override
	public void bindTemplate(Node node, TemplateId tplId) throws ALRepositoryException {
		getTemplateCarrier().setTemplate(node.getIdentifier(), tplId) ;
	}

	@Override
	public void bindTemplate(NodeType nodeType, TemplateId tplId) throws ALRepositoryException {
		getTemplateCarrier().setTemplate(nodeType.getName(), tplId) ;
	}


	public void close() {
		dc._destroy() ;
	}

	@Override
	public GenericIterator<ShareInfo> getShareInfos() throws ALRepositoryException {
		List<ShareInfo> result = new ArrayList<ShareInfo>() ;
		NRows row = getShareCarrier().getNodeInfos();
		while(row.next()){
			result.add(ShareInfo.create(row.getString("shareName"), row.getString("workspaceName"), row.getString("nodePath"), "T".equals(row.getString("enableWrite"))));
		}
		return new GenericIteratorHelper<ShareInfo>(result) ;
	}

	
	
	protected  int addURIAddress(String prefix, String address) {
		return getNamespaceCarrier().createWith(prefix, address);
	}

	protected  void removeURIAddress(String prefix) {
		getNamespaceCarrier().removeWith(prefix);
	}

	protected  boolean existURIAddress(String prefix) {
		return getNamespaceCarrier().existBy(prefix) ;
	}

	protected  URI getURIAddress(String prefix) throws NamespaceException {
		NRows rows = getNamespaceCarrier().retrieveBy(prefix) ;
    	if (rows.first()){
    		return new URI(prefix, rows.getString("address")) ;
    	} else {
    		throw new NamespaceException("NOT FOUND : " + prefix);
    	}
	}

	protected  GenericIterator<URI> listURIAddress() {
		List<URI> result = new ArrayList<URI>() ;
		NRows rows = getNamespaceCarrier().listBy() ;
		while(rows.next()){
			result.add(new URI(rows.getString("prefix"), rows.getString("address"))) ;
		}
		return new GenericIteratorHelper<URI>(result);
	}

}
