package com.bleujin.lore.core.dimpl;

import java.util.List;

import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.observation.LinkEvent;
import com.bleujin.lore.core.observation.NodeEvent;
import com.bleujin.lore.core.observation.PropertyEvent;
import com.bleujin.lore.core.observation.Event.EventType;

public abstract class EventProcessor {

	public abstract void process(List<Event> pendingEvent) throws ALRepositoryException ; 

	public abstract void process(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException ;

	

	protected void handleEvent(List<Event> pendingEvent) throws ALRepositoryException {
		for (Event event : pendingEvent) {
			if (EventType.NODE_ADDED.equals(event.getType())) {
				createNodeEvent(event, ((NodeEvent) event).getNode());
			} else if (EventType.NODE_UPDATED.equals(event.getType())) {
				updateNodeEvent(event, ((NodeEvent) event).getNode());
			} else if (EventType.NODE_REMOVED.equals(event.getType())) {
				delNodeEvent(event, (Node)((NodeEvent) event).getNode());
			} else if (EventType.NODE_MOVED.equals(event.getType())) {
				moveNodeEvent(event, (Node)((NodeEvent) event).getNode());
			} else if (EventType.NODE_CHECKOUTED.equals(event.getType())) {
				checkOutEvent(event, (NodeVersion)((NodeEvent) event).getNode()) ;
			}

			if (EventType.LINK_ADDED.equals(event.getType())) {
				createLinkEvent(event, ((LinkEvent) event).getLink());
			} else if (EventType.LINK_REMOVED.equals(event.getType())) {
				delLinkEvent(event, ((LinkEvent) event).getLink());
			}
			
			
			if (EventType.PROPERTY_ADDED.equals(event.getType())) {
				createPropertyEvent(event, ((PropertyEvent) event).getProperty());
			} else if (EventType.PROPERTY_CHANGED.equals(event.getType())) {
				changePropertyEvent(event, ((PropertyEvent) event).getProperty());
			}
		}
	}

	protected abstract void createPropertyEvent(Event event, Property property) throws ALRepositoryException ;

	protected abstract void updateNodeEvent(Event event, INode node) throws ALRepositoryException ;

	protected abstract void changePropertyEvent(Event event, Property property) throws ALRepositoryException ;

	protected abstract void delLinkEvent(Event event, Link link) throws ALRepositoryException ;

	protected abstract void createLinkEvent(Event event, Link link) throws ALRepositoryException ;

	protected abstract void checkOutEvent(Event event, NodeVersion node) throws ALRepositoryException ;

	protected abstract void moveNodeEvent(Event event, Node node) throws ALRepositoryException ;

	protected abstract void delNodeEvent(Event event, Node node) throws ALRepositoryException ;

	protected abstract void createNodeEvent(Event event, INode node) throws ALRepositoryException ;

}
