package com.bleujin.lore.core.dimpl;

import org.apache.ecs.xml.XML;

import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.observation.Event.Mode;

public class LinkImpl implements Link{
 
	private static final long serialVersionUID = -8026988106473858685L;
	private transient UserSession session ;
	private final String fromNodeId;
	private final String toNodeId;
	private final LinkType type;
	private Mode mode ;
	private Position position ;

	
	private transient Node fromNode = null;
	private transient Node toNode = null ;
	private LinkImpl(UserSession session, String fromNodeId, String toNodeId, LinkType type, Mode mode, Position position) {
		this.session = session ;
		this.fromNodeId = fromNodeId ; 
		this.toNodeId = toNodeId;
		this.type = type;
		this.mode = mode ;
		this.position = position ;
    }

	private LinkImpl(UserSession session, Node fromNode, Node toNode, LinkType type, Mode mode, Position position) {
		this(session, fromNode.getIdentifier(), toNode.getIdentifier(), type, mode, position) ;
		this.fromNode = fromNode ;
		this.toNode = toNode ;
	}
	
	
	public static LinkImpl create(UserSession session, Node fromNode, Node toNode, LinkType type, Position position) {
		return new LinkImpl(session, fromNode, toNode, type, Mode.CREATED, position) ;
	}

	public static LinkImpl load(UserSession session, String fromNodeId, String toNodeId, LinkType type) {
		return new LinkImpl(session, fromNodeId, toNodeId, type, Mode.NORMAL, Position.PENDENCY) ;
	}

	public void setMode(Mode mode){
		this.mode = mode ;
	}
	
	public Mode getMode(){
		return this.mode ;
	}
	

	// 다소 억지스럽지만.. link가 create 됐을때 LocalStore에서는 from 정보를 가져와서는 안된다.
	public Node getFrom() throws ALRepositoryException {
		return (fromNode == null) ? session.getNode(fromNodeId) : fromNode ;
	}

	public LinkType getType() {
		return this.type;
	}

	public Node getTo() throws ALRepositoryException {
		return (fromNode == null) ? session.getNode(toNodeId) : toNode;
    }
	
	public Position getPosition() {
		return position ;
	}
	
	public boolean equals(Object obj){
		if (obj == null || !(obj instanceof Link)) return false ;
		
		Link that = (Link)obj ;
		return	this.fromNodeId.equals(that.getFromId()) 
			&&  this.type.equals(that.getType()) ;
	}
	
	public int hashCode(){
		return getFromId().hashCode() + getToId().hashCode() ;
	}
	
	public String toString(){
        return "from:" + getFromId() + " to:" + getToId() + " - " + type ;
	}
	
	public XML toXML(){
		XML result = new XML(LINK) ;
		result.addElement(new XML(FROM).addElement(fromNodeId)) ;
		result.addElement(new XML(LINK_TYPE).addElement(type.getString())) ;
		result.addElement(new XML(TO).addElement(toNodeId)) ;

		return result ;
	}

	public String getFromId() {
	    return fromNodeId;
    }

	public String getToId() {
	    return toNodeId;
    }
	
	public Link newClone(UserSession session){
		return new LinkImpl(session, this.fromNodeId, this.toNodeId, this.type, this.mode, this.position) ;
	}
}
