package com.bleujin.lore.core.dimpl;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.io.IOUtils;

import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.lock.DefaultReadsWriteLock;
import com.bleujin.lore.core.common.lock.ReadsWriteLock;
import com.bleujin.lore.core.node.Node;

public class LocalFileStore {

	private File file ;
	private ReadsWriteLock rwlock ;
	LocalFileStore(String fileName){
		this.file = new File(fileName) ;
		this.rwlock = new DefaultReadsWriteLock() ;
	}
	
	static LocalFileStore create(String fileName) throws IOException{
		File file = new File(fileName);
		if (! file.getParentFile().exists()) file.getParentFile().mkdirs() ;
		if (! file.exists()) file.createNewFile() ;
		
		return new LocalFileStore(fileName) ;
	}
	
	public void clear() throws InterruptedException, IOException{
		try {
			rwlock.writeLock() ;
			file.delete() ;
			file.createNewFile() ;
		} finally {
			rwlock.writeUnLock() ;
		}
	}
	
	public Collection<Node> readNode() throws InterruptedException, IOException, ClassNotFoundException{
		try {
			rwlock.readLock() ;
			if (file.length() == 0) return new ArrayList<Node>() ;
			
			FileInputStream fi = new FileInputStream(file) ;
			ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(IOUtils.toByteArray(fi))) ; 
			Collection<Node> result = (Collection<Node>) in.readObject();
			in.close() ;
			fi.close() ;
			// Debug.debug("readNode", this);
			return result ;
		} finally {
			rwlock.readUnLock() ;
		}
	}
	
	public void writeNode(Collection<Node> nodes) throws InterruptedException, IOException{
		try {
			rwlock.writeLock() ;
			
			FileOutputStream fo = new FileOutputStream(file);
			ObjectOutputStream out = new ObjectOutputStream(fo);

			out.writeObject(nodes);
			out.flush();
			out.close();
			fo.close() ;
			Debug.happy(this, "writeNode");
		} finally {
			rwlock.writeUnLock() ;
		}
	}

	public void close() {
		Debug.happy(this, "Call Close") ;
	}
}
