package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang.math.RandomUtils;
import org.apache.ecs.xml.XML;

import com.bleujin.lore.core.bean.Rating;
import com.bleujin.lore.core.bean.Register;
import com.bleujin.lore.core.bean.SourceFrom;
import com.bleujin.lore.core.exception.IllegalNodeNameException;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.persitence.NodeBean;

public class NodeBeanImpl implements NodeBean {
 
	private static final long serialVersionUID = 4104888040423773058L;
	
	private NodeTypeId selfTypeId ;
	private NodeName name;
	private String explain ;
	private String status ;
	private String uuid;
	private long viewCount;
	private Register register ;

	private SourceFrom from  = SourceFrom.EMPTY;
	private List<Rating> ratings = new ArrayList<Rating>();
	private String workspaceName ;
	
	private NodeBeanImpl(NodeTypeId selfTypeId, NodeName name, String explain, String uuid, Register register, long viewCount, String workspaceName){
		this.selfTypeId = selfTypeId ;
		this.name = name ;
		this.explain = explain ;
		this.uuid = uuid ;
		this.register = register ;
		this.viewCount = viewCount ;
		this.workspaceName = workspaceName ;
	}

	static NodeBeanImpl create(NodeType selfType, NodeName name, String explain, Register regInfo, String workspaceName) {
		String newUUID = new UUID(name.hashCode() + RandomUtils.nextLong(), new Date().getTime()).toString();
		return new NodeBeanImpl(selfType.getId(), name, explain, newUUID, regInfo, 0L, workspaceName) ;
	}
	
	static NodeBeanImpl load(RowToNodeBean row) throws IllegalNodeNameException{
		return new NodeBeanImpl(row.getNodeTypeId(), NodeName.nodeName(row.getName()), row.getExplain(), row.getUUID(), row.getRegister(), row.getViewCount(), row.getWorkspaceName()) ;
	}
	
	
	public NodeBean newClone(){
		return new NodeBeanImpl(this.selfTypeId, this.name, this.explain, this.uuid, this.register, this.viewCount, this.workspaceName) ;
	}
	
	
	public void setName(NodeName name) {
		this.name = name ;
	}

	public NodeName getName() {
		return this.name;
	}
	
	public void setExplain(String explain){
		this.explain = explain ;
	}
	
	public String getExplain() {
		return this.explain ;
	}

	public String getIdentifier() {
		return uuid;
	}

	public long getViewCount() {
		return viewCount;
	}

	public void setRegister(Register register){
		this.register = register ;
	}

	public Register getRegister(){
		return register ;
	}
	
	public void setSourceFrom(SourceFrom from){
		this.from = from ;
	}
	
	public SourceFrom getSourceFrom(){
		return this.from ;
	}
	
	public void addRating(Rating rating){
		ratings.add(rating) ;
	}
	
	public XML toXML(){
		XML result = new XML(XML_TAG_ROOT) ;
		result.addAttribute(XML_TAG_NAME, getName().getString()) ;
		result.addAttribute(XML_TAG_EXPLAIN, getExplain() == null ? "" : getExplain()) ;
		result.addAttribute(XML_TAG_UUID, getIdentifier()) ;
		result.addAttribute(XML_TAG_REGISTER, getRegister().toString()) ;
		result.addAttribute(XML_TAG_VISIT, String.valueOf(getViewCount())) ;

		return result ;
	}

	public NodeTypeId getNodeTypeId() {
	    return selfTypeId;
    }

	public void increaseViewCount() {
		this.viewCount++;
    }

	public String getStatus() {
    	return status;
    }

	public void setStatus(String status) {
    	this.status = status;
    }

	public String getWorkspaceName() {
		return this.workspaceName;
	}

	public String toString(){
		return uuid ;
	}

}
