package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.bleujin.lore.core.bean.Register;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IConstant;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.IllegalNodeNameException;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.observation.Event.Mode;
import com.bleujin.lore.core.persitence.NodeBean;

public class NodeDBImpl extends NodeImpl implements IConstant {

    private static final long serialVersionUID = -7513782534350067084L;


	public final static int UNKNOWN_VERSION = -1 ;
	private NodeDBImpl(UserSession session, String parentId, NodeBean bean, int versionNum, Mode mode){
		super(session, mode, parentId, bean, versionNum, false);
	}

	static INode create(UserSession session, Node parent, NodeType ntype, NodeName name, String explain) {
		Register regInfo = new Register(session.getUserId()) ;
		String workspaceName = session.getWorkspace().getName() ;
		return new NodeDBImpl(session, parent.getIdentifier(), NodeBeanImpl.create(ntype, name, explain, regInfo, workspaceName), INIT_VERSION, Mode.CREATED);
	}
	
	static NodeDBImpl load(UserSession session, RowToNodeBean row) throws IllegalNodeNameException {
		return new NodeDBImpl(session, row.getParentNodeId(), NodeBeanImpl.load(row), INIT_VERSION, Mode.NORMAL);
	}
	
	public Node newClone(UserSession session) {
		NodeDBImpl node = new NodeDBImpl(session,getParentId(), getBean().newClone(), getVersionNum(), getMode()) ;
		return node;
	}
	
	public GenericIterator<Node> getDescentNode(int maxDepth) throws ALRepositoryException {
		// TODO ... 바꿔야 함. 
		
		List<Node> result = new ArrayList<Node>();
		retrieveChildNodeTo(this, result, maxDepth);

		return new GenericIteratorHelper<Node>(Collections.unmodifiableCollection(result));
	}

	
	private void retrieveChildNodeTo(Node parent, List target, int maxDepth) throws ALRepositoryException {
		if (maxDepth <= 0)
			return;
		
		GenericIterator<Node> iter = parent.getChildNode(Page.ALL);
		
		for (Node child : iter) {
			if (child == null) continue ; // not saved node
			if (child.getMode().isDeleted()) continue ;
			
			target.add(child);
			retrieveChildNodeTo(child, target, --maxDepth);
		}
	}

	


}
