package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ALRepositoryRuntimeException;
import com.bleujin.lore.core.exception.AccessDeniedException;
import com.bleujin.lore.core.exception.NoSuchPropertyDefinitionException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.observation.Event.EventType;
import com.bleujin.lore.core.observation.Event.Mode;
import com.bleujin.lore.core.persitence.ILocalStore;
import com.bleujin.lore.core.persitence.NodeBean;
import com.bleujin.lore.core.value.Value;

public abstract class NodeImpl extends AbstractNode {

	private static final long serialVersionUID = -5643550797930663007L;

	private Mode mode;
	private transient UserSession session;
	private String parentId;
	private NodeBean bean;
	private int versionNum;
	private List<Link> links;
	private Map<IDString, Property> properties;
	private boolean isLoadProperty = false;

	protected NodeImpl(UserSession session, Mode mode, String parentId, NodeBean bean, int versionNum, boolean isLoadProperty) {
		this.session = session;
		this.mode = mode;
		this.parentId = parentId;
		this.bean = bean;
		this.versionNum = versionNum;
		this.properties = new HashMap<IDString, Property>();
		this.links = new ArrayList<Link>();
		this.isLoadProperty = isLoadProperty;
	}

	public Node newClone(UserSession session) {
		this.session = session;
		setMode(Mode.NORMAL);
		return this;
	}
	
	
	// use clone
	protected String getParentId(){
		return this.parentId ;
	}
	
	// use clone
	
	protected void copyLink(NodeImpl target){
		target.links = this.links ; // link is not versionable
	}
	
	protected void copyProperty(NodeImpl target){
		Map<IDString, Property> cloneProperties = new HashMap<IDString, Property>() ;
		for (Property property : this.properties.values()) {
			PropertyDefinition pd = PropertyDefinitionImpl.create(property.getId().getString(), property.getType()) ;
			cloneProperties.put(property.getId(), PropertyImpl.load(target, pd, property.getValue().newClone())) ;
		}
		target.properties = cloneProperties ;
	}
	
	protected void setVersionNum(int versionNum){
		this.versionNum = versionNum ;
	}
	

	public NodeVersion checkIn() throws ALRepositoryException {
		if (getMode().isCreated() || getMode().isUpdated() || getMode().isDeleted())
			throw new AccessDeniedException("exception.version.checkin[not_saved]");
		// versionNum++ ; //
		return VersionImpl.checkIn(this);
	}

	public GenericIterator<Link> getLinks(Page page) throws ALRepositoryException {
		return getLocalStore().getLinks(this, page);
	}

	public GenericIterator<Link> getLinks(LinkType linkType, Page page) throws ALRepositoryException {
		return getLocalStore().getLinks(this, linkType, page);
	}

	public GenericIterator<Node> getChildNode(Page page) throws ALRepositoryException {
		return getLocalStore().getChildNodes(this, page);
	}
	
	public GenericIterator<Node> getNodes(LinkType type, Page page) throws ALRepositoryException {
		return getLocalStore().getNodes(this, type, page);
	}

	public void copy(Node parent) throws ALRepositoryException {
		if (this.getParent().isSame(parent)) {
			throw new ALRepositoryException("exception.node.copy[invalid_location]");
		}

		INode newNode = getSession().createNode(parent, getNodeType(), getName());

		GenericIterator<Property> properties = this.getProperties();
		while (properties.hasNext()) {
			Property property = properties.next().newClone();
			newNode.setProperty(property.getString(), property.getValue());
		}
		// 현재 deep 카피는 패스.
//		GenericIterator<Node> childs = getChildNode();
//		for (Node child : childs) {
//			child.copy(newNode.save());
//		}
	}

	public void move(Node newParent) throws ALRepositoryException {
		// Node dest = parent.getParent() ;
		Node dest = newParent;
		while (!dest.isRoot()) {
			if (dest.isSame(this)) {
				throw new ALRepositoryException("exception.node.move[invalid_location]");
			}
			dest = dest.getParent();
		}
		// ..
		// TODO remove before parent link...
		this.parentId = newParent.getIdentifier();

		setMode(Mode.MOVED);
		getSession().fireEvent(this, EventType.NODE_MOVED);

	}

	public void setDefaultProperty() throws ALRepositoryException {
		Map<IDString, Property> defaults = new HashMap<IDString, Property>();
		GenericIterator<MappedDefinition> pds = getNodeType().getMappedDefinitions();
		for (MappedDefinition pd : pds) {
			Property property = PropertyImpl.load(this, pd, pd.getDefaultValue());
			defaults.put(pd.getId(), property);
			getSession().fireEvent(property, EventType.PROPERTY_ADDED);
		}
		this.properties = defaults;

	}

	public Node getParent() throws ALRepositoryException {
		if (parentId == null)
			return getSession().getRootNode();
		return getSession().getNode(parentId);
	}

	public UserSession getSession() {
		if (session == null) throw new ALRepositoryRuntimeException("exception.session.null");
		return this.session;
	}

	public NodeBean getBean() {
		return bean;
	}

	protected ILocalStore getLocalStore() {
		return getSession().getLocalStore();
	}

	public void setMode(Mode mode) {
		this.mode = mode;
	}

	public Mode getMode() {
		return mode;
	}

	public void setParent(Node parent) {
		this.parentId = parent.getIdentifier();
	}

	public boolean isRoot() {
		// because of shareNode(Session of other workspace login)
		return StringUtil.isAlphaUnderbarStartWith(getIdentifier()) && (getIdentifier().indexOf("/") == -1) && (getIdentifier().indexOf("-") == -1);

		// return getUUIDString().equals(session.getWorkspace().getName());
	}

	public int getVersionNum() {
		return versionNum;
	}

	public void addLink(Node target, LinkType linkType, Position position) throws ALRepositoryException {
		Link link = getSession().createLink(this, target, linkType, position);
		if (Position.AFTER.equals(position)) {
			links.add(link);
		} else {
			links.add(0, link);
		}
	}

	public GenericIterator<Property> getProperties() throws ALRepositoryException {
		loadReal();
		return new GenericIteratorHelper<Property>(properties.values());
	}

	public Property getProperty(IDString propId) throws ALRepositoryException {
		loadReal();
		return properties.get(propId);
	}

	public void setProperty(IDString propId, Value value) throws ALRepositoryException {
		loadReal();
		// check property
		if (!getNodeType().containsPropId(propId.getString()))
			throw NoSuchPropertyDefinitionException.create(propId);
		// not null...
		assert properties.get(propId) == null : "not setted default value...";
		properties.get(propId).setValue(value);
		// if requiredType is refernce or path, add Link
		createRefernceLink(getProperty(propId).getType(), value);

		getSession().loadMemory(this);
	}

	private void loadReal() throws ALRepositoryException {
		if ((!getMode().isCreated()) && (!this.isLoadProperty)) {
			GenericIterator<Property> ps = getLocalStore().getProperties(this);
			for (Property p : ps) {
				this.properties.put(p.getId(), p);
			}
			this.isLoadProperty = true;
		}
	}

//	// TransientLocalStore.save 때문에 어쩔수 없이 만듬
//
//	// internal path...
	List<Link> getIternalLinks() throws ALRepositoryException {
		List<Link> oldLink = new ArrayList<Link>();
		for (Link link : links) {
			// if (getSession().getNode(link.getToId()).getMode().isCreated()) {
			// Debug.debug(getSession().getNode(link.getToId()));
			// continue ;
			// }
			oldLink.add(link);
		}
		return oldLink;
	}
	
	List<Link> getTestLink(){
		return links ;
	}
//
//	void removeChild(Node child) throws ALRepositoryException {
//		for (Link link : links) {
//			if (LinkTypeImpl.CHILD.equals(link.getType()) && link.getToId().equals(child.getIdentifier())) {
//				links.remove(link);
//				return;
//			}
//		}
//
//	}

}
