package com.bleujin.lore.core.dimpl;

import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.NamespaceException;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.persitence.NamespaceRegistry;
import com.bleujin.lore.core.persitence.URI;

public class NodeTypeIdImpl implements NodeTypeId{

	private static final long serialVersionUID = -4040499305997614059L;
	public final static String SEPERATOR = ":" ;
	private final static String DEFAULT_PREFIX = "default";

	private URI uri ;
	private IDString shortId ;
	private NodeTypeIdImpl(URI uri, IDString shortId){
		this.uri = uri ;
		this.shortId = shortId ;
	}
	
	
	public final static NodeTypeId create(NamespaceRegistry namespace, String ntId) throws NamespaceException{
		if (ntId == null || "*".equals(ntId)) return NodeTypeIdImpl.NO_SUPERTYPE_ID;
		if (StringUtil.isBlank(ntId)) throw new IllegalArgumentException("NodeTypeId must be not blank") ;

		if (StringUtil.contains(ntId, SEPERATOR)) {
			String prefix = StringUtil.substringBefore(ntId, SEPERATOR) ;
			String shortId = StringUtil.substringAfter(ntId, SEPERATOR) ;
			
			URI myURI = namespace.getURI(prefix);
			if (myURI == null) throw NamespaceException.notFoundPrefix(prefix) ;
			return  new NodeTypeIdImpl(myURI, IDString.shortId(shortId)) ;
		} else {
			return new NodeTypeIdImpl(namespace.getURI(DEFAULT_PREFIX), IDString.shortId(ntId)) ;
		}
	}

	public boolean isRootType(){
		return "*".equals(getShortId()) || StringUtil.isBlank(getShortId()) ;
	}

	public boolean equals(Object obj){
		if (! (obj instanceof NodeTypeId)) return false ;
		return getString().equals(((NodeTypeId)obj).getString()) ;
	}
	
	
	public int hashCode(){
		return getString().hashCode() ;
	}

	public String getString() {
		return uri.getPrefix() + SEPERATOR + shortId.getString();
	}


	public String getPrefix() {
		return uri.getPrefix();
	}


	public String getShortId() {
		return shortId.getString();
	}
	
	public String toString(){
		return getShortId() ;
	}
	
}
