package com.bleujin.lore.core.dimpl;

import java.util.List;

import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.NamespaceException;
import com.bleujin.lore.core.exception.NoSuchPropertyDefinitionException;
import com.bleujin.lore.core.nodetype.AbstractNodeType;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.persitence.GlobalStore;

public class NodeTypeImpl extends AbstractNodeType{

	private static final long serialVersionUID = 3752931498017515781L;
	public final static NodeType NO_SUPERTYPE = null;

	private transient GlobalStore gstore ;
	private NodeTypeId selfTypeId ;

	private String superTypeName ;
	private String memberTypeName ;
	private List<MappedDefinition> pdefinitions ;
	private String primaryPropertyId = null ;
	
	private NodeTypeImpl(GlobalStore gstore, String superTypeName, String memberTypeName, NodeTypeId selfTypeId, String primaryProperty) throws NamespaceException{
		this.gstore = gstore ;
		this.superTypeName = superTypeName ;
		this.memberTypeName = memberTypeName ;
		this.selfTypeId = selfTypeId ;
		this.pdefinitions = null ;
		this.primaryPropertyId = primaryProperty ; 
	}

	static NodeTypeImpl create(GlobalStore globalStore, NodeTypeId supertypeId, NodeTypeId selfTypeId) throws NamespaceException {
		return new NodeTypeImpl(globalStore, supertypeId.getString(), NodeTypeIdImpl.NO_SUPERTYPE_ID.getString(), selfTypeId, null) ;
	}

	static NodeType load(AbstractGlobalStore globalStore, String superName, String memberName, String ntId, String primaryPropId) throws NamespaceException {
		NodeTypeId selfTypeId = globalStore.toNodeTypeId(ntId) ;

		return new NodeTypeImpl(globalStore, superName, memberName, selfTypeId, primaryPropId) ;
	}
	
	static NodeType create(GlobalStore globalStore, NewNodeType newType) throws NamespaceException, ALRepositoryException{
		NodeTypeImpl nodetype = new NodeTypeImpl(globalStore, newType.getSuperType().getName(), newType.getMemberTypeId().getString(), newType.getId(), newType.getPrimaryPropertyId()) ;
		nodetype.pdefinitions = newType.getMappedDefinitions().toList() ;
		
		return nodetype;
	}
	
	
	public static void load(GlobalStore gstore, NodeTypeImpl nodetype){
		nodetype.gstore = gstore ;
	}


	public NodeTypeId getId() {
		return selfTypeId ; 
	}

	public NodeType getSuperType() throws ALRepositoryException {
		return getGlobalStore().getNodeType(superTypeName) ;
	}

	public void setMemberType(NodeType memberType) throws ALRepositoryException{
		this.memberTypeName = memberType.getName() ;
		getGlobalStore().setMemberType(this, memberType) ;
	}

	public NodeTypeId getMemberTypeId() throws ALRepositoryException{
		return getGlobalStore().toNodeTypeId(memberTypeName) ;
	}

	
	public GenericIterator<MappedDefinition> getMappedDefinitions() throws ALRepositoryException {
		loadCache() ;
		return new GenericIteratorHelper<MappedDefinition>(this.pdefinitions) ;
	}

	void setMappedDefinition(List<MappedDefinition> pdefinitions){
		this.pdefinitions = pdefinitions ;
	}
	
	public MappedDefinition getMappedDefinition(IDString propId) throws ALRepositoryException {
		loadCache() ;
		
		for(MappedDefinition pd : pdefinitions){
			if (pd.getId().equals(propId)) {
				return pd ;
			}
		}
		throw NoSuchPropertyDefinitionException.create(propId) ;
	}

	private synchronized void loadCache() throws ALRepositoryException{
		if (pdefinitions == null) {
			this.pdefinitions = gstore.getMappedDefinition(this).toList() ;
		}
	}
	

	public boolean containsPropId(IDString propId) throws ALRepositoryException {
		loadCache() ;
		for(PropertyDefinition pd : pdefinitions){
			if (pd.getId().equals(propId)) {
				return true ;
			}
		}		
		return false;
	}

	
	public void setPrimary(PropertyDefinition pd) throws ALRepositoryException {
		if (containsPropId(pd.getName())){
			this.primaryPropertyId = pd.getName() ;
			getGlobalStore().setPrimaryProperty(this, pd) ;
		} else {
			throw NoSuchPropertyDefinitionException.create(pd.getId()) ;
		}
	}
	
	@Deprecated
	public PropertyDefinition getPrimaryProperty() throws ALRepositoryException {
		if (StringUtil.isBlank(primaryPropertyId) || ! containsPropId(primaryPropertyId)) 
			throw NoSuchPropertyDefinitionException.create(primaryPropertyId) ;
		return getMappedDefinition(getPrimaryPropertyId()) ;
	}

	public String getPrimaryPropertyId() {
		return primaryPropertyId;
	}
	
	@Override
	public GlobalStore getGlobalStore() {
		return gstore;
	}

	public String toString() {
		return getClass().getName() + "["  + getName() + ", member:" + memberTypeName + "]";
	}


	public NewNodeType updateFor() throws ALRepositoryException {
		return NodeTypeTemplateImpl.updateFor(this);
	}


}
