package com.bleujin.lore.core.dimpl;

import java.util.HashMap;
import java.util.Map;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.NoSuchPropertyDefinitionException;
import com.bleujin.lore.core.mapping.IDefaultValue;
import com.bleujin.lore.core.mapping.IValueConstraint;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class NodeTypeTemplateImpl implements NewNodeType {

	private NodeType superType ;
	private NodeTypeId selfId ;
	private String primaryPropertyId = null;
	private NodeTypeId memberTypeId = NodeTypeId.NO_SUPERTYPE_ID ;
	private Map<IDString, MappedDefinition> mds = new HashMap<IDString, MappedDefinition>() ;
	
	private NodeTypeTemplateImpl(NodeType superType, NodeTypeId selfId) {
		this.superType = superType ;
		this.selfId = selfId ;
	}
	
	static NewNodeType create(NodeType superType, NodeTypeId selfId) throws ALRepositoryException{
		return create(superType, selfId, new PropertyDefinition[0]) ;
	}

	static NewNodeType create(NodeType superType, NodeTypeId selfId, PropertyDefinition[] newPds) throws ALRepositoryException{
		NewNodeType newType = new NodeTypeTemplateImpl(superType, selfId) ;
		newType.addPropertyDefinition(superType.getMappedDefinitions().toList().toArray(new PropertyDefinition[0])) ;
		newType.addPropertyDefinition(newPds) ;
		return newType ;
	}

	public void addPropertyDefinition(String propName, Property.Type requireType){
		addPropertyDefinition(new PropertyDefinition[]{PropertyDefinitionImpl.create(propName, requireType)}); 
	}

	public void addPropertyDefinition(String propName, Property.Type requireType, IValueConstraint constraint, IDefaultValue defaultValue) throws NoSuchPropertyDefinitionException{
		addPropertyDefinition(new PropertyDefinition[]{PropertyDefinitionImpl.create(propName, requireType)});
		getDefinition(propName).setConstraint(constraint) ;
		getDefinition(propName).setDefaultValue(defaultValue) ;
	}

	
	public void addPropertyDefinition(PropertyDefinition[] pds) {
		for (PropertyDefinition pd : pds) {
			mds.put(pd.getId(), MappedDefinition.create(pd)) ;
		}
	}

	public void setConstraint(PropertyDefinition prop, IValueConstraint constraint) throws NoSuchPropertyDefinitionException {
		getDefinition(prop.getName()).setConstraint(constraint) ;
	}

	public void setConstraint(String propName, IValueConstraint constraint) throws NoSuchPropertyDefinitionException {
		getDefinition(propName).setConstraint(constraint) ;
	}

	public void setDefaultValue(PropertyDefinition prop, IDefaultValue defaultValue) throws NoSuchPropertyDefinitionException {
		getDefinition(prop.getId()).setDefaultValue(defaultValue) ;
	}

	public void setDefaultValue(String propName, IDefaultValue defaultValue) throws NoSuchPropertyDefinitionException {
		getDefinition(propName).setDefaultValue(defaultValue) ;
	}

	public void setMemberType(NodeType memberType) {
		this.memberTypeId = memberType.getId() ;
	}
	
	public void setSelfMemberType(){
		this.memberTypeId = selfId ;
	}

	public void setPrimary(PropertyDefinition primary) throws NoSuchPropertyDefinitionException {
		this.primaryPropertyId = primary.getId().getString() ;
	}

	public boolean containsPropId(String propName) throws ALRepositoryException {
		return mds.containsKey(propName);
	}

	public MappedDefinition getMappedDefinition(IDString propId) throws NoSuchPropertyDefinitionException  {
		return getDefinition(propId);
	}

	public MappedDefinition getMappedDefinition(String propName) throws NoSuchPropertyDefinitionException  {
		return getDefinition(propName);
	}

	public GenericIterator<MappedDefinition> getMappedDefinitions() {
		return new GenericIteratorHelper<MappedDefinition>(mds.values());
	}

	public NodeTypeId getMemberTypeId() throws ALRepositoryException {
		return memberTypeId;
	}

	public NodeTypeId getId(){
		return selfId ;
	}
	
	public String getName() {
		return selfId.getString();
	}

	public String getPrimaryPropertyId() {
		return primaryPropertyId;
	}

	public NodeType getSuperType() throws ALRepositoryException {
		return superType;
	}
	
	private MappedDefinition getDefinition(String propName) throws NoSuchPropertyDefinitionException{
		return getDefinition(IDString.propId(propName)) ;
	}
	
	private MappedDefinition getDefinition(IDString propId) throws NoSuchPropertyDefinitionException{
		if (mds.containsKey(propId)) {
			return mds.get(propId) ;
		} else {
			throw NoSuchPropertyDefinitionException.create(propId) ;
		}
	}

	public void setPrimary(String propName) throws NoSuchPropertyDefinitionException {
		setPrimary(getDefinition(propName)) ;
	}
	
	public static NewNodeType updateFor(NodeType nodeType) throws ALRepositoryException {
		NodeTypeTemplateImpl updateTemplate = new NodeTypeTemplateImpl(nodeType.getSuperType(), nodeType.getId()) ;
		for (MappedDefinition md: nodeType.getMappedDefinitions()) {
			updateTemplate.mds.put(md.getId(), md) ;
		}
		updateTemplate.memberTypeId = nodeType.getMemberTypeId() ;
		updateTemplate.primaryPropertyId = nodeType.getPrimaryPropertyId() ;
		return updateTemplate;
	}
	
}
