package com.bleujin.lore.core.dimpl;

import java.util.HashMap;
import java.util.Map;

import org.apache.ecs.xml.XML;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.ItemVisitor;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.node.BinaryPropertyValue;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.observation.Event.EventType;
import com.bleujin.lore.core.value.ArrayValue;
import com.bleujin.lore.core.value.Value;

public class PropertyImpl implements Property {

	private static final long serialVersionUID = 676021261935160999L;
	private Node node;
	private PropertyDefinition propertyDefinition;
	private Value value;
	private boolean hasArrayValue = false ;

	private PropertyImpl(Node node, PropertyDefinition propertyDefinition, boolean hasArray, Value value) {
		this.node = node;
		this.propertyDefinition = propertyDefinition;
		this.hasArrayValue = hasArray ;
		this.value = (value == null) ? Value.EMPTY : value;
	}
	
	static Property load(Node node, String pdIdString, Property.Type requiredType, boolean hasArray, String valueString) throws ALRepositoryException {
		IDString propId = IDString.propId(pdIdString) ;
		// Property.Type requiredType = Property.Type.fromName(typeCdString);
		Value value = toValue(node, requiredType, valueString) ;
		return new PropertyImpl(node, PropertyDefinitionImpl.create(propId, requiredType), hasArray, value) ;
	}

	static Property load(Node node, PropertyDefinition propertyDefinition, Value value) {
		return new PropertyImpl(node, propertyDefinition, (value instanceof ArrayValue)  , value) ;
	}

	static Property create(Node node, PropertyDefinition propertyDefinition, Value value) {
		return new PropertyImpl(node, propertyDefinition, value instanceof ArrayValue, value) ;
	}

	static Value toValue (Node node, Property.Type requiredType, String valueString) throws ALRepositoryException {
		Value value = null ;
		if (requiredType.equals(Property.Type.DATE)) { // Calendar
			if (valueString == null){
				value = Value.newValue(requiredType, null) ;
			} else {
				value = Value.newValue(requiredType, DateUtil.stringToCalendar(valueString)) ;
			}
		}else if (requiredType.equals(Property.Type.BINARY)) { //BinaryPropertyValue
			value = Value.newValue(requiredType, BinaryPropertyValue.toBinaryValue(valueString)) ;
		} else if (requiredType.equals(Property.Type.NAME)){
			value = Value.newValue(requiredType, valueString) ;
		} else if (requiredType.equals(Property.Type.REFERENCE)){
			value = Value.newValue(requiredType, node.getSession().getNode(valueString)) ;
		} else {
			value = Value.newValue(requiredType, valueString) ;
		}
		return value ;
	}
	
	public boolean equals(Object o){
		if (! (o instanceof Property)) {
			return false;
		}
		
		Property that = (Property)o ;
		return this.node.equals(that.getNode()) && this.getId().equals(that.getId()) ;
	}
	
	public int hashCode(){
		return this.node.hashCode() + this.getId().hashCode() ;
	}
	
	public Property newClone() {
		return new PropertyImpl(node, this.propertyDefinition, this.hasArrayValue, (Value) this.value.newClone());
	}

	public IDString getId() {
		return propertyDefinition.getId();
	}

	public Value getValue() {
		return this.value;
	}
	
	public boolean hasArrayValue(){
		return this.hasArrayValue ;
	}
	
	public Value[] getValues() throws ALRepositoryException{
		if (hasArrayValue() && value instanceof ArrayValue) {
			return ((ArrayValue)value).toList().toArray(new Value[0]) ;
		} else if ( hasArrayValue() && !(value instanceof ArrayValue)) {
			return node.getSession().getValues(node, this);
		} else {
			return new Value[]{this.value} ;
		}
	}

	public Property.Type getType() {
		return propertyDefinition.requiredType();
	}

	public String getTypeName() {
		return propertyDefinition.requiredType().toString();
	}

	public String getString() {
		return this.value.getString();
	}

	public Node getNode() {
		return this.node;
	}

	public void setValue(Value value) throws ALRepositoryException {
		this.value = value;
		this.hasArrayValue = (value instanceof ArrayValue) ;

		node.getSession().fireEvent(this, EventType.PROPERTY_CHANGED) ;
	}
	
	public static Property notDefinedProperty(Node aboutNode, PropertyDefinition propertyDefinition) {
		return PropertyImpl.create(aboutNode, propertyDefinition, Value.EMPTY);
	}

	public String toString() {
		return this.propertyDefinition.getName() + " : " + getString();
	}

	public boolean equalValue(Property property) {
		return value.equals(property.getValue());
	}

	public XML toXML() {
		XML result = new XML(getId().getString());

		result.addAttribute("id", getId().getString());
		result.addAttribute(PROPERTY_TYPE, getTypeName());
		result.addAttribute(VALUE, value.getString() == null ? "" : value.getString());

		return result;
	}

	public Object getObject() throws ALRepositoryException {
		return getObject(new HashMap());
	}
	
	public Object getObject(Map param) throws ALRepositoryException {
		return this.value.getObject(param, this.node) ;
	}
	
	public void accept(ItemVisitor v) {
		v.visit(this) ;
	}

}
