package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ItemNotFoundException;
import com.bleujin.lore.core.exception.NodeException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.observation.UserBean;
import com.bleujin.lore.core.observation.Event.EventType;
import com.bleujin.lore.core.observation.Event.Mode;
import com.bleujin.lore.core.persitence.ILocalStore;
import com.bleujin.lore.core.search.searcher.NodeSearcher;
import com.bleujin.lore.core.value.Value;

public class SessionStore {

	
	// session level 접근으로 동시성은 고려하지 않는다. 
	private ILocalStore local ;
	private UserBean userInfo ;

	private Set<Node> pendingNodes ;
	private Map<String, Link> links ;
	
	SessionStore(ILocalStore localStore, UserBean userInfo) {
	    this.local = localStore ;
	    this.userInfo = userInfo ;
	    pendingNodes = new HashSet<Node>() ;
	    links = new HashMap<String, Link>() ;
    }
	
    INode createNode(UserSession session, Node parent, NodeType nodetype, NodeName name, String explain) throws ALRepositoryException {
    	INode newNode = local.createNode(session, parent, nodetype, name, explain) ;
    	if(exist(session, newNode.getPath())) throw new NodeException("same path already exist.. : " + newNode.getPath()) ;

    	session.fireEvent(newNode, EventType.NODE_ADDED);
    	newNode.setParent(parent) ;
    	newNode.setDefaultProperty() ;

		// loadMemory(newNode) ;

		return newNode ;
    }

    Link createLink(UserSession session, Node from, Node to, LinkType linkType, Position position){
    	
    	Link link =  LinkImpl.create(session, from, to, linkType, position);
    	links.put(link.toString(), link) ;

    	return link ;
    }
    
    void checkOut(NodeVersion version) throws ALRepositoryException {
    	for(Node node : pendingNodes){
    		if (node.getIdentifier().equals(version.getIdentifier())){
    			pendingNodes.remove(node) ;
    			break;
    		}
    	}
    	
    	version.setMode(Mode.CHECKOUT) ;
    	loadMemory(version) ;
    }

    void delNode(Node del) throws ALRepositoryException {
		GenericIterator<Node> childs = del.getChildNode() ;
		while(childs.hasNext()){
			Node child = childs.next();
			delNode(child) ;
		}
		
		for(Node node : pendingNodes){
			if (del.getIdentifier().equals(node.getIdentifier())) {
				node.setMode(Mode.DELETED) ;				
			}
		}
    }
	
    boolean exist(UserSession session, String path) throws ALRepositoryException {
    	for(Node node : pendingNodes){
    		if(path.equals(node.getPath())) {
    			// return true ;
    			if (node.getMode().isCreated()) return true;
    			else continue;
    		}
    	}
    	
		return local.exist(session, getWorkspacePath(session, path)) ;
    }

    
    
    
    private String getWorkspacePath(UserSession session, String path) {
		return session.getWorkspace().getName() + path;
	}

	Node getNodeByUUID(UserSession session, String uuid) throws ALRepositoryException {
		if (session.getWorkspace().getName().equals(uuid)) return session.getRootNode() ;
		for(Node node : pendingNodes){
			if (uuid.equals(node.getIdentifier())){
				if (node.getMode().isDeleted()) throw new ItemNotFoundException() ;
				//Debug.debug(this, "FOUND SESSION CACHE:" + node);
				return node ;
			}
		}
		
		Node node = local.getNodeByUUID(session, uuid) ;
		return node ;
    }

    Node getNodeByPath(UserSession session, String path) throws ALRepositoryException {
    	if (StringUtil.isBlank(path) || "/".equals(path)) return session.getRootNode() ;
    	
		for(Node node : pendingNodes){
			if (path.equals(node.getPath())){
				if (node.getMode().isDeleted()) throw new ItemNotFoundException() ;
				Debug.debug(this, "FOUND SESSION CACHE : ");
				return node ;
			}
		}
		Node node = local.getNodeByPath(session, path) ;
		return node ;
    }

    void save(Node node) throws ALRepositoryException {
    	local.indexNode(pendingEvent, node) ;
		local.save(pendingEvent, node) ;
		
		if (node.getMode().isDeleted()) return ; 
		node.setMode(Mode.NORMAL) ;
    }
    
    void save() throws ALRepositoryException {
		local.indexNode(pendingEvent, pendingNodes, links) ;
		local.save(pendingEvent) ;
		pendingEvent.clear() ;

		for(Node node : pendingNodes){
			if (node.getMode().isDeleted()) continue ;
			node.setMode(Mode.NORMAL) ;
			for(Link link : node.getLinks(Page.ALL)){
				link.setMode(Mode.NORMAL) ;
			}
		}
		
		for (Link link : links.values()) {
			link.setMode(Mode.NORMAL) ;
		}
		
		this.pendingNodes.clear();
		this.links.clear() ;
    }
    

    NodeSearcher getSearcher(UserSession session, Node startNode) {
    	return local.getSearcher(session, startNode) ;
    }
    
    ILocalStore getLocalStore(){
    	return local ;
    }


    private boolean existSession(Node currNode){
    	for(Node node : pendingNodes){
    		if(currNode.getIdentifier().equals(node.getIdentifier())) {
    			return !node.getMode().isDeleted() ;
    		}
    	}
    	return false ;
    }

	void loadMemory(Node node) {
//		for(Node cnode : pendingNodes){
//			if (cnode.getIdentifier().equals(node.getIdentifier())) {
//				//cnode.setMode(node.getMode()) ;
//				return ;
//			} 
//		}
		if (node.isRoot()) return ;
//		if (nodes.contains(node)) {
//			return ;
//		}
//		Debug.debug(this, "LOAD [" + node.getMode() + "] " + node + " " + nodes.contains(node));
//		if (! node.getMode().equals(Mode.NORMAL)) return ;
		
//		pendingNodes.add(node) ;
	}

	public void confirm() {
		// Debug.debug(this, "NODE IN MEMROY : " + nodes);
		for(Link link : links.values()){
			Debug.debug("LINK IN MEMROY : " + link);
		}
	}

	
	private List<Event> pendingEvent = new ArrayList<Event>() ;
	public void handle(Event event) {
		pendingEvent.add(event) ;
	}

	List<Event> getPendingEvent() {
		return Collections.unmodifiableList(pendingEvent);
	}

	void refresh(boolean keepChanges) throws ALRepositoryException {
		if (keepChanges){
			save() ;
		}
	    pendingNodes.clear() ;
	    links.clear();
	    pendingEvent.clear() ;
	}


	boolean hasPendingChanges(){
		return pendingEvent.size() > 0 ;
	}

	public GenericIterator<Node> getNodes(UserSession session, NodeType nodetype, boolean includeSub, Page page) throws ALRepositoryException{
		return local.getNodes(session, nodetype, includeSub, page) ;
	}

	public Value[] getValues(Node node, Property property) throws ALRepositoryException {
		return local.getValues(node, property);
	}

	
}
