package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.List;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.AccessDeniedException;
import com.bleujin.lore.core.exception.ItemNotFoundException;
import com.bleujin.lore.core.exception.ShareNodeException;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.ShareNode;
import com.bleujin.lore.core.persitence.NodeBean;
import com.bleujin.lore.core.value.Value;

public class ShareNodeImpl extends AbstractShareNode implements ShareNode{

	private static final long serialVersionUID = -2517417260139415380L;
	
	private final UserSession session ;
	private final AbstractNode base ;
	private final ShareInfo shareInfo ;
	private final ShareNode parent ;

	private ShareNodeImpl(UserSession session, Node node, ShareInfo shareInfo, ShareNode parent) {
		this.session = session ;
		this.base = (AbstractNode)node ;
		this.shareInfo = shareInfo ;
		this.parent = parent ;
	}
	
	public static ShareNode shareRoot(UserSession session, Node node, ShareInfo shareInfo){
		return new ShareNodeImpl(session, node, shareInfo, null); 
	}

	@Override
	protected NodeBean getBean() {
		return base.getBean();
	}

	public String getName() {
		return isRoot() ? shareInfo.getShareName() : base.getName() ;
	}
	
	// property -> node access 문제는 authority 책임
	@Override
	public Property getProperty(IDString propId) throws ALRepositoryException {
		return base.getProperty(propId);
	}

	public GenericIterator<Property> getProperties() throws ALRepositoryException {
		return base.getProperties();
	}
	
	@Override
	public void setProperty(IDString propId, Value value) throws ALRepositoryException {
		checkEnableWrite();
		base.setProperty(propId, value) ;
	}

	private void checkEnableWrite() {
		if (shareInfo.isEnableWrite() == false) {
			throw new ShareNodeException("exception.sharenode.info.readonly") ;
		}
	}

	
	public ShareNode getParent() throws ItemNotFoundException, AccessDeniedException, ALRepositoryException {
		if (parent == null) throw new ShareNodeException("exception.sharenode.noparent") ;
		return parent;
	}


	public void setDefaultProperty() throws ALRepositoryException {
		base.setDefaultProperty() ;
	}

	public GenericIterator<ShareNode> getChildNode() throws ALRepositoryException {
		return getChildNode(Page.HUNDRED_ONE);
	}

	public GenericIterator<ShareNode> getChildNode(Page page) throws ALRepositoryException {
		GenericIterator<Node> nodes = base.getChildNode(page) ;
		List<ShareNode> result = new ArrayList<ShareNode>() ;
		for(Node node : nodes){
			result.add(new ShareNodeImpl(this.session, node, this.shareInfo, this)) ;
		}
		
		return new GenericIteratorHelper<ShareNode>(result);
	}

	@Override
	protected INode getBaseNode() {
		return base;
	}

	public Node getTestNode() {
		return base;
	}
	
	public ShareNode save() throws ALRepositoryException {
		base.save() ;
		return this;
	}

	public boolean isRoot() {
		return parent == null;
	}

	// .............
	// not oppened.. because using AbstractINode 
	
	public UserSession getSession() {
		return session;
	}
	public void setParent(Node node) throws ALRepositoryException {
		base.setParent(node) ;
	}
	public void addLink(Node target, LinkType linkType, Position position) throws ALRepositoryException {
		base.addLink(target, linkType, position) ;
	}



}
