package com.bleujin.lore.core.dimpl;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.exception.LockException;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Lock;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.workspace.TestCredential;

public class TestLock extends TestCaseParent {

	public void testBaseLock() throws Exception {
		Node parent = saveNode(objectType, "parent");
		Node child = createNodeSave(parent, objectType, "child");

		boolean isDeep = true;
		Lock lock = parent.lock(isDeep);
		assertTrue(child.isLocked());
		assertTrue(parent.isLocked());

		lock.unlock();
		assertFalse(child.isLocked());
		assertFalse(parent.isLocked());

		lock = parent.lock(true);
		Lock anotherLock = parent.lock(false);
		assertEquals(lock, anotherLock);
		lock.unlock();

		Lock theThirdLock = parent.lock(false);

		try {
			anotherLock = parent.lock(true);
			fail();
		} catch (LockException ignore) {
		}

		theThirdLock.unlock();
		assertFalse(child.isLocked());
	}

	public void testHierarchicalLock() throws Exception {
		Node parent = saveNode(objectType, "parent");
		INode newNode = session.createNode(parent, objectType, "child");
		Node child = newNode.save() ;

		Lock lock = parent.lock(false);
		assertNotNull(lock);
		Lock childLock = child.lock(false);
		assertNotNull(childLock);

		assertFalse(lock.equals(childLock));

		lock.unlock();

		assertFalse(parent.isLocked());
		assertTrue(child.isLocked());

		lock = parent.lock(true);
		assertEquals(lock, child.getLock());
		assertEquals(parent.getLock(), child.getLock());

		childLock.unlock();
		assertTrue(child.isLocked());

		lock.unlock();
		assertTrue(!child.isLocked());
	}
	
	public void testSessionLock() throws Exception {
		Node parent = session.createNode(session.getRootNode(), objectType, "parent").save();
		Node child = session.createNode(parent, objectType, "child").save();
		
		session.save() ;
		
		UserSession otherSession = anotherLore.login(new TestCredential("anotherUser")) ;
		
		Node parentOther = otherSession.getNode("/parent");
		Node childOther = otherSession.getNode("/parent/child");

		assertEquals(parent, parentOther);
		assertEquals(child, childOther);
		
		Lock l = parent.lock(true) ;
		try {
			parentOther.lock(false) ;
			fail();
		} catch(LockException ignore){}
		
		try {
			childOther.lock(false) ;
			fail();
		} catch(LockException ignore){}
		
		
		assertEquals(true, parentOther.isLocked()) ;
		Lock parentLock = parentOther.getLock() ;
		try {
			parentLock.unlock();
			fail() ;
		}catch(LockException ignore){
		}

		
		
		UserSession sameUserSession = anotherLore.login(new TestCredential("bleujin")) ;
		Node parentSame = sameUserSession.getNode("/parent");
		
		
		parentSame.lock(false) ;
		Lock lock = parentSame.getLock() ;
		lock.unlock() ;
		
		assertEquals(false, childOther.isLocked()) ;
		
	}
	
	public void testLogout() throws Exception {
		Node parent = saveNode(objectType, "parent");
		Node child = createNodeSave(parent, objectType, "child");
		
		parent.lock(true) ;
		
		assertEquals(true, parent.isLocked()) ;
		assertEquals(true, child.isLocked()) ;
		
		session.logout() ;
		
		assertEquals(false, parent.isLocked()) ;
		assertEquals(false, child.isLocked()) ;
		
	}
	
	
	public void testFinally() throws Exception {
		
		int i = 0 ;
		try {
			throw new IllegalArgumentException("runtime");
		} catch( Exception e){
			
		} finally{
			i++ ;
		}
		assertEquals(1, i) ;
	}
}