package com.bleujin.lore.core.dimpl;

import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class TestNodeType extends TestCaseParent{

	public void testNamespace() throws Exception {
		NodeType robotType = getRobotType() ;
		assertEquals("default:robot", robotType.getName()) ;
    }
	
	public void testCreateType() throws Exception {
		NodeType aaaType = registerNodeType(objectType, "aaa", new PropertyDefinition[0]);

		assertNotNull(aaaType) ;
		assertEquals(true, ntManager.hasNodeType("aaa")) ;
		
	}

	public void testRemoveType() throws Exception {
		NodeType newType = registerNodeType(objectType, "aaa", new PropertyDefinition[0]);
		ntManager.unregister("aaa") ;

		assertEquals(false, ntManager.hasNodeType("aaa")) ;
		
	}

	public void testGetType() throws Exception {
		ntManager.register(objectType, "_person", new PropertyDefinition[0]) ;

		NodeType personType =  ntManager.getNodeType("_person");
		
		assertEquals(objectType, personType.getSuperType()) ;
    }
	
	
	public void testMemberType() throws Exception {
		NewNodeType robotTemplate = getRobotTypeTemplate() ; 
		robotTemplate.setSelfMemberType() ;
		NodeType robot = registerNodeType(robotTemplate) ;
		assertTrue(robot.getMappedDefinitions().size() == 1) ;

		// test MemberType
		Node p2 = saveNode(robot, "p2") ;
		
		Node obj =saveNode(objectType, "obj") ;
		try {
			Debug.debug(this.getClass(), p2.getNodeType().toString());
			p2.addMemberNode(obj) ;
			fail() ;
		} catch(IllegalArgumentException ignore){
			
		}
		
		assertEquals(robot.getMemberTypeId(), ntManager.getNodeType("robot").getMemberTypeId()) ;
    }

	
	public void testInheritType() throws Exception {
	    NodeType robotType = getRobotType() ;
	    GenericIterator<MappedDefinition> mds = robotType.getMappedDefinitions() ;
	    
	    assertEquals(1, mds.size()) ;
	    String explain = "explain";
		assertEquals(explain, mds.next().getName()) ;
	    
	    
	    String mp = "mpmp";
		NodeType extend_robot = registerNodeType(robotType, "extend_robot", new PropertyDefinition[]{createPropertyDefinition(mp, Property.Type.STRING)}) ;

	    mds = extend_robot.getMappedDefinitions() ;
	    assertEquals(2, mds.size()) ;
	    assertEquals(robotType, extend_robot.getSuperType()) ;
	    assertEquals(true, extend_robot.containsPropId(mp)) ;
	    assertEquals(true, extend_robot.containsPropId(explain)) ;
	    

	    String gp = "gpgp";
		NodeType grand_extend_robot = registerNodeType(extend_robot, "grand_extend_robot", new PropertyDefinition[]{createPropertyDefinition(gp, Property.Type.STRING)}) ;
	    mds = grand_extend_robot.getMappedDefinitions() ;
	    assertEquals(extend_robot, grand_extend_robot.getSuperType()) ;
	    assertEquals(3, mds.size()) ;
	    
	    assertEquals(true, grand_extend_robot.containsPropId(explain.toUpperCase())) ;
	    assertEquals(true, grand_extend_robot.containsPropId(mp.toUpperCase())) ;
	    assertEquals(true, grand_extend_robot.containsPropId(gp.toUpperCase())) ;
    }
	
	public void testUpdate() throws Exception {
		NewNodeType updateType = getRobotType().updateFor() ;
		updateType.setConstraint("explain", new RequireConstraint()) ;
		ntManager.register(updateType, true) ;
		
		NodeType robotType = ntManager.getNodeType("robot");
		MappedDefinition md = robotType.getMappedDefinition("explain") ;
		assertEquals(new RequireConstraint().getClass().getName(), md.getConstraint().getClass().getName()) ;

		updateType.addPropertyDefinition("year", Property.Type.LONG) ;
		ntManager.register(updateType, true) ;
	}

	
}
