package com.bleujin.lore.core.dimpl;

import java.math.BigDecimal;
import java.util.Calendar;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.NotMatchePropertyTypeException;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.node.BinaryPropertyValue;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.value.TestValue;
import com.bleujin.lore.core.value.Value;

public class TestProperty extends TestCaseParent{

	public void testValid() throws Exception {
		String string = "firstName";
		PropertyDefinition pd_firstname = createPropertyDefinition(string, Property.Type.STRING) ;
		Value v = Value.stringValue("bleujin") ;
		
		NewNodeType newType =  createNewNodeType(objectType, "other", new PropertyDefinition[]{pd_firstname}) ;
		newType.setConstraint(pd_firstname,  new RequireConstraint()) ;
		NodeType other = registerNodeType(newType) ;  
		Node node = saveNode(other, "nayouuung");
		
	    node.setProperty(string, v);
	    Property property = node.getProperty(string) ;
	    assertTrue(node.getViolateConstraintMessage().size() == 0) ;

	    node.setProperty(string, Value.EMPTY) ;
	    property = node.getProperty(string) ;
	    assertFalse(node.getViolateConstraintMessage().size() == 0) ;
    }
	
	public void testPropertyValue() throws Exception {
		Node node = saveNode(getRobotType(), "dummy") ;
		
		Property explain = node.getProperty("explain") ;
		assertEquals(null, explain.getObject()) ;
		
		explain.setValue(Value.stringValue("abcdefg")) ;
		assertEquals("abcdefg", explain.getObject()) ;
		
		session.save() ;
		
		Node loadNode = session.getNode(node.getUUIDString());
		//Debug.debug(this, loadNode.getMode());
		assertEquals("abcdefg", loadNode.getString("explain")) ;

		explain = loadNode.getProperty("explain") ;
		//Debug.debug(this, loadNode + " " + loadNode.getSession());
		
		Debug.debug(this, loadNode + "" + loadNode.getSession());
		Debug.debug(this, explain.getNode());
		
		explain.setValue(Value.stringValue("123456")) ;
		
		session.save() ;
		
		loadNode = session.getNode(node.getUUIDString());
		assertEquals("123456", loadNode.getString("explain")) ;
	}
	
	
	public void testProperty() throws Exception {
		Node testFirstNode = createTestNode() ;

		String _string = "string";
		String _binary = "binary";
		String _long = "long";
		String _double = "double";
		String _date = "date";
		String _boolean = "boolean";
		String _name = "name";
		String _path = "path";
		String _reference = "reference";
		String _text = "text";
		String _decimal = "decimal";
		String _vtext = "vtext";
		PropertyDefinition[] pds = new PropertyDefinition[]{
				createPropertyDefinition(_string, Property.Type.STRING), 
				createPropertyDefinition(_binary, Property.Type.BINARY), 
				createPropertyDefinition(_long, Property.Type.LONG), 
				createPropertyDefinition(_double, Property.Type.DOUBLE), 
				createPropertyDefinition(_date, Property.Type.DATE), 
				createPropertyDefinition(_boolean, Property.Type.BOOLEAN), 
				createPropertyDefinition(_name, Property.Type.NAME), 
				createPropertyDefinition(_path, Property.Type.PATH), 
				createPropertyDefinition(_reference, Property.Type.REFERENCE), 
				createPropertyDefinition(_text, Property.Type.TEXT), 
				createPropertyDefinition(_decimal, Property.Type.DECIMAL), 
				createPropertyDefinition(_vtext, Property.Type.VIRTUAL_TEXT) 
		} ;
		
		NodeType allType = registerNodeType(objectType, "allType", pds) ;

		Node node = createNodeSave(testFirstNode, allType, "allType") ;
		node.setProperty(_string, (Object)"123") ;
		node.setProperty(_binary, (Object)TestValue.binaryValue) ;
		node.setProperty(_long, (Object)12311212121212L) ;
		node.setProperty(_double, (Object)123.3) ;
		Calendar cal = Calendar.getInstance();
		cal.setTime(DateUtil.stringToDate("20060504-141224")) ;
		node.setProperty(_date, (Object)cal) ;
		node.setProperty(_boolean, (Object)true) ;
		node.setProperty(_name, (Object) IDString.unknownId("namedd")) ;
		node.setProperty(_path, (Object) testFirstNode.getPath()) ;
		node.setProperty(_reference, (Object)testFirstNode) ;
		node.setProperty(_text, (Object)new StringBuffer("abcdd")) ;
		node.setProperty(_decimal, (Object)new BigDecimal(123)) ;
		node.setProperty(_vtext, (Object)"$node.getIdentifier()") ;
		session.save() ;
		
		
		GenericIterator<Property> properties = node.getProperties() ;
		while(properties.hasNext()){
			Property property = properties.next() ;
			assertEquals(allType.getMappedDefinition(property.getId()).requiredType(), property.getValue().getPropertyType()) ;
		}
		
		assertEquals("123", node.getProperty(_string).getString()) ;
		assertEquals("111.jpg", node.getProperty(_binary).getString()) ;
		assertEquals("12311212121212", node.getProperty(_long).getString()) ;
		assertEquals("123.3", node.getProperty(_double).getString()) ;
		assertEquals("20060504-141224", node.getProperty(_date).getString()) ;
		assertEquals("true", node.getProperty(_boolean).getString()) ;
		assertEquals("namedd", node.getProperty(_name).getString()) ;
		assertEquals("/" + getTestNodeName(), node.getProperty(_path).getString()) ;
		assertEquals(testFirstNode.getIdentifier(), node.getProperty(_reference).getString()) ;
		assertEquals("abcdd", node.getProperty(_text).getString()) ;
		assertEquals("123", node.getProperty(_decimal).getString()) ;
		assertEquals("$node.getIdentifier()", node.getProperty(_vtext).getString()) ;
		

		assertEquals("123", node.getProperty(_string).getObject()) ;
		assertEquals("jpg", ((BinaryPropertyValue)node.getProperty(_binary).getObject()).getContentType() ) ;
		assertEquals(12311212121212L, node.getProperty(_long).getObject()) ;
		assertEquals(123.3D, node.getProperty(_double).getObject()) ;
		assertEquals("20060504-141224", DateUtil.calendarToString((Calendar)node.getProperty(_date).getObject()) ) ;
		assertEquals(Boolean.TRUE, node.getProperty(_boolean).getObject()) ;
		assertEquals("namedd", node.getProperty(_name).getObject()) ;
		assertEquals(session.getNode("/" + getTestNodeName()), node.getProperty(_path).getObject()) ;
		assertEquals(session.getNode("/" + getTestNodeName()), node.getProperty(_reference).getObject()) ;
		assertEquals(true, node.getProperty(_text).getObject() instanceof StringBuffer) ;
		assertEquals(new StringBuffer("abcdd").toString(), node.getProperty(_text).getObject().toString()) ;
		assertEquals(new BigDecimal(123), node.getProperty(_decimal).getObject()) ;
		assertEquals(node.getIdentifier(), node.getProperty(_vtext).getObject()) ;
		
		node.setProperty(_boolean, "TRUE") ;
		assertEquals(true, node.getProperty(_boolean).getValue().getBoolean()) ;

		node.setProperty(_boolean, "true") ;
		assertEquals(true, node.getProperty(_boolean).getValue().getBoolean()) ;

		node.setProperty(_boolean, "1213") ;
		assertEquals(false, node.getProperty(_boolean).getValue().getBoolean()) ;

		try {
			node.setProperty(_long, "123a") ;
			fail() ;
		} catch(NotMatchePropertyTypeException ignore){
		}

	}
	

}
