package com.bleujin.lore.core.dimpl;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.mapping.StringDefaultValue;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class TestSerialization extends TestCaseParent {

	private ByteArrayOutputStream bout = null ;

	public void setUp() throws Exception {
		super.setUp() ;
		bout = new ByteArrayOutputStream();
	}
	
	public void xtestSerial() throws Exception {
		List<String> twoList= new ArrayList<String>();
		twoList.add("1234") ;
		twoList.add("abcd") ;

		ObjectOutputStream out = new ObjectOutputStream(bout);
		out.writeObject(twoList);
		out.close();
		

		ByteArrayInputStream bis = new ByteArrayInputStream(bout.toByteArray());
		ObjectInputStream in = new ObjectInputStream(bis);
		List<String> readList = (List<String>) in.readObject();
		Debug.debug(this, Arrays.toString(readList.toArray(new String[0])) );
		
		bis = new ByteArrayInputStream(bout.toByteArray());
		in = new ObjectInputStream(bis);
		readList = (List<String>) in.readObject();
		Debug.debug(this, Arrays.toString(readList.toArray(new String[0])) );
	}
	
	
	
	public void testSaveNodeType() throws Exception {

		String explain = "explain";
		PropertyDefinition pd = session.getNodeTypeManager().createPropertyDefinition(explain, Property.Type.STRING) ;
		NewNodeType newType = createNewNodeType(objectType, "abcd", new PropertyDefinition[]{pd});
		newType.setConstraint(explain, new RequireConstraint());
		newType.setDefaultValue(explain, new StringDefaultValue("abcd")) ;
		NodeType abcd = registerNodeType(newType) ;

		
		GenericIterator<NodeType> nodetypes = ntManager.getAllNodeTypes("default") ;
		List<NodeType> types = nodetypes.toList();
		assertEquals(2, types.size()) ;
		
		writeObject(types) ;
		writeObject(types) ;
		writeObject(types) ;
		
		List<NodeType> readTypes = (List<NodeType>)readObject() ;
		assertEquals(2, readTypes.size()) ;

		//Debug.debug(this, readTypes.get(0).toXML());
	}
	
	public void testNode() throws Exception {
		NodeType robotType = getRobotType() ;
		Node node = saveNode(robotType, "abcdefg") ;
		node.setProperty("explain", "newNode") ;
		
		writeObject(node) ;
		
		Node readNode = (Node)readObject() ;
		assertNotNull(readNode) ;
//		Debug.debug(this, readNode.toXML());
//		Debug.debug(this, readNode.getSession());

	}

	private void writeObject(Object obj) throws IOException {
		ObjectOutputStream out = new ObjectOutputStream(bout);

		out.writeObject(obj);
		out.close();
	}

	private Object readObject() throws IOException, ClassNotFoundException {

		ByteArrayInputStream bis = new ByteArrayInputStream(bout.toByteArray());
		ObjectInputStream in = new ObjectInputStream(bis);
		return in.readObject();

	}
}
