package com.bleujin.lore.core.dimpl;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.value.Value;

public class TestSessionStoreProcedure extends TestCaseParent{

	private int robot_default = 3 ; 
	private SessionStore store ;

	public void setUp() throws Exception {
		super.setUp() ;
		store = session.getSessionStore() ;
	}
	
	public void testFirst() throws Exception {
		assertEquals(0, store.getPendingEvent().size()) ;
		
		Node node = saveNode(getRobotType(), "sm949") ;
		assertEquals(true, store.getPendingEvent().size() > 0) ;
		assertEquals(robot_default, store.getPendingEvent().size()) ;
	}
	
	public void testDelNode() throws Exception {

		Node node = saveNode(getRobotType(), "sm949") ;
		node.delete() ;
		assertEquals(robot_default + 1, store.getPendingEvent().size()) ;
	}
	
	public void testAddLink() throws Exception {
		Node testNode = createNodeSave(session.getRootNode(), objectType, "test");
		session.save() ;
		Node node = saveNode(getRobotType(), "sm949") ;

		node.addLink(testNode, LinkTypeImpl.load("parent"), Position.AFTER) ;
		assertEquals(robot_default + 1, store.getPendingEvent().size()) ;
	}
	
	public void testSetProperty() throws Exception {
		Node node = saveNode(getRobotType(), "sm949") ;
		
		node.setProperty("explain", Value.stringValue("SM949")) ;
		assertEquals(robot_default + 1, store.getPendingEvent().size()) ;

		node.setProperty("explain", Value.stringValue("SM949")) ;
		assertEquals(robot_default + 1, store.getPendingEvent().size()) ;
	}
	

	public void testProperty2() throws Exception {
		Node node = saveNode(getRobotType(), "sm949") ;
		session.save() ;
		
		node.setProperty("explain", Value.stringValue("SM949")) ;
		assertEquals(1, store.getPendingEvent().size()) ;
		
		Property property = node.getProperty("explain") ;
		session.save() ;
		assertEquals(0, store.getPendingEvent().size()) ;
		
		property.setValue(Value.stringValue("abc")) ;
		assertEquals(1, store.getPendingEvent().size()) ;
	}
	
	
	public void testCheckOut() throws Exception {
		Node node = saveNode(getRobotType(), "sm949") ;
		session.save() ;

		NodeVersion v = node.checkIn() ;
		v.checkOut() ;
		assertEquals(2, store.getPendingEvent().size()) ;
	}

	
	public void testCheckOut2() throws Exception {
		Node node = saveNode(getRobotType(), "sm949") ;
		session.save() ;

		NodeVersion v = node.checkIn() ;
		v.setProperty("explain", "SM") ;
		v.checkOut() ;
		assertEquals(2, store.getPendingEvent().size()) ;
	}


}
