package com.bleujin.lore.core.dimpl;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.exception.ItemNotFoundException;
import com.bleujin.lore.core.exception.ShareNodeException;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.ShareNode;
import com.bleujin.lore.core.persitence.ILocalStore;
import com.bleujin.lore.core.workspace.TestCredential;
import com.bleujin.lore.core.workspace.Workspace;

public class TestShare extends TestCaseParent{

	private Workspace otherWorkspace ;
	private String otherWorkspaceName = "other";
	private Node targetNode ;
	private String shareName = "/share/name";
	
	private String explain = "explain";

	public void setUp() throws Exception {
		super.setUp() ;
		// for test
		ILocalStore local = session.getLocalStore() ;
		otherWorkspace = Workspace.create(otherWorkspaceName, anotherLore, local) ;
		
		anotherLore.addNewWorkspace(otherWorkspace) ;

		targetNode = saveNode(getRobotType(), "abcd") ;
		session.save() ;
		workspace.shareNode(shareName, "/abcd", ShareNode.ENABLE_WRITE) ;
		workspace.shareNode(shareName + "/read", "/abcd", ShareNode.READ_ONLY) ;
	}

	public void testShare() throws Exception {
		
		UserSession otherSession = anotherLore.login(new TestCredential("bleuhero"), "other") ;

		try {
			Node shareNode = otherSession.getNode("/abcd") ;
			fail() ;
		} catch(ItemNotFoundException ignore){
		}


		try {
			ShareNode shareNode = otherSession.getShareNode("/share/not_exist_name") ;
			fail() ;
		} catch(ShareNodeException ignore){
		}

		ShareNode shareNode = otherSession.getShareNode(shareName) ;
		assertEquals(shareName, shareNode.getName()) ;
	}
	
	public void testDuplicate() throws Exception {
		try {
			workspace.shareNode(shareName, "/abcd", ShareNodeImpl.ENABLE_WRITE) ;
			fail() ;
		} catch(ShareNodeException ignore){
			
		}
		
	}
	
	
	public void testProperty() throws Exception {
		targetNode.setProperty(explain, "abcdefg") ;
		session.save() ;

		ShareNode shareNode = session.getShareNode(shareName) ;
		assertEquals("abcdefg", shareNode.getString(explain)) ;
		
	}
	
	public void testSetProperty() throws Exception {
		UserSession otherSession = anotherLore.login(new TestCredential("bleuhero"), "other") ;
		ShareNode shareNode = otherSession.getShareNode(shareName) ;
		shareNode.setProperty(explain, "123456") ;
		otherSession.save() ;
		
		shareNode = otherSession.getShareNode(shareName) ;
		assertEquals("123456", shareNode.getString(explain)) ;
		
		Node node = session.getNode("/abcd") ;
		assertEquals("123456", node.getString(explain)) ;
	}
	

	public void testEnableWrite() throws Exception {
		UserSession otherSession = anotherLore.login(new TestCredential("bleuhero"), "other") ;
		ShareNode shareNode = otherSession.getShareNode(shareName + "/read") ;
		try {
			shareNode.setProperty(explain, "123456") ;
			fail() ;
		} catch(ShareNodeException ignore){
		}
	}
	
	
	public void testSave() throws Exception {
		Node node = session.getNode("/abcd");
		node.setProperty(explain, "init") ;

		ShareNodeImpl shareNode = (ShareNodeImpl)session.getShareNode(shareName) ;
	}
	
	
	public void testChild() throws Exception {
		targetNode.setProperty(explain, "abcd") ;
		
		INode child1 = targetNode.createChild(getRobotType(), "child1") ;
		child1.setProperty(explain, "init") ;
		session.save() ;

		UserSession session_1 = anotherLore.login(new TestCredential("session_1"), "other") ;
		ShareNode shareNode_1 = session_1.getShareNode(shareName) ;

		
		assertEquals(false, session_1.getShareNode(shareName) == session_1.getShareNode(shareName));

		
		try {
			shareNode_1.getParent() ;
			fail() ;
		}catch(ShareNodeException ignore){
		}
		
		GenericIterator<ShareNode> childNodes_1 = shareNode_1.getChildNode();
		assertEquals(1, childNodes_1.size()) ;
		ShareNode child_1 = childNodes_1.next();
		
		assertEquals("abcd", child_1.getParent().getString(explain)) ;
		child_1.setProperty(explain, "modified") ;
		
		// nextByOther.save() ;
		// otherSession.save() ;
		
		UserSession session_2 = anotherLore.login(new TestCredential("session_2"), "other") ;
		ShareNode shareNode_2 = session_2.getShareNode(shareName) ;

		GenericIterator<ShareNode> childNodes_2 = shareNode_2.getChildNode();
		assertEquals(1, childNodes_2.size()) ;
		ShareNode child_2 = childNodes_2.next() ;
		
		assertEquals(false, child_1 == child_2) ;
		
		assertEquals("init", child_2.getString(explain)) ;
		
	}
	
	
}
