package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.RandomStringUtils;

import com.bleujin.framework.util.RandomUtil;
import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.AccessDeniedException;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.workspace.AnotherLore;
import com.bleujin.lore.core.workspace.TestCredential;
import com.bleujin.lore.impl.sample.SampleNode;

public class TestSpeed extends TestCaseParent {

	public void testThread() throws Exception {

		new SampleNode(session).impSimple(0.2F); // sample:emp, loc, dept

		List<String> store = new ArrayList<String>();
		List<Thread> threads = new ArrayList<Thread>() ;
		for (int i = 0; i < 5; i++) {
			Thread w = new WriterThread(anotherLore, "writer_" + i, store);
			threads.add(w) ;
			w.start() ;
		}
		Thread.sleep(500) ;
		for (int i = 0; i < 50; i++) {
			Thread r = new ReaderThread(anotherLore, "reader_" + i, store);
			threads.add(r) ;
			r.start() ;
		}

		for (Thread thread : threads) {
			thread.join() ;
		}
	}
}

class ReaderThread extends Thread {
	private UserSession session;

	private List<String> store;

	public ReaderThread(AnotherLore lore, String name, List<String> store) throws AccessDeniedException {
		super(name) ;
		this.session = lore.login(new TestCredential(name));
		this.store = store;
	}

	public void run() {
		try {
			for (int i = 0; i < 10; i++) {
				if (store.size() > 0) { 
					String str = store.get(RandomUtil.nextInt(store.size()));
					String nodeinfo = RandomUtil.nextBoolean() ? StringUtil.substringBefore(str, ",") : StringUtil.substringAfter(str, ",");
					Node node = session.getNode(nodeinfo);
					node.getProperties();
					Debug.debug(node + " read");
				} else {
					Debug.debug("store empty");
					Thread.sleep(RandomUtil.nextInt(5) * 100);
				}
				// Thread.sleep(RandomUtil.nextInt(5) * 100);
			}
		} catch (ALRepositoryException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}

class WriterThread extends Thread {

	private UserSession session;
	private List<String> store;

	public WriterThread(AnotherLore lore, String name, List<String> store) throws AccessDeniedException {
		super(name) ;
		this.session = lore.login(new TestCredential(name));
		this.store = store;
	}

	public void run() {
		try {
			for (int i = 0; i < 20; i++) {
				Node parent = session.getNode("/_sample/_emp");
				//Debug.debug("read node", parent, DateUtil.calendarToString(Calendar.getInstance()));
				
				NodeType nodeType = session.getNodeTypeManager().getNodeType("sample:emp");
				//Debug.debug("read type", nodeType, DateUtil.calendarToString(Calendar.getInstance()));
				
				INode newNode = session.createNode(parent, nodeType, RandomStringUtils.randomAlphanumeric(10)) ;
				//Debug.debug("create node", newNode, DateUtil.calendarToString(Calendar.getInstance()));
				
				Node node = newNode.save();
				//Debug.debug("save node", newNode, DateUtil.calendarToString(Calendar.getInstance()));
				
				store.add(node.getIdentifier() + "," + node.getPath());
				Debug.debug(node + " created");
				// Thread.sleep(RandomUtil.nextInt(10) * 100);
			}
		} catch (ALRepositoryException e) {
			e.printStackTrace();
		} 
	}
}
