package com.bleujin.lore.core.dimpl;

import java.util.Calendar;

import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.VersionHistory;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class TestVersion extends TestCaseParent{
	
	private NodeType personType ;
	
	private String firstName = "firstName";
	private String midName = "midName";
	private String lastName = "lastName";

	public void setUp() throws Exception {
		super.setUp() ;
		personType = registerNodeType(objectType, "person", new PropertyDefinition[]{createPropertyDefinition(firstName, Property.Type.STRING), createPropertyDefinition(midName, Property.Type.STRING), createPropertyDefinition(lastName, Property.Type.STRING)});
	}

	public void testDefault() throws Exception {
		Node bleujin = saveNode(personType, "bleujin") ;
		assertEquals(null, bleujin.getString(firstName)) ;
		NodeVersion v = bleujin.checkIn() ;
		v.setProperty(firstName, "bleu") ;
		v.save() ;
		assertEquals("bleu", v.getString(firstName)) ;
		
		Node load = session.getNode("/bleujin") ;
		assertEquals("bleu", load.getString(firstName)) ;
	}
	
	public void testCheckOut() throws Exception {
	    Node bleujin = createNodeSave(session.getRootNode(), personType, "bleujin") ;

	    bleujin.setProperty(firstName, "bleujin") ;
		session.save() ;

		NodeVersion v = bleujin.checkIn() ;
		v.setProperty(firstName, "bleujin2") ;
		v.checkOut() ;
		session.save() ;

		v = bleujin.checkIn() ;
		v.setProperty(firstName, "bleujin3") ;
		v.checkOut() ;
		session.save() ;

		v = bleujin.checkIn() ;
		v.setProperty(firstName, "bleujin4") ;
		v.checkOut() ;
		session.save() ;

		v = bleujin.checkIn() ;
		v.setProperty(firstName, "bleujin5") ;
		v.checkOut() ;
		session.save() ;
		
		Node loadNode = session.getNode("/bleujin") ;
		assertEquals("bleujin5", loadNode.getVersionHistory().getRecentNode().getString(firstName)) ;
		assertEquals(4, loadNode.getVersionHistory().getAllVersions().size()) ;
		
		assertEquals("bleujin5", loadNode.getString(firstName)) ;
		
	}
	
	
	public void testVersion() throws Exception {
	    Node bleujin = createNodeSave(session.getRootNode(), personType, "bleujin") ;
		bleujin.setProperty(firstName, "bleujin") ;
		
	    Node hero = saveNode(personType, "hero") ;
	    hero.setProperty(firstName, "hero") ;
	    
	    bleujin.addMemberNode(hero) ;
	    session.save() ;
	    

	    assertEquals("hero", bleujin.getMemberNode(Page.ALL).next().getString(firstName)) ;
	    

	    NodeVersion v2 = hero.checkIn() ;
	    v2.setProperty(firstName, "hhhh") ;
	    v2.checkOut() ;
	    session.save() ;
	    
	    Node loadBleujin = session.getNode("/bleujin") ;

	    assertEquals("hhhh", loadBleujin.getMemberNode(Page.ALL).next().getString(firstName));
	    assertEquals("hhhh", bleujin.getMemberNode(Page.ALL).next().getString(firstName)) ;
	    
	    Node toNode = bleujin.getLinks(Page.ALL).next().getTo();
//		Debug.debug(this, toNode);
//		for (Property p : toNode.getProperties()) {
//			Debug.debug(this, p);
//		}
	    
	    assertEquals("hhhh", toNode.getString(firstName));
	    
	    // assertEquals("hhhh", ) ;
    }
	
	
	public void xtestBeforeDirection() throws Exception {
	    Node hero = saveNode(personType, "hero") ;
	    hero.setProperty(firstName, "hero") ;
	    
	    Node hero_child = createNodeSave(hero, personType, "hero child") ;
	    session.save() ;

	    NodeVersion v2 = hero.checkIn() ;
	    v2.setProperty(firstName, "hhhh") ;
	    v2.checkOut() ;
	    session.save() ;
	    
	    assertEquals("hhhh", session.getNode("/hero/hero child").getParent().getString(firstName));
	    // 얘는 여전히 과거의 값을 가르킴... 이전에 메모리에 남아 았는 Node들의 처리에 관한 문제..
	    assertEquals("hhhh", hero_child.getParent().getString(firstName)) ; 
	}
	
	
	public void testClone() throws Exception {
	    Node n0 = saveNode(personType, "bleujin") ;
	    n0.setProperty(firstName, "before") ;

	    session.save();
	    
	    NodeVersion v1 = n0.checkIn() ;
	    v1.setProperty(firstName, "after") ;
	    
	    assertEquals(0, n0.getVersionHistory().getAllVersions().size()) ;
	    v1.checkOut() ;
	    
	    assertEquals(1, v1.getVersionNum()) ;
	    session.save() ;
	    assertEquals(1, n0.getVersionHistory().getAllVersions().size()) ;

	    // assertTrue(n.isSame(v1)) ; // true
	    assertEquals(0, n0.getVersionNum()) ;
	    assertEquals(1, v1.getVersionNum()) ;

	    VersionHistory vhistory = n0.getVersionHistory() ;
	    Node current = vhistory.getRecentNode();
	    Node before = vhistory.getVersion(0);

	    assertEquals(0, vhistory.getRecentNode().getVersionNum()) ;

	    Debug.debug(this.getClass(), "before:" + before + ", current:" + current);
//	    Debug.debug("before :" + ((VersionImpl)before).getBaseNode().getProperty(firstName));
//	    Debug.debug("current:" + current.getProperty(firstName));
//	    Debug.debug(((VersionImpl)before).getBaseNode() == current);
	    
	    
		assertEquals("before", before.getString(firstName)) ;
		assertEquals("after", current.getString(firstName)) ;

		assertEquals(0, current.getVersionNum()) ;
		assertEquals(1, before.getVersionNum()) ;


    }
	
	public void testConflict() throws Exception {
		Node n0 = createNodeSave(session.getRootNode(), personType, "firstNode") ;
	    n0.setProperty(firstName, "1") ;
	    n0.setProperty(midName, "jin") ;
	    session.save() ;

	    NodeVersion v1 = n0.checkIn() ;
	    v1.setProperty(firstName, "2") ;
	    v1.checkOut() ;
	    session.save();
	    
	    Node pastNode = session.getNode("/firstNode") ;
	    NodeVersion v2 = pastNode.checkIn() ;
	    v2.setProperty(firstName, "3") ;
	    
	    v2.checkOut() ;
	    
	    Node theNode = session.getNode("/firstNode") ;
	    assertEquals("jin", theNode.getString(midName)) ;
    }
	
	public void testPropertyVersion() throws Exception {
		Node rootNode = session.getRootNode() ;
		Node n0 = createNodeSave(rootNode, personType, "firstNode") ;
	    n0.setProperty(firstName, "before") ;
	    n0.setProperty(midName, "jin");
		n0.setProperty(lastName, "park");

	    n0.save() ;
	    assertEquals(1, rootNode.getChildNode().size()) ;

	    
	    Node v0 = n0.getVersionHistory().getRecentNode();
		assertEquals(3, v0.getProperties().size()) ;
		assertTrue(v0.hasProperty(firstName)) ;
	    assertEquals("before", v0.getProperty(firstName).getValue().getString());


	    NodeVersion v1 = n0.checkIn() ;
	    v1.setProperty(firstName, "after") ;
	    v1.setProperty(lastName, "park");
	    v1.checkOut();
	    //traceOn();
	    v1.save() ;

	    
	    assertEquals("after", v1.getProperty(firstName).getString());
	    assertEquals("before", n0.getProperty(firstName).getString());
	    
	    VersionHistory history = n0.getVersionHistory();
	    
	    Debug.debug(this, this, history.getAllVersions().size());
	    
	    assertEquals(true, n0.isSame(history.getVersion(0)) );
	    assertEquals(1, history.getVersion(0).getVersionNum());
	    
	    assertEquals("after", history.getRecentNode().getProperty(firstName).getValue().getString());
	    assertEquals("before", history.getVersion(0).getProperty(firstName).getValue().getString());

	    assertEquals("park", history.getVersion(0).getProperty(lastName).getValue().getString());
	    assertEquals("jin", history.getVersion(0).getProperty(midName).getValue().getString());
	}
	
	public void testRegister() throws Exception {
		Node testFirstNode = createTestNode() ;
		assertEquals(session.getUserId(), testFirstNode.getRegister().getMyId()) ;
		Node n0 = createNodeSave(testFirstNode, personType, "firstNode") ;
		session.save() ;
		
		Thread.sleep(1500) ;
		
		NodeVersion v1 = n0.checkIn() ;
		v1.checkOut() ;
		session.save() ;
		
		
		Node currNode = n0.getVersionHistory().getRecentNode() ;
		NodeVersion beforeNode = n0.getVersionHistory().getVersion(0) ;
		
//		for (NodeVersion v : n0.getVersionHistory().getAllVersions()) {
//			Debug.debug(this, v + " " + v.getCreated());
//		}
		
		Calendar currNodeDate = currNode.getCreated();
		Calendar beforeNodeDate = beforeNode.getCreated() ;
		assertEquals(-1, beforeNodeDate.compareTo(currNodeDate)) ;
		
	}
	
}