package com.bleujin.lore.core.dimpl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.ShareNode;
import com.bleujin.lore.core.observation.Event;

public class TransientEventProcessor extends EventProcessor{

	private LocalFileStore fos ;
	private Map<String, Node> nodeMap ;
	private Map<String, List<NodeVersion>> versions ;
	
	private Map<String, WeightEvent> events = new HashMap<String, WeightEvent>() ;
	TransientEventProcessor(LocalFileStore fos, Map<String, List<NodeVersion>> versions) throws ALRepositoryException{
		this.fos = fos ;
		this.nodeMap = readNodes() ;
		this.versions = versions ;
	}
	
	@Override
	public void process(List<Event> pendingEvent) throws ALRepositoryException {
		handleEvent(pendingEvent) ;
		affectMap();
		writeNode() ;
		events.clear() ;
	}

	@Override
	public void process(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException {
		handleEvent(pendingEvent) ;
		if (thisNode instanceof ShareNode){
			Node oriNode = nodeMap.get(thisNode.getIdentifier()) ;
			for (Property p : thisNode.getProperties()) {
				// TODO .. hum. =t=;
				// oriNode.setProperty(p.getId(), p.getValue()) ;
			}
			nodeMap.put(thisNode.getIdentifier(), oriNode) ;
		} else {
			Map<String, WeightEvent> aboutWeightEvents = new HashMap<String, WeightEvent>() ;
			
			for (WeightEvent wevent : events.values()) {
				Debug.happy(wevent, wevent.getNodeIdentifier().equals(thisNode.getIdentifier()));
				if (wevent.getNodeIdentifier().equals(thisNode.getIdentifier())){
					pendingEvent.remove(wevent.getEvent()) ; // removePendingEvent
					aboutWeightEvents.put(thisNode.getIdentifier(), wevent) ;
				}
			}
			this.events = aboutWeightEvents ;
			affectMap(); // affect
		}
		writeNode() ;
	}

	
	
	@Override
	protected void checkOutEvent(Event event, NodeVersion version) throws ALRepositoryException {
		Node beforeNode = nodeMap.get(version.getIdentifier()).newClone(version.getSession());
		
		addNewVesion(beforeNode, version) ;
		put(WeightEvent.updateEvent(event, version)) ; // getBaseNode() ?
	}

	private void addNewVesion(Node beforeNode, NodeVersion newVersion) throws ALRepositoryException {
		List<NodeVersion> entry = null ;
		if (versions.containsKey(newVersion.getIdentifier())) {
			entry = versions.get(newVersion.getIdentifier());
		} else {
			entry = new ArrayList<NodeVersion>();
		}
		VersionImpl beforeVersion = VersionImpl.make(beforeNode);
		beforeVersion.setVersionNum(beforeNode.getVersionNum()+1) ;
		entry.add(beforeVersion);
		versions.put(beforeNode.getIdentifier(), entry) ;
	}

	
	@Override
	protected void createLinkEvent(Event event, Link link) throws ALRepositoryException {
		put(WeightEvent.updateEvent(event, link.getFrom())) ;
	}

	@Override
	protected void createNodeEvent(Event event, INode newNode) throws ALRepositoryException {
		put(WeightEvent.createEvent(event, newNode)) ;
	}

	@Override
	protected void changePropertyEvent(Event event, Property property) {
		put(WeightEvent.updateEvent(event, property.getNode())) ;
	}

	@Override
	protected void createPropertyEvent(Event event, Property property) {
		put(WeightEvent.createEvent(event, property.getNode())) ;
	}

	@Override
	protected void delLinkEvent(Event event, Link link) throws ALRepositoryException {
		put(WeightEvent.updateEvent(event, link.getFrom())) ;
	}

	@Override
	protected void delNodeEvent(Event event, Node node) throws ALRepositoryException {
		put(WeightEvent.deleteEvent(event, node)) ;
	}
	
	private void removeNode(Node node) throws ALRepositoryException{
		for (Node child : node.getChildNode()) {
			removeNode(child);
		}
		nodeMap.remove(node.getIdentifier()) ;
	}
	

	@Override
	protected void moveNodeEvent(Event event, Node node) throws ALRepositoryException {
		put(WeightEvent.updateEvent(event, node)) ;
	}

	@Override
	protected void updateNodeEvent(Event event, INode node) throws ALRepositoryException {
		put(WeightEvent.updateEvent(event, node)) ;
	}
	
	
	
	private void put(WeightEvent wevent){
		WeightEvent savedEvent = events.get(wevent.getNodeIdentifier()) ;
		if (savedEvent == null || savedEvent.isLess(wevent)){
			events.put(wevent.getNodeIdentifier(), wevent) ;
		}
	}
	
	private void writeNode() throws ALRepositoryException {
		try {
			List<Node> nodes = new ArrayList<Node>(nodeMap.values()) ;
			fos.writeNode(nodes) ;
		} catch (IOException ex) {
			throw new ALRepositoryException(ex);
		} catch (InterruptedException e) {
			throw new ALRepositoryException(e);
		}
	}

	private void affectMap() throws ALRepositoryException {
		for (WeightEvent event : events.values()) {
			if (event.isDelete()){
				removeNode(event.getNode()) ;
			} else if (event.isUpdate()){
				nodeMap.put(event.getNodeIdentifier(), event.getNode()) ;
			} else if (event.isCreate()) {
				nodeMap.put(event.getNodeIdentifier(), event.getNode()) ;
			}
		}
	}

	private Map<String, Node> readNodes() throws ALRepositoryException {
		try {
			Collection<Node> nodes = fos.readNode() ;
			Map<String, Node> result = new HashMap<String, Node>() ;
			for (Node node : nodes) {
				result.put(node.getIdentifier(), node) ;
			}
			return result;
		} catch (ClassNotFoundException e) {
			throw new ALRepositoryException(e);
		} catch (IOException e) {
			throw new ALRepositoryException(e);
		} catch (InterruptedException e) {
			throw new ALRepositoryException(e);
		}
	}

}

class WeightEvent{
	
	private int weight ;
	private Event event ;
	private Node target ;
	
	final static int CREATE_WEIGHT = 1 ;
	final static int UPDATE_WEIGHT = 2 ;
	final static int DELETE_WEIGHT = 3 ;
	
	private WeightEvent(int weight, Event event, INode node) {
		this.weight = weight ;
		this.event = event ;
		this.target = (Node)node ;
	}
	
	final static WeightEvent createEvent(Event event, INode node) {
		return new WeightEvent(CREATE_WEIGHT, event, node) ;
	}
	final static WeightEvent updateEvent(Event event, INode node) {
		return new WeightEvent(UPDATE_WEIGHT, event, node) ;
	}
	final static WeightEvent deleteEvent(Event event, INode node) {
		return new WeightEvent(DELETE_WEIGHT, event, node) ;
	}
	
	public boolean isDelete(){
		return weight == DELETE_WEIGHT ;
	}
	public boolean isUpdate(){
		return weight == UPDATE_WEIGHT ;
	}
	public boolean isCreate(){
		return weight == CREATE_WEIGHT ;
	}
	
	public Node getNode(){
		return target ;
	}
	
	public Event getEvent() {
		return event ;
	}
	
	public String getNodeIdentifier(){
		return target.getIdentifier() ;
	}
	
	public boolean isLess(WeightEvent other){
		return this.weight <= other.weight ; 
	}
	
	public String toString(){
		return event + "[weight:" + weight +  "] : " + target ;
	}
	
}