package com.bleujin.lore.core.dimpl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.lock.DefaultReadsWriteLock;
import com.bleujin.lore.core.common.lock.ReadsWriteLock;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ItemNotFoundException;
import com.bleujin.lore.core.exception.NoSuchNodeTypeException;
import com.bleujin.lore.core.exception.ShareNodeException;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.mapping.IDefaultValue;
import com.bleujin.lore.core.mapping.IValueConstraint;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeExistsException;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.persitence.NamespaceRegistry;
import com.bleujin.lore.core.persitence.Sequence;
import com.bleujin.lore.core.persitence.URI;
import com.bleujin.lore.extend.template.BindTemplate;
import com.bleujin.lore.extend.template.Template;
import com.bleujin.lore.extend.template.TemplateId;
import com.bleujin.lore.extend.template.TemplateImpl;

/**
 * 
 * Test용이라서 Thread UnSafe하다. 
 *
 */

public class TransientGlobalStore extends AbstractGlobalStore implements Serializable{


	private static final long serialVersionUID = 5114106617619313453L;

	private NamespaceRegistry namespace ;
	
	private Map<TemplateId, Template> templates = new HashMap<TemplateId, Template>() ;
	private Map<String, TemplateId> templateBind = new HashMap<String, TemplateId>() ;
	private Map<String, URI> namespaceStore = new HashMap<String, URI>() ;
	private Map<String, ShareInfo> shares = new HashMap<String, ShareInfo>() ;
	private Map<String, LinkType> storeLinkType = new HashMap<String, LinkType>(); // name, linkType
	private Set<NodeType> nodetypes = new HashSet<NodeType>() ;
	private Sequence sequence = TransientSequence.getInstance() ;
	private transient ReadsWriteLock rwlock = null ;

	private String saveFileName ;
	private TransientGlobalStore(String fileName){
		this.saveFileName = fileName ;
		this.namespace = new NamespaceRegistry(this) ;
		this.rwlock = new DefaultReadsWriteLock() ;
	}
	
	public final static TransientGlobalStore create(String saveFileName) throws IOException, ClassNotFoundException{
		return readThis(saveFileName) ;
	}
	

	@Override
	public NewNodeType createNodeType(NodeType superType, String ntId, PropertyDefinition[] definitions) throws ALRepositoryException {
		return newNodeType(this, superType, ntId, definitions) ;
    }
	
	public GenericIterator<NodeType> register(NewNodeType[] newNodeTypes, boolean allowUpdate) throws ALRepositoryException {
		Set<NodeType> readTypes = readNodeTypes() ;

		List<NodeType> result = new ArrayList<NodeType>() ;
		for (NewNodeType newNodeType : newNodeTypes) {
			NodeType newType = NodeTypeImpl.create(this, newNodeType);
			if (allowUpdate == false && readTypes.contains(newType)) throw NodeTypeExistsException.create(newNodeType.getName()) ;
			result.add(newType) ;
		}
		for (NodeType newType : result) {
			readTypes.remove(newType) ;
			readTypes.add(newType) ;
		}
		
		saveThis() ;
		return new GenericIteratorHelper<NodeType>(result) ;
	}

	
	@Override
	public GenericIterator<MappedDefinition> getMappedDefinition(NodeType nodetype) throws ALRepositoryException{
		NodeType readType = readType(nodetype.getId()) ;
		return new GenericIteratorHelper<MappedDefinition>(readType.getMappedDefinitions()) ;
	}

	
	@Override
	public boolean includeNodeType(String ntId) throws ALRepositoryException {
		Set<NodeType> readTypes = readNodeTypes() ;
		final NodeTypeId nodetypeId = toNodeTypeId(ntId) ;
		for (NodeType nodetype : readTypes) {
			if (nodetypeId.equals(nodetype.getId())) return true ;
		}
		return false ;
    }

	@Override
	public GenericIterator<NodeType> getAllNodeTypes(String prefixId) throws ALRepositoryException {
		List<NodeType> result = new ArrayList<NodeType>() ;
		Set<NodeType> readTypes = readNodeTypes() ;

		for (NodeType nodetype : readTypes) {
			if (StringUtil.isBlank(prefixId) || nodetype.getId().getPrefix().equals(prefixId)){
				NodeTypeImpl.load(this, (NodeTypeImpl)nodetype) ;
				result.add(nodetype) ;
			}
		}
		return new GenericIteratorHelper<NodeType>(Collections.unmodifiableCollection(result));
	}

	@Override
	public LinkType getLinkType(String linkTypeId) {
		LinkType linkType = LinkTypeImpl.load(linkTypeId);
		storeLinkType.put(linkType.getString(), linkType); // 대체되어도 상관없음. 필요한건 단지 name뿐..
		
		return linkType;
    }

	
	public void setMemberType(NodeType nodetype, NodeType memberType) throws ALRepositoryException {
		reaplaceNodetype(nodetype);
	}
	
	@Override
	public void setPrimaryProperty(NodeType nodetype, PropertyDefinition pd) throws ALRepositoryException {
		reaplaceNodetype(nodetype);
	}
	
	
	@Deprecated
	private GenericIterator<LinkType> getAllLinkTypes() {
		return new GenericIteratorHelper<LinkType>(storeLinkType.values());
    }

	protected NodeType getNodeType(AbstractGlobalStore globalStore, NodeTypeId ntId) throws ALRepositoryException {
		if (ntId == null || StringUtil.isBlank(ntId.getString()) || "*".equals(ntId.getShortId())) return NodeType.NO_SUPERTYPE ; // top
		
		return readType(ntId) ;
	}

	@Override
	public void removeNodeType(String ntId) throws ALRepositoryException {
		Set<NodeType> readTypes = readNodeTypes() ;
		final NodeTypeId nodetypeId = toNodeTypeId(ntId) ;
		for (NodeType nodetype : readTypes) {
			if (nodetypeId.equals(nodetype.getId())) readTypes.remove(nodetype) ;
		}

		writeNodeType(readTypes) ;
	    // TODO : 노드는 어떻게????
    }

	@Override
	public void setPropertyConstraint(NodeType nodetype, PropertyDefinition prop, IValueConstraint constraint) throws ALRepositoryException {
		reaplaceNodetype(nodetype);
	}

	@Override
	public void setPropertyDefaultValue(NodeType nodetype, PropertyDefinition prop, IDefaultValue defaultValue) throws ALRepositoryException {
		reaplaceNodetype(nodetype);
	}
	
	
	private synchronized void writeNodeType(Set<NodeType> nodetypes) throws ALRepositoryException {
		this.nodetypes = nodetypes ;
		saveThis() ;
	}

	private synchronized Set<NodeType> readNodeTypes() throws ALRepositoryException {
		return nodetypes ;
	}
	
	private NodeType readType(NodeTypeId ntid) throws ALRepositoryException {
		for(NodeType nodetype : nodetypes){
			if (ntid.equals(nodetype.getId())) {
				NodeTypeImpl.load(this, (NodeTypeImpl)nodetype) ;
				return nodetype ;
			}
		}
		throw NoSuchNodeTypeException.create(ntid.getString());
	}
	
	private void reaplaceNodetype(NodeType nodetype) throws ALRepositoryException {
		Set<NodeType> readTypes = readNodeTypes() ;
		readTypes.remove(nodetype) ;
		readTypes.add(nodetype) ;
		writeNodeType(readTypes) ;
	}


	public void confirm(){
		
	}
	
	public NamespaceRegistry getNamespace(){
		return namespace ;
	}
	
	
	@Override
	public void clearStore() throws ALRepositoryException {
		File saveFile = new File(saveFileName) ;
		if (saveFile.exists()) saveFile.delete();
		
		this.templates =  new HashMap<TemplateId, Template>() ;
		this.templateBind = new HashMap<String, TemplateId>() ;
		this.namespaceStore = new HashMap<String, URI>() ;
		this.shares = new HashMap<String, ShareInfo>() ;
		this.storeLinkType = new HashMap<String, LinkType>(); // name, linkType
		this.nodetypes = new HashSet<NodeType>();
		this.sequence = TransientSequence.getInstance() ;
		
	}

	@Override
	public Sequence getSequence() {
		return sequence ;
	}

	@Override
	public ShareInfo getShareInfo(String shareName) throws ItemNotFoundException {
		if (! shares.containsKey(shareName)) throw new ShareNodeException("execption.sharenode.not_found_shareInfo[" + shareName + "]") ;
		return shares.get(shareName) ;
	}

	@Override
	public GenericIterator<ShareInfo> getShareInfos() {
		return new GenericIteratorHelper<ShareInfo>(shares.values());
	}

	
	@Override
	public void shareNode(String shareName, String workspaceName, String nodePath, boolean enableWrite) {
		if(shares.containsKey(shareName)) throw new ShareNodeException("execption.sharenode.already_exist[" + shareName + "]") ;
		shares.put(shareName, ShareInfo.create(shareName, workspaceName, nodePath, enableWrite)) ;
		saveThis() ;
	}

	
	@Override
	public Template saveTemplate(TemplateId tplId, Template template) {
		templates.put(tplId, template) ;
		saveThis() ;
		return template;
	}

	@Override
	public Template getTemplate(UserSession session, TemplateId createId) throws ALRepositoryException {
		Template template = templates.get(createId);
		return TemplateImpl.load(session, createId, template.getContent()) ;
	}

	@Override
	public GenericIterator<TemplateId> getTemplates(UserSession session) throws ALRepositoryException {
		return new GenericIteratorHelper<TemplateId>(templates.keySet());
	}

	@Override
	public Template getBoundTemplate(Node node) throws ALRepositoryException {
		TemplateId tplId = null ;
		if (templateBind.containsKey(node.getIdentifier())) {
			tplId = templateBind.get(node.getIdentifier()) ;
		} else {
			for (NodeType nodetype = node.getNodeType() ; nodetype != NodeType.NO_SUPERTYPE; nodetype = nodetype.getSuperType()){
				if (templateBind.containsKey(nodetype.getId().getString())){
					tplId = templateBind.get(nodetype.getId().getString()) ;
				}
			}
		}
		
		Template foundTemplate = templates.get(tplId);
		if (foundTemplate != null) {
			return BindTemplate.create(node, tplId, foundTemplate.getContent()) ;
		} else {
			return BindTemplate.createDefault(node) ;
		}
	}

	@Override
	public void bindTemplate(Node node, TemplateId tplId) {
		templateBind.put(node.getIdentifier(), tplId) ;
		saveThis() ;
	}

	@Override
	public void bindTemplate(NodeType nodeType, TemplateId tplId) {
		templateBind.put(nodeType.getName(), tplId) ;
		saveThis() ;
	}

	public void close() {
		saveThis() ;
	}

	protected  int addURIAddress(String prefix, String address) {
		if (existURIAddress(prefix)) return 0 ;
		namespaceStore.put(prefix, new URI(prefix, address)) ;
		saveThis() ;
		return 1;
	}

	protected  void removeURIAddress(String prefix) {
		namespaceStore.remove(prefix) ;
		saveThis() ;
	}

	protected  boolean existURIAddress(String prefix) {
		return namespaceStore.containsKey(prefix);
	}

	protected  URI getURIAddress(String prefix) {
		return namespaceStore.get(prefix);
	}

	protected  GenericIterator<URI> listURIAddress() {
		return new GenericIteratorHelper<URI>(namespaceStore.values());
	}

	
	private void saveThis() {
		try {
			try {
				rwlock.writeLock() ;
				File saveFile = new File(saveFileName) ;
				FileOutputStream fos = new FileOutputStream(saveFile) ;
				ObjectOutputStream oos = new ObjectOutputStream(fos) ;
				oos.writeObject(this) ;
				oos.close() ;
				fos.close() ;
			} finally {
				rwlock.writeUnLock() ;
			}
		} catch (InterruptedException ignore) {
			ignore.printStackTrace();
		} catch (IOException ignore) {
			ignore.printStackTrace();
		}
	}
	
	private static TransientGlobalStore readThis(String fileName) throws FileNotFoundException, IOException, ClassNotFoundException{

		File file = new File(fileName) ;
		if (! file.getParentFile().exists()) file.getParentFile().mkdirs() ;
		if (! file.exists()) file.createNewFile() ;
		
		Debug.happy(file.getAbsolutePath());
		if (file.length() == 0){
			return new TransientGlobalStore(fileName) ;
		}else {
			ObjectInputStream input = new ObjectInputStream(new FileInputStream(file)) ;
			TransientGlobalStore tgs = (TransientGlobalStore)input.readObject() ;
			tgs.rwlock = new DefaultReadsWriteLock() ;
			return tgs ;
		}
	}
	

}
