package com.bleujin.lore.core.dimpl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.ecs.xml.XML;

import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.bean.Rating;
import com.bleujin.lore.core.bean.Register;
import com.bleujin.lore.core.bean.SourceFrom;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ALRepositoryRuntimeException;
import com.bleujin.lore.core.exception.NoSuchPropertyDefinitionException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.mapping.ValidMessage;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Lock;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.ShareNode;
import com.bleujin.lore.core.node.VersionHistory;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.observation.LinkEvent;
import com.bleujin.lore.core.observation.NodeEvent;
import com.bleujin.lore.core.observation.PropertyEvent;
import com.bleujin.lore.core.observation.UserBean;
import com.bleujin.lore.core.observation.Event.EventType;
import com.bleujin.lore.core.observation.Event.Mode;
import com.bleujin.lore.core.persitence.GlobalStore;
import com.bleujin.lore.core.persitence.ILocalStore;
import com.bleujin.lore.core.persitence.NodeBean;
import com.bleujin.lore.core.persitence.Sequence;
import com.bleujin.lore.core.search.searcher.NodeSearcher;
import com.bleujin.lore.core.session.SessionContext;
import com.bleujin.lore.core.session.SimpleSession;
import com.bleujin.lore.core.valid.ValidChain;
import com.bleujin.lore.core.value.Value;
import com.bleujin.lore.core.workspace.LockManager;
import com.bleujin.lore.core.workspace.ObservationManager;
import com.bleujin.lore.core.workspace.Workspace;
import com.bleujin.lore.extend.template.TemplateManager;
import com.bleujin.lore.extend.template.TemplateManagerImpl;

public class UserSession extends SimpleSession{

	// public static UserSession EMPTY_SESSION = new UserSession(null, "N/A", );
	// static {
	// EMPTY_SESSION.setUserId("bleujin");
	// EMPTY_SESSION.setIpAddress("127.0.0.1");
	// EMPTY_SESSION.setConnDate(DateUtil.currentdate2String()) ;
	// EMPTY_SESSION.setLangCd("ko");
	// }

	private static final long serialVersionUID = 4624342269823865162L;
	private Workspace workspace ;
	private ObservationManager observationManager ;
	private LockManager lockManager ;
	private ValidChain vchain ;
	private SessionStore store ;
	private UserBean userBean ;
	private NodeTypeManager nodeTypeManager ;
	private TemplateManager templateManager ;

	private UserSession(Workspace workspace, SessionContext context, String sessionId, UserBean userBean) {
		super(context, sessionId, 10*60) ;
		this.workspace = workspace ;
		this.observationManager = workspace.getObservationManager();
		this.vchain = workspace.getValidChain() ;
		this.userBean = userBean ;

		this.store = new SessionStore(workspace.getLocalStore(), userBean);
		this.lockManager = workspace.getLockManager() ;
		this.nodeTypeManager = new NodeTypeManagerImpl(workspace.getGlobalStore()) ;
		this.templateManager = TemplateManagerImpl.create(this, workspace.getGlobalStore()) ;
	}

	public final static UserSession create(Workspace workspace, SessionContext context, String sessionId, UserBean userBean){
		return new UserSession(workspace, context, sessionId, userBean) ;
	}
	
	// inDirect Access

	public NodeTypeManager getNodeTypeManager(){
		return nodeTypeManager;
	}
	public TemplateManager getTemplateManager() {
		return templateManager ;
	}
	public Sequence getSequence() {
		return getGlobalStore().getSequence();
	}
	
	
	
	public LinkType createLinkType(String linkTypeName) throws ALRepositoryException {
		return getGlobalStore().createLinkType(linkTypeName);
	}
	public Link createLink(Node from, Node to, LinkType linkType, Position position) throws ALRepositoryException {
		Link link = store.createLink(this, from, to, linkType, position) ;
		
		fireEvent(link, EventType.LINK_ADDED);
		loadMemory(from) ;
		return link ;
	}

	
	

	public INode createNode(Node parent, NodeType nodetype, String name) throws ALRepositoryException {
		return createNode(parent, nodetype, NodeName.nodeName(name), "") ;
	}

	public INode createNode(Node parent, NodeType nodetype, String name, String explain) throws ALRepositoryException {
		return createNode(parent, nodetype, NodeName.nodeName(name), explain) ;
	}

	public INode createNode(Node parent, NodeType nodetype, NodeName name, String explain) throws ALRepositoryException {
		INode newNode = store.createNode(this, parent, nodetype, name, explain) ;
		
		return newNode;
	}

	public void delNode(Node node) throws ALRepositoryException {
		store.delNode(node) ;
		fireEvent(node, EventType.NODE_REMOVED) ;
		loadMemory(node) ;
	}
	
	
	public ShareNode getShareNode(String shareName) throws ALRepositoryException{
		return workspace.getShareNode(this, shareName) ;
	}

	public GenericIterator<ShareNode> getShareNodes() throws ALRepositoryException{
		return workspace.getShareNodes(this) ;
	}

	
	public Node getRootNode() {
		return new RootNode(this) ;
	}
	
	public Node getNode(String nodeInfo) throws ALRepositoryException{
		if (nodeInfo == null) return getRootNode() ; // root
		if (StringUtil.isEmpty(nodeInfo)) return getRootNode() ; // root
		if (nodeInfo.equals(getWorkspace().getName())) return getRootNode() ;
		if (StringUtil.isAlphaUnderbarStartWith(nodeInfo) && (nodeInfo.indexOf("/") == -1) && (nodeInfo.indexOf("-") == -1)) return getRootNode() ;
		if (nodeInfo.startsWith(AbstractNode.ID_BEGIN_MARK) && nodeInfo.endsWith(AbstractNode.ID_END_MARK)) return getNodeByUUID(StringUtil.substringBetween(nodeInfo, AbstractNode.ID_BEGIN_MARK, AbstractNode.ID_END_MARK)) ;
		if (nodeInfo.startsWith("/")) return getNodeByPath(nodeInfo);
		return getNodeByUUID(nodeInfo) ;
	}

	private Node getNodeByPath(String path) throws ALRepositoryException {
		return store.getNodeByPath(this, path);
	}

	private Node getNodeByUUID(String uuid) throws ALRepositoryException {
		return store.getNodeByUUID(this, uuid) ;
	}
	
	public boolean existByPath(String path) throws ALRepositoryException{
		return store.exist(this, path);
	}

	public void refresh(boolean keepChanges) throws ALRepositoryException{
		store.refresh(keepChanges) ;
	}
	
	public boolean hasPendingChanges(){
		return store.hasPendingChanges() ;
	}
	
	public void save(Node node) throws ALRepositoryException {
		store.save(node) ;
	}
	
	public void save() throws ALRepositoryException {
		store.save() ;
	}

	//
	public void logout() throws ALRepositoryException {
		logout(false);
	}
	
	
	// session timeout
	public void removeAttributeAtTimeout(){
		super.removeAttributeAtTimeout() ;
		try {
			lockManager.releaseOwnerLock(this.userBean) ;
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}


	public void logout(boolean toFlush) throws ALRepositoryException {
		invalidate() ;
		try {
			lockManager.releaseOwnerLock(this.userBean) ;
			if (toFlush) {
				store.save() ;
			}
			// this.store = new SessionStore(workspace.getLocalStore(), userBean);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	public boolean isLive() {
		return true;
	}

	public UserBean getUserBean() {
		return userBean;
	}

	public String getUserId(){
		return userBean.getUserId() ;
	}

	public Workspace getWorkspace() {
		return this.workspace ;
	} 
	
	private GlobalStore getGlobalStore() {
		return workspace.getGlobalStore() ;
	}

	
	public ILocalStore getLocalStore() {
		return store.getLocalStore();
	}
	
	public NodeSearcher getSearcher() {
	    return store.getSearcher(this, getRootNode()) ;
    }

	public NodeSearcher getSearcher(Node startNode) {
	    return store.getSearcher(this, startNode) ;
    }


	public void checkOut(NodeVersion version) throws ALRepositoryException {
		store.checkOut(version) ;
		fireEvent(version, EventType.NODE_CHECKOUTED);
	}

	// lock 
	public Lock lock(Node node, boolean isDeep) throws ALRepositoryException{
		try {
			return lockManager.addLock(node, isDeep);
		} catch (InterruptedException e) {
			throw new ALRepositoryException(e) ;
		}
	}

	public boolean isLocked(Node node) throws ALRepositoryException {
		try {
			return lockManager.isLocked(node);
		} catch (InterruptedException e) {
			throw new ALRepositoryException(e) ;
		}
    }

	public Lock getLock(Node node) throws ALRepositoryException {
		try {
			return lockManager.getLock(node);
		} catch (InterruptedException e) {
			throw new ALRepositoryException(e) ;
		}
    }
	
	

	private boolean isValid(Node node) throws ALRepositoryException {
		return vchain.isValid(node);
	}
	

	// event
	
	public void fireEvent(INode node, EventType eventType) {
		fireEvent(createEvent(node, eventType));
	}

	public void fireEvent(Link link, EventType eventType) {
		fireEvent(createEvent(link, eventType));
	}

	public void fireEvent(Property property, EventType eventType) {
		fireEvent(createEvent(property.getNode(), property, eventType));
	}

	public void fireEvent(Event event) {
		store.handle(event) ;
		observationManager.fireEvent(event);
	}

	private NodeEvent createEvent(INode node, EventType eventType) {
		return new NodeEvent(node, eventType, userBean);
	}

	private LinkEvent createEvent(Link link, EventType eventType) {
		return new LinkEvent(link, eventType, userBean);
	}

	private PropertyEvent createEvent(Node node, Property property, EventType eventType) {
		return new PropertyEvent(node, property, userBean, eventType);
	}

	


	public boolean hasPermission(String absPath, String actions) throws ALRepositoryException{
		return getWorkspace().isAllowed(this.userBean, getNode(absPath), actions) ;
	}


	public String toString(){
		return getClass().getSimpleName() + "[" + userBean.toString() + ", workspace:" + workspace.getName() + "]" ;
	}
	
	
	// case delete, checkout, move etc..
	public void loadMemory(Node node) {
		store.loadMemory(node) ;
	}
	SessionStore getSessionStore(){
		return store ;
	}
	
	// testOnly
	public void confirmStore(){
		store.confirm() ;
	}

	public GenericIterator<Node> getNodes(NodeType nodetype, boolean includeSub, Page page) throws ALRepositoryException {
		return store.getNodes(this, nodetype, includeSub, page) ;
		
	}

	Value[] getValues(Node node, Property property) throws ALRepositoryException {
		return store.getValues(node, property) ;
	}



	
	// only test
	public void printPendingEvent(){
		Debug.debug(this, store.getPendingEvent());
	}




}




final class RootNode extends AbstractNode{
	

	private transient UserSession session ;
	private final String exceptionMessage = "exception.node.rootnode.ghost";
	
	RootNode(UserSession session){
		this.session = session ;
	}
	
	final NodeBean rootNodeBean = new NodeBean(){
 
		private static final long serialVersionUID = -8121212949745579811L;

		public void addRating(Rating rating) {}
		public void setExplain(String explain){}
		public String getExplain() {return "GHOST ROOT";}
		public String getIdentifier() {
			if (session == null) throw new IllegalArgumentException("exception.session.illegal_state[null]");
			return session.getWorkspace().getName();
		}
		
		public void setName(NodeName name){
			; // ignore
		}
		public NodeName getName() {return NodeName.ROOT_NAME;}

		public NodeTypeId getNodeTypeId() {
			return NodeTypeIdImpl.NO_SUPERTYPE_ID ;
		}

		public Register getRegister() {
			return Register.EMPTY;
		}

		public SourceFrom getSourceFrom() {
			return SourceFrom.EMPTY;
		}

		public String getStatus() {
			return "";
		}

		public long getViewCount() {
			return 0;
		}

		public void setRegister(Register register) {
			; // ignore
		}

		public void setSourceFrom(SourceFrom from) {
			; // ignore
		}

		public XML toXML() throws ALRepositoryException {
			XML result = new XML(XML_TAG_ROOT) ;
			return result ;
		}

		public String getWorkspaceName() {
			return session.getWorkspace().getName();
		}
		public void increaseViewCount() {

		}
		public NodeBean newClone() {
			return this; // ... 
		}
	
	} ;
	
	private static final long serialVersionUID = 7289261999281299266L;
	
	public void setParent(Node parent){
		throw new ALRepositoryRuntimeException(exceptionMessage) ;
	}
	
	public void addLink(Node target, LinkType linkType, Position position) throws ALRepositoryException {
		Link link = getSession().createLink(this, target, linkType, position) ;
	}
	public void addMemberNode(Node member) throws ALRepositoryException {
		throw new ALRepositoryRuntimeException(exceptionMessage) ;
	}
	public NodeVersion checkIn() throws ALRepositoryException {
		throw new ALRepositoryRuntimeException(exceptionMessage) ;
	}
	public void copy(Node parent) throws ALRepositoryException {
		throw new ALRepositoryRuntimeException(exceptionMessage) ;
	}
	public void delete() throws ALRepositoryException {
		throw new ALRepositoryRuntimeException(exceptionMessage) ;
	}
	public NodeBean getBean() {
		return rootNodeBean;
	}
	public GenericIterator<Node> getDescentNode(int maxDepth) throws ALRepositoryException {
		List<Node> result = new ArrayList<Node>();
		retrieveChildNodeTo(this, result, maxDepth);

		return new GenericIteratorHelper<Node>(Collections.unmodifiableCollection(result));
	}
	public GenericIterator<Node> getChildNode(Page page) throws ALRepositoryException {
		return getLocalStore().getChildNodes(this, page) ;
	}

	private void retrieveChildNodeTo(Node parent, List target, int maxDepth) throws ALRepositoryException {
		if (maxDepth <= 0)
			return;
		
		GenericIterator<Node> iter = parent.getChildNode(Page.ALL);
		
		for (Node child : iter) {
			if (child == null) continue ; // not saved node
			if (child.getMode().isDeleted()) continue ;
			
			target.add(child);
			retrieveChildNodeTo(child, target, --maxDepth);
		}
	}	
	
	public GenericIterator<Link> getLinks(Page page) throws ALRepositoryException {
		return getLinks(null, page) ;
	}
	public GenericIterator<Link> getLinks(LinkType linkType, Page page) throws ALRepositoryException {
		Debug.debug("exception.node.rootnode.illegal_access[getlink]") ;
		return new GenericIteratorHelper<Link>(new ArrayList<Link>()) ;
	}

	public Lock getLock() throws ALRepositoryException {
		// TODO Auto-generated method stub
		return null;
	}
	public GenericIterator<Node> getMemberNode(Page page) throws ALRepositoryException {
		return new GenericIteratorHelper<Node>(new ArrayList<Node>());
	}
	public Mode getMode() {
		return Mode.NORMAL;
	}
	public String getName() {
		return NodeName.ROOT_NAME.getString();
	}
	public NodeType getNodeType(){
		return NodeType.NO_SUPERTYPE;
	}
	public GenericIterator<Node> getNodes(LinkType type, Page page) throws ALRepositoryException {
		return session.getLocalStore().getNodes(this, type, page) ;
	}
	
	public Node getParent() throws ALRepositoryException {
		throw new ALRepositoryRuntimeException(exceptionMessage) ;
	}
	public String getPath() throws ALRepositoryException {
		return "";
	}
	public GenericIterator<Property> getProperties() throws ALRepositoryException {
		return new GenericIteratorHelper<Property>(new ArrayList<Property>()) ;
	}
	public Property getProperty(IDString propId) throws ALRepositoryException {
		throw NoSuchPropertyDefinitionException.create(propId) ;
	}
	public UserSession getSession() {
		return session;
	}
	public String getString(IDString propId) throws ALRepositoryException {
		throw NoSuchPropertyDefinitionException.create(propId) ;
	}
	public VersionHistory getVersionHistory() throws ALRepositoryException {
		return new TransientVersionHistory(new ArrayList<NodeVersion>(), this) ;
	}
	public int getVersionNum() {
		return 0;
	}
	public long getViewCount() {
		return 0;
	}
	public GenericIterator<ValidMessage> getViolateConstraintMessage() throws ALRepositoryException {
		return new GenericIteratorHelper<ValidMessage>(new ArrayList<ValidMessage>());
	}
	public boolean hasProperties() throws ALRepositoryException {
		return false;
	}
	public boolean hasProperty(IDString propId) throws ALRepositoryException {
		return false;
	}
	public boolean isLocked() throws ALRepositoryException {
		return false;
	}
	public boolean isRoot() {
		return true;
	}

	public Lock lock(boolean isDeep) throws ALRepositoryException {
		throw new ALRepositoryException("exception.node.rootnode.illegal_access[lock]");
	}
	public void move(Node parent) throws ALRepositoryException {
		throw new ALRepositoryException("exception.node.rootnode.illegal_access[move]");
	}
	public Node save() throws ALRepositoryException {
		; // ignore
		return this ;
	}
	public GenericIterator<Node> searchNode(String searchText, Page page) throws ALRepositoryException {
		return getSession().getLocalStore().searchNode(session, session.getRootNode(), searchText, page) ;
	}
	public void setMode(Mode mode) {
		; // ignore
	}
	public void setProperty(IDString propId, Value value) throws ALRepositoryException {
		throw NoSuchPropertyDefinitionException.create(propId) ;
	}
	
	public Node newClone(UserSession session) {
		this.session = session ;
		return this;
	}

	public void setProperties(Map<IDString, Property> properties) {
		; // ignore
	}
	public void setDefaultProperty() throws ALRepositoryException {
		; // ignore
	}

}
