package com.bleujin.lore.core.dimpl;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import org.apache.ecs.xml.XML;

import com.bleujin.lore.core.bean.Rating;
import com.bleujin.lore.core.bean.Register;
import com.bleujin.lore.core.bean.SourceFrom;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.IllegalNodeNameException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.mapping.ValidMessage;
import com.bleujin.lore.core.node.Lock;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.VersionHistory;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.observation.Event.Mode;
import com.bleujin.lore.core.persitence.NodeBean;
import com.bleujin.lore.core.value.Value;

public class VersionDBImpl extends AbstractNode implements NodeVersion {
 
	private static final long serialVersionUID = -7028117640210956856L;
	private Node baseNode ;
	private int versionNum ;
	private NodeBean nodeBean ;
	private Map<IDString, Property> properties ;
	private VersionDBImpl(final Node node, int versionNum, final Register register) {
		this.baseNode = node ;
		this.versionNum = versionNum ;
		this.properties = new HashMap<IDString, Property>() ;
		this.nodeBean = new NodeBean(){
 
			private static final long serialVersionUID = 7781164957328442382L;

			public void addRating(Rating rating) {
				node.addRating(rating) ;
			}
			
			public void setName(NodeName name){
				((AbstractNode)node).setName(name) ;
			}

			public NodeName getName() {
				try {
					return NodeName.nodeName(node.getName());
				} catch (IllegalNodeNameException e) {
					throw new IllegalArgumentException(e) ;
				}
			}

			public String getExplain() {
				return node.getExplain();
			}
			public void setExplain(String explain){
				node.setExplain(explain) ;
			}

			public NodeTypeId getNodeTypeId(){
				return node.getNodeTypeId() ;
			}

			public Register getRegister() {
				return register;
			}

			public SourceFrom getSourceFrom() {
				return node.getSourceFrom();
			}

			public String getStatus() {
				return node.getStatus();
			}

			public String getIdentifier() {
				return node.getIdentifier();
			}

			public long getViewCount() {
				return node.getViewCount();
			}

			public void setRegister(Register register) {
				node.setRegister(register) ;
			}

			public void setSourceFrom(SourceFrom from) {
				node.setSourceFrom(from) ;
			}

			public XML toXML() throws ALRepositoryException {
				XML result = new XML(XML_TAG_ROOT) ;
				result.addAttribute(XML_TAG_NAME, getName().getString()) ;
				result.addAttribute(XML_TAG_EXPLAIN, getExplain() == null ? "" : getExplain()) ;
				result.addAttribute(XML_TAG_EXPLAIN, getIdentifier()) ;
				result.addAttribute(XML_TAG_REGISTER, getRegister().toString()) ;
				result.addAttribute(XML_TAG_VISIT, String.valueOf(getViewCount())) ;

				return result ;
			}

			public String getWorkspaceName() {
				return node.getWorkspaceName();
			}
			public void increaseViewCount() {

			}

			public NodeBean newClone() {
				return nodeBean; // TODO.
			}

		} ;
	}

	public static VersionDBImpl load(Node node, int versionNum, Register register) {
		return new VersionDBImpl(node, versionNum, register) ;
	}

	public void setParent(Node parent) throws ALRepositoryException {
		throw new UnsupportedOperationException("exception.node.version.readonly[checkOut]") ;
	}
	
	public void checkOut() throws ALRepositoryException {
		throw new UnsupportedOperationException("exception.node.version.readonly[checkOut]") ;
	}
	public NodeVersion checkIn() throws ALRepositoryException {
		throw new UnsupportedOperationException("exception.node.version.readonly[checkIn]") ;
	}

	public void delete() throws ALRepositoryException {
		throw new UnsupportedOperationException("exception.node.version.readonly[delete]") ;
	}
	
	public void copy(Node parent) throws ALRepositoryException {
		throw new UnsupportedOperationException("exception.node.version.readonly[copy]") ;
	}

	public void setProperty(IDString propId, Value value) throws ALRepositoryException {
		throw new UnsupportedOperationException("exception.node.version.readonly[setProperty]") ;
	}
	

	public int compareTo(NodeVersion that) {
		// TODO Auto-generated method stub
		return 0;
	}

	public Calendar getCreated() {
		return  baseNode.getRegister().getCreateDate();
	}

	public Node getRecentNode() throws ALRepositoryException {
		return baseNode;
	}

	
	
	
	
	

	public void addLink(Node target, LinkType linkType, Position position) throws ALRepositoryException {
		baseNode.addLink(target, linkType, position) ;
	}

	public void addMemberNode(Node member) throws ALRepositoryException {
		baseNode.addMemberNode(member) ;
	}

	public NodeBean getBean() {
		return this.nodeBean ;
	}

	public GenericIterator<Node> getDescentNode(int maxDepth) throws ALRepositoryException {
		return baseNode.getDescentNode(maxDepth);
	}

	public GenericIterator<Node> getChildNode(Page page) throws ALRepositoryException {
		return baseNode.getChildNode(page);
	}

	public GenericIterator<Link> getLinks(Page page) throws ALRepositoryException {
		return baseNode.getLinks(page);
	}

	public GenericIterator<Link> getLinks(LinkType linkType, Page page) throws ALRepositoryException {
		return baseNode.getLinks(linkType, page);
	}

	public Lock getLock() throws ALRepositoryException {
		return baseNode.getLock();
	}

	public GenericIterator<Node> getMemberNode(Page page) throws ALRepositoryException {
		return baseNode.getMemberNode(page);
	}

	public Mode getMode() {
		return baseNode.getMode();
	}

	public String getName() {
		return baseNode.getName() ;
	}

	public NodeType getNodeType() throws ALRepositoryException {
		return baseNode.getNodeType();
	}

	public GenericIterator<Node> getNodes(LinkType type, Page page) throws ALRepositoryException {
		return baseNode.getNodes(type, page);
	}

	public Node getParent() throws ALRepositoryException {
		return baseNode.getParent();
	}

	public String getPath() throws ALRepositoryException {
		return baseNode.getPath();
	}

	public UserSession getSession() {
		return baseNode.getSession();
	}

	public VersionHistory getVersionHistory() throws ALRepositoryException {
		return baseNode.getVersionHistory();
	}

	public int getVersionNum() {
		return versionNum;
	}

	public long getViewCount() {
		return baseNode.getViewCount();
	}

	public boolean isLocked() throws ALRepositoryException {
		return baseNode.isLocked();
	}

	public boolean isSame(Node that) {
		return baseNode.isSame(that);
	}

	public boolean isSameVersion(Node that) {
		return isSame(that) && this.versionNum == that.getVersionNum();
	}

	public Lock lock(boolean isDeep) throws ALRepositoryException {
		return baseNode.lock(isDeep);
	}

	public void move(Node parent) throws ALRepositoryException {
		baseNode.move(parent) ;
	}

	public Node save() throws ALRepositoryException {
		baseNode.save() ;
		return this ;
	}

	public GenericIterator<Node> searchNode(String searchText, Page page) throws ALRepositoryException {
		return baseNode.searchNode(searchText, page);
	}

	public void setMode(Mode mode) {
		baseNode.setMode(mode) ;
	}



	public GenericIterator<Property> getProperties() throws ALRepositoryException {
		loadReal() ;
		return new GenericIteratorHelper<Property>(properties.values()) ;
	}

	public Property getProperty(IDString propId) throws ALRepositoryException {
		loadReal();
		return properties.get(propId) ;
	}
	
	private boolean isLoadProperty = false ;
	private synchronized void loadReal() throws ALRepositoryException {
		if (isLoadProperty == false) {
			GenericIterator<Property> ps = getSession().getLocalStore().getProperties(this) ;
			for(Property p : ps){
				this.properties.put(p.getId(), p) ;
			}
			this.isLoadProperty = true ;
		}
	}
	
	
	public String getString(IDString propId) throws ALRepositoryException {
		return getProperty(propId).getString();
	}

	public boolean hasProperties() throws ALRepositoryException {
		return baseNode.hasProperties() ;
	}

	public boolean hasProperty(String propId) throws ALRepositoryException {
		return baseNode.hasProperty(propId) ;
	}

	public GenericIterator<ValidMessage> getViolateConstraintMessage() throws ALRepositoryException {
		return getNodeType().getViolateConstraintMessage(this) ;
	}

	public XML toXML() throws ALRepositoryException {
		XML result = new XML(VERSION) ;
		
		result.addElement( ((AbstractNode)baseNode).getBean().toXML()) ;

		GenericIterator<Property> properties = getProperties() ;
		while(properties.hasNext()){
			result.addElement(properties.next().toXML()) ;
		}
		
		
		return result ;
	}

	public boolean isRoot() {
		return getIdentifier().equals(getSession().getWorkspace().getName());
	}

	public Node newClone(UserSession session) {
		return this;
	}

	public void setDefaultProperty() throws ALRepositoryException {
		;
	}

	public NodeVersion setVersionNum(int versionNum) {
		; // in db
		return this;
	}


}
