package com.bleujin.lore.core.dimpl;

import java.util.Calendar;

import org.apache.ecs.xml.XML;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.AccessDeniedException;
import com.bleujin.lore.core.exception.ItemNotFoundException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.observation.Event.Mode;
import com.bleujin.lore.core.persitence.NodeBean;
import com.bleujin.lore.core.value.Value;

public class VersionImpl extends AbstractNode implements NodeVersion{

    private static final long serialVersionUID = -9142641851416776535L;
	private Calendar createDate;
	private boolean isCheckOut = false ;
	private AbstractNode baseNode ;
	private int versionNum ;

	private VersionImpl(Node node, Calendar createDate) {
		this.baseNode = (AbstractNode)node.newClone(node.getSession()) ;
		
		this.createDate = createDate;
		this.versionNum = node.getVersionNum() ;
	}
	
	public static VersionImpl checkIn(Node node) throws ALRepositoryException{
		VersionImpl version = new VersionImpl(node, Calendar.getInstance()) ;
		return version ;
	}
	
	public static VersionImpl make(Node node) throws ALRepositoryException {
		return new VersionImpl(node, node.getCreated()) ; 
	}
	
	

	
	public Calendar getCreated() {
		return createDate;
	}
	
	public Node getRecentNode() throws ALRepositoryException {
		return getLocalStore().getVersionHistory(this).getRecentNode();
	}

	public void checkOut() throws ALRepositoryException {
		confirmCheckOut() ;
		
		versionNum++ ;
		getSession().checkOut(this);
		this.isCheckOut = true ;
	}

	public VersionImpl setVersionNum(int versionNum){
		this.versionNum = versionNum ;
		return this ;
	}
	
	private void confirmCheckOut() {
		if (isCheckOut) new IllegalStateException("exception.version.checkout[already_checkouted]") ;
	}

	public void delete() throws ALRepositoryException {
		throw new UnsupportedOperationException("exception.version.readonly[not_deleted]") ;
	}
	
	public void copy(Node parent) throws ALRepositoryException {
		throw new UnsupportedOperationException("exception.version.readonly[copy]") ;
	}


	public GenericIterator<Property> getProperties() throws ALRepositoryException {
		return baseNode.getProperties() ;

	}

	public Property getProperty(IDString propId) throws ALRepositoryException {
		return baseNode.getProperty(propId) ;
	}

	@Override
	public Node save() throws ALRepositoryException{
		if (!isCheckOut) new IllegalStateException("checkout 되지 않아서 save 할수 없음.") ;
		baseNode.save() ;
		return this ;
	}
	
	public void setProperty(IDString propId, Value value) throws ALRepositoryException {
		confirmCheckOut() ;
		baseNode.setProperty(propId, value) ;
	}

	public int compareTo(NodeVersion that) {
		return this.getVersionNum() - that.getVersionNum() ;
    }

	public XML toXML() throws ALRepositoryException{
		XML result = super.toXML();
		result.addElement(new XML("created").addElement(DateUtil.calendarToGMTString(getCreated()))) ;
		
		return result ;
	}

	protected NodeBean getBean() {
		return baseNode.getBean();
	}

	public NodeVersion checkIn() throws ALRepositoryException {
		return baseNode.checkIn();
	}

	public GenericIterator<Node> getChildNode(Page page) throws ALRepositoryException {
		return baseNode.getChildNode(page);
	}

	public GenericIterator<Node> getDescentNode(int maxDepth) throws ALRepositoryException {
		return baseNode.getDescentNode(maxDepth);
	}

	public GenericIterator<Link> getLinks(Page page) throws ALRepositoryException {
		return baseNode.getLinks(page);
	}

	public GenericIterator<Link> getLinks(LinkType linkType, Page page) throws ALRepositoryException {
		return baseNode.getLinks(linkType, page);
	}

	public Mode getMode() {
		return baseNode.getMode();
	}

	public GenericIterator<Node> getNodes(LinkType type, Page page) throws ALRepositoryException {
		return baseNode.getNodes(type, page);
	}

	public UserSession getSession() {
		return baseNode.getSession();
	}

	public int getVersionNum() {
		return versionNum;
	}

	
	public Node newClone(UserSession session) {
		return new VersionImpl(this.baseNode.newClone(session), this.createDate) ;
	}

	public void move(Node parent) throws ALRepositoryException {
		baseNode.move(parent) ;
	}

	public void setMode(Mode mode) {
		baseNode.setMode(mode) ;
	}

	public void addLink(Node target, LinkType linkType, Position position) throws ALRepositoryException {
		baseNode.addLink(target, linkType, position) ;
	}

	public Node getParent() throws ItemNotFoundException, AccessDeniedException, ALRepositoryException {
		return baseNode.getParent();
	}

	public void setParent(Node node) throws ALRepositoryException {
		baseNode.setParent(node) ;
	}

	public void setDefaultProperty() throws ALRepositoryException {
		baseNode.setDefaultProperty() ;
	}
	
	public AbstractNode getBaseNode(){
		return baseNode ;
	}
}
