package com.bleujin.lore.core.dimpl.carrier;

import junit.framework.TestCase;

import com.bleujin.framework.db.DBController;
import com.bleujin.lore.core.dimpl.NodeTypeManager;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.workspace.AnotherLore;
import com.bleujin.lore.core.workspace.TestCredential;
import com.bleujin.lore.core.workspace.Workspace;
import com.bleujin.lore.core.workspace.config.InitLore;

public class DBStoreParentTestCase extends TestCase{
    protected static DBController dc = DBController.getTestInstance() ; ;

    protected AnotherLore anotherLore = null ; 
    protected UserSession session = null ;
    protected Workspace workspace = null ;
    protected Node root = null ;
    protected NodeType OBJECT_TYPE = null ; 
    protected NodeTypeManager ntManager = null;

    public DBStoreParentTestCase(){
    	this("noname");
    }

    public DBStoreParentTestCase(String name){
        super(name) ;
    }

    protected String TEST_NODE_NAME = "test";
    protected void setUp() throws Exception {
        super.setUp();
        dc.init();

    	anotherLore = InitLore.init() ;
    	session = anotherLore.login(new TestCredential("bleujin")) ;
    	workspace = session.getWorkspace() ;
    	ntManager = session.getNodeTypeManager() ;

    	OBJECT_TYPE = ntManager.register(NodeType.NO_SUPERTYPE, "_object", new PropertyDefinition[0]) ;
		root = createNode(session.getRootNode(), OBJECT_TYPE, TEST_NODE_NAME);
    	
    	// root.save() ;
    	session.save() ;
    }

    protected  void tearDown() throws Exception {
        super.tearDown();
        
        workspace.clearStore() ;
    	this.anotherLore = null ;
    	this.session = null ;
    	this.workspace = null ;
    	this.root = null ;
    	this.OBJECT_TYPE = null ;
    	anotherLore.destroyWorld() ;
        dc.getDBManager().destroyPoolConnection();
    }

    protected Node createNode(Node parent, NodeType nodetype, String name) throws ALRepositoryException{
    	return session.createNode(parent, nodetype, name).save() ;
    }
    
	protected PropertyDefinition getOrCreateProperty(String pid, Property.Type requireType) throws ALRepositoryException{
		return ntManager.createPropertyDefinition(pid, requireType) ;
	}

}

