package com.bleujin.lore.core.dimpl.carrier;

import com.bleujin.framework.db.DBController;
import com.bleujin.framework.db.NRows;
import com.bleujin.framework.db.RepositoryException;
import com.bleujin.framework.db.Row;
import com.bleujin.framework.db.Rows;
import com.bleujin.framework.db.procedure.IUserProcedure;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ItemNotFoundException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.nodetype.NodeType;

public class NodeCarrier extends Carrier {

	public NodeCarrier(DBController dc) {
		super(dc);
	}

	public int copy(String srcUUID, String newUUID, String destUUID) {
		IUserProcedure upt = createUserProcedure("node@copyWith(?,?,?)");
		upt.addParam(0, srcUUID);
		upt.addParam(1, newUUID);
		upt.addParam(2, destUUID);

		return execUpdate(upt);
	}

	public int move(String srcUUID, String destUUID) {
		IUserProcedure upt = createUserProcedure("node@moveWith(?, ?)");
		upt.addParam(0, srcUUID);
		upt.addParam(1, destUUID);

		return execUpdate(upt);
	}

	public boolean exist(String path) {
		IUserProcedure upt = createUserProcedure("node@existBy(?)");
		upt.addParam(0, path);

		return execQuery(upt).firstRow().getInt("result") > 0;
	}

	public Row retrieveByUUID(String workspaceName, String uuid) throws ItemNotFoundException {
		IUserProcedure upt = createUserProcedure("node@retrieveBy(?,?)");
		upt.addParam(workspaceName).addParam(uuid);

		Rows rows = execQuery(upt);
		try {
			return rows.firstRow();
		} catch (RepositoryException e) {
			throw ItemNotFoundException.create("uuid", uuid);
		}
	}

	public Row retrieveByPath(String path) throws ALRepositoryException {
		IUserProcedure upt = createUserProcedure("node@retrieveByPath(?)");
		upt.addParam(0, path);
		Rows rows = execQuery(upt);
		try {
			return rows.firstRow();
		} catch (RepositoryException e) {
			throw ItemNotFoundException.create("path", path);
		}
	}

	public NRows getNodes(NodeType nodetype, String workspaceName, boolean includeSub, Page page) {
		IUserProcedure upt = createUserProcedure("node@listNodeBy(?,?,?,?,?)");
		upt.addParam(0, nodetype.getId());
		upt.addParam(1, workspaceName);
		upt.addParam(2, includeSub ? T : F);
		upt.addParam(3, page.getListNum()) ;
		upt.addParam(4, page.getPageNo()) ;
		return execNQuery(upt);
	}

	public int clear() {
		IUserProcedure upt = createUserProcedure("node@clearWith()");
		return execUpdate(upt);
	}

	@Deprecated
	public int increaseViewCount(String uuid) {
		IUserProcedure upt = createUserProcedure("node@increaseViewCount(?)");
		upt.addParam(uuid);

		return 1;
	}

	public NRows listChildBy(Node node, Page page) {
		IUserProcedure upt = createUserProcedure("node@childBy(?,?,?)");
		upt.addParam(node.getIdentifier()).addParam(page.getListNum()).addParam(page.getPageNo()) ;
		return execNQuery(upt);
	}

}
