package com.bleujin.lore.core.dimpl.carrier;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.NoSuchPropertyDefinitionException;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.value.Value;

public class NodeCarrierTest extends DBStoreParentTestCase {

	private NodeType personType = null ; 
	private String FIRST_NAME = "firstName";
	
	public void setUp() throws Exception {
		super.setUp() ;
		PropertyDefinition firstName = getOrCreateProperty(FIRST_NAME, Property.Type.STRING) ;
		NewNodeType newType = ntManager.createNodeTypeTemplate(OBJECT_TYPE, "IPersion", new PropertyDefinition[]{firstName}) ;
		newType.setConstraint(firstName, new RequireConstraint()) ;
		personType = ntManager.register(newType, false) ;
	}
	
	public void testCaseInsensitive() throws Exception {
		GenericIterator<MappedDefinition> pds = personType.getMappedDefinitions() ;
		assertEquals(1, pds.size()) ;
		
		Node testNode = createNode(root, personType, "test") ;
		testNode.setProperty(FIRST_NAME, "bleu") ;
		assertEquals("bleu", testNode.getProperty(FIRST_NAME).getString()) ;
		
		testNode.save() ;
    }

	public void testLinkSize() throws Exception {
		GenericIterator<MappedDefinition> pds = personType.getMappedDefinitions() ;
		assertEquals(1, pds.size()) ;

		Node bleuNode = createNode(root, personType, "bleu") ;
		bleuNode.setProperty(FIRST_NAME, "bleu") ;
		
		Node jinNode = createNode(root, personType, "jin") ;
		jinNode.setProperty(FIRST_NAME, "jin") ;

		session.save() ;
		assertEquals(2, root.getLinks(Page.ALL).size()) ;
		
	}
	
	
	public void testCreateNode() throws Exception {
		Node testNode = createNode(root, personType, "test") ;
	    assertTrue(testNode.hasProperties()) ;

	    try {
	    	testNode.setProperty("firstName_", Value.stringValue("bleujin")) ;
	    	fail() ;
	    } catch (NoSuchPropertyDefinitionException ignore){
	    }
		
		testNode.setProperty(FIRST_NAME, Value.stringValue("bleujin")) ;

	    Property getProperty = testNode.getProperty(FIRST_NAME) ;
	    assertNotNull(getProperty) ;
	    assertEquals("bleujin", getProperty.getString()) ;

    }

}
