package com.bleujin.lore.core.dimpl.carrier;

import com.bleujin.framework.db.DBController;
import com.bleujin.framework.db.NRows;
import com.bleujin.framework.db.procedure.IUserProcedure;
import com.bleujin.framework.db.procedure.IUserProcedureBatch;
import com.bleujin.framework.db.procedure.UserProcedures;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.NoSuchPropertyDefinitionException;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class NodeTypeCarrier extends Carrier {

	public NodeTypeCarrier(DBController dc) {
		super(dc);
	}

	public int removeType(NodeTypeId ntId) {
		IUserProcedure upt = createUserProcedure("nodetype@removeWith(?)");
		return execUpdate(upt.addParam(ntId));
	}

	public NRows getType(NodeTypeId ntId) {
		IUserProcedure upt = createUserProcedure("nodetype@retrieveBy(?)");
		upt.addParam(ntId);

		return execNQuery(upt);
	}

	public boolean isIncludeType(NodeTypeId ntId) {
		IUserProcedure upt = createUserProcedure("nodetype@existBy(?)");
		upt.addParam(ntId);
		return execQuery(upt).firstRow().getInt("result") > 0;
	}

	public void createType(NewNodeType[] newNoeTypes, boolean allowUpdate) throws NoSuchPropertyDefinitionException, ALRepositoryException {

		UserProcedures upts = new UserProcedures("nodetype create");
		for (NewNodeType newNodeType : newNoeTypes) {
			IUserProcedure upt = createUserProcedure("nodetype@createWith(?,?,?,?,?)");
			upt.addParam(newNodeType.getId()).addParam(newNodeType.getName()).addParam(newNodeType.getSuperType().getId()).addParam(newNodeType.getMemberTypeId()).addParam(newNodeType.getPrimaryPropertyId());
			upts.add(upt);

			IUserProcedureBatch batch = createUserProcedureBatch("nodetype@addDefinition(?,?,?,?,?)");
			for (MappedDefinition md : newNodeType.getMappedDefinitions()) {
				batch.addBatchParam(0, newNodeType.getId());
				batch.addBatchParam(1, md.getId());
				batch.addBatchParam(2, md.getPropertyDefinition().requiredType().toString());
				batch.addBatchParam(3, md.getConstraint().getConfigurationExpression());
				batch.addBatchParam(4, md.getDefaultConstraint().getConfigurationExpression());
			}

			upts.add(batch);
		}

		execUpdate(upts);
	}

	public void updateType(NewNodeType[] updateTypes, boolean allowUpdate) throws ALRepositoryException {
		UserProcedures upts = new UserProcedures("nodetype update");
		for (NewNodeType updateType : updateTypes) {
			IUserProcedure upt = createUserProcedure("nodetype@updateWith(?,?,?,?,?)");
			upt.addParam(updateType.getId()).addParam(updateType.getName()).addParam(updateType.getSuperType().getId()).addParam(updateType.getMemberTypeId()).addParam(updateType.getPrimaryPropertyId());
			upts.add(upt);

			IUserProcedureBatch batch = createUserProcedureBatch("nodetype@setDefinition(?,?,?,?,?)");
			for (MappedDefinition md : updateType.getMappedDefinitions()) {
				batch.addBatchParam(0, updateType.getId());
				batch.addBatchParam(1, md.getId());
				batch.addBatchParam(2, md.getPropertyDefinition().requiredType().toString());
				batch.addBatchParam(3, md.getConstraint().getConfigurationExpression());
				batch.addBatchParam(4, md.getDefaultConstraint().getConfigurationExpression());
			}

			upts.add(batch);
		}

		execUpdate(upts);	}


	
	public NRows listBy(String prefix) {
		IUserProcedure upt = createUserProcedure("nodetype@listBy(?)");
		upt.addParam(prefix);

		return execNQuery(upt);
	}

	public int clear() {
		IUserProcedure upt = createUserProcedure("nodetype@clearWith()");
		return execUpdate(upt);
	}

	@Deprecated
	public int setMemberType(NodeTypeId id, NodeTypeId memberTypeId) {
		IUserProcedure upt = createUserProcedure("nodetype@setMemberWith(?,?)");
		return execUpdate(upt.addParam(id).addParam(memberTypeId));
	}

	@Deprecated
	public int setPrimary(NodeTypeId id, PropertyDefinition pd) {
		IUserProcedure upt = createUserProcedure("nodetype@setPrimaryWith(?, ?)");
		return execUpdate(upt.addParam(id).addParam(pd.getId()));

	}

}
