package com.bleujin.lore.core.io;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

public class LimitedInputStream extends FilterInputStream {

	private long length;

	private long currPos = 0L;

	public LimitedInputStream(InputStream input, long offset, long length) throws IOException {
		super(input);
		this.length = length;
		input.skip(offset);
	}

	public long skip(long length) throws IOException {
		currPos += length;
		return super.skip(length);
	}

	public int read() throws IOException {
		currPos++;
		if (currPos > length)
			return -1;

		return in.read();
	}

	public int available() throws IOException {
		return (int) (length - currPos);
	}

	public int read(byte[] buf, int off, int len) throws IOException {
		int toRead = Math.min((int) (length - currPos), len);
		if (0 >= toRead) {
			return -1;
		} else {
			int readCount = in.read(buf, off, toRead);
			currPos += toRead;
			return readCount;
		}
	}
}