package com.bleujin.lore.core.io;

import java.io.DataInputStream;
import java.io.EOFException;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.HashMap;
import java.util.Map;

public class MapInputStream extends FilterInputStream implements MapStream {
	private final DataInputStream din;
	private final CharsetDecoder decoder;

	public MapInputStream(InputStream in) {
		super(in);
		din = new DataInputStream(in);

		Charset utf = Charset.forName("UTF-8");
		decoder = utf.newDecoder();
	}

	public Map readMap() throws IOException {
		Map map = new HashMap();

		int size = din.readInt();

		for (int i = 0; i < size; ++i) {
			Object key = readObject();
			Object value = readObject();
			map.put(key, value);
		}

		return map;
	}

	protected String decode(byte bs[]) throws IOException {
		try {
			ByteBuffer bb = ByteBuffer.wrap(bs);
			CharBuffer cb = decoder.decode(bb);
			return cb.toString();
		} catch (CharacterCodingException cce) {
			throw new IOException(cce + "");
		}
	}

	protected String readString() throws IOException {
		int length = din.readInt();
		byte bs[] = new byte[length];
		din.readFully(bs);
		return decode(bs);
	}

	protected Object readObject() throws IOException {
		int type = din.read();

		if (type == -1) {
			throw new EOFException();
		}

		if (type == MAPSTREAM_INTEGER) {
			return new Integer(din.readInt());
		} else if (type == MAPSTREAM_DOUBLE) {
			return new Double(din.readDouble());
		} else if (type == MAPSTREAM_STRING) {
			return readString();
		} else if (type == MAPSTREAM_MAP) {
			return readMap();
		} else {
			throw new IOException("Bad stream type: " + type);
		}
	}
}
