package com.bleujin.lore.core.io;

import java.io.DataOutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;
import java.util.Iterator;
import java.util.Map;

public class MapOutputStream extends FilterOutputStream implements MapStream {
	private final DataOutputStream dout;
	private final CharsetEncoder encoder;

	public MapOutputStream(OutputStream out) {
		super(out);
		dout = new DataOutputStream(out);

		Charset utf = Charset.forName("UTF-8");
		encoder = utf.newEncoder();
	}

	public void write(Map map) throws IOException {
		dout.writeInt(map.size());
		for (Iterator it = map.keySet().iterator(); it.hasNext();) {
			Object key = it.next();
			Object value = map.get(key);
			writeObject(key);
			writeObject(value);
		}
	}

	protected byte[] encode(String s) throws IOException {
		try {
			CharBuffer cb = CharBuffer.wrap(s);
			ByteBuffer bb = encoder.encode(cb);
			int length = bb.limit();
			byte bs[] = new byte[length];
			bb.get(bs);
			return bs;
		} catch (CharacterCodingException cce) {
			throw new IOException(cce + "");
		}
	}

	protected void writeString(String s) throws IOException {
		byte encoded[] = encode(s);
		dout.writeInt(encoded.length);
		dout.write(encoded);
	}

	protected void writeObject(Object o) throws IOException {
		if (o instanceof Integer) {
			dout.write(MAPSTREAM_INTEGER);
			dout.writeInt(((Integer) o).intValue());
		} else if (o instanceof Double) {
			dout.write(MAPSTREAM_DOUBLE);
			dout.writeDouble(((Double) o).doubleValue());
		} else if (o instanceof String) {
			dout.write(MAPSTREAM_STRING);
			writeString((String) o);
		} else if (o instanceof Map) {
			dout.write(MAPSTREAM_MAP);
			write((Map) o);
		}
	}
}

interface MapStream {
	public byte MAPSTREAM_INTEGER = 0;
	public byte MAPSTREAM_DOUBLE = 1;
	public byte MAPSTREAM_MAP = 2;
	public byte MAPSTREAM_STRING = 3;
}
