package com.bleujin.lore.core.io;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

import org.apache.commons.io.IOUtils;

public class UTF8Writer extends Writer {
	private OutputStream output;

	private byte[] b;

	public UTF8Writer(OutputStream sink) {
		this(sink, 2048);
	}

	public UTF8Writer(OutputStream sink, int bufferSize) {
		this.output = sink;
		b = new byte[bufferSize];
	}

	public void close() throws IOException {
		output.close();
	}

	public void write(String s) throws UnsupportedEncodingException, IOException {
		IOUtils.copy(new ByteArrayInputStream(s.getBytes("UTF-8")), output);
	}

	public void write(char[] cbuf, int off, int len) throws IOException {
		int posEnd = b.length - 3;
		int pos = 0;
		int end = off + len;
		while (off < end) {
			// USASCII writer -- if ever needed
			// int n = len > b.length ? b.length : len;
			// for(int i = 0; i < n; ++i) {
			// b[i] = (byte)cbuf[pos+i];
			// }
			// sink.write(b, 0, n);
			// len -= n;
			// pos += n;
			int cc = cbuf[off++];
			if (cc <= 0x7F && cc != 0) {
				b[pos++] = (byte) cc;
			} else if (cc > 0x07FF) {
				b[pos++] = (byte) (0xE0 | (cc >> 12));
				b[pos++] = (byte) (0x80 | ((cc >> 6) & 0x3F));
				b[pos++] = (byte) (0x80 | (cc & 0x3F));
			} else {
				b[pos++] = (byte) (0xC0 | (cc >> 6));
				b[pos++] = (byte) (0x80 | (cc & 0x3F));
			}
			if (pos > posEnd) {
				output.write(b, 0, pos);
				pos = 0;
			}
		}
		output.write(b, 0, pos);
	}

	public void flush() throws IOException {
		output.flush();
	}
}
