package com.bleujin.lore.core.io;

import java.io.IOException;
import java.io.InputStream;

public class VirtualInputStream extends InputStream {

	private long toPos;
	private long currPos = 0L;
	private ISeekableStream input;

	public VirtualInputStream(ISeekableStream input, long offset, int length) throws IOException {
		this.input = input;
		this.currPos = offset;
		this.toPos = offset + length;
	}

	public long skip(long length) throws IOException {
		long actual = Math.min(available(), length);
		currPos += actual;
		return actual;
	}

	public int read() throws IOException {
		synchronized (input) {
			if (input.getPos() != currPos)
				input.seek(currPos);
			currPos++;
			if (currPos > toPos)
				return -1;

			return input.read();
		}
	}

	public int available() throws IOException {
		return (int) (toPos - currPos);
	}

	public void close() throws IOException {
		; // noaction.
	}

	public int read(byte[] buf, int off, int len) throws IOException {
		synchronized (input) {
			if (input.getPos() != currPos) {
				assert false : "should not enter this area, getPos: " + input.getPos() + ", currPos: " + currPos;
				input.seek(currPos);
			}

			int toRead = Math.min((int) (toPos - currPos), len);
			if (0 >= toRead) {
				return -1;
			} else {
				int readCount = input.read(buf, off, toRead);
				currPos += readCount;
				return readCount;
			}
		}
	}
}
