package com.bleujin.lore.core.link;

import java.util.List;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.dimpl.LinkTypeImpl;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class TestLink extends TestCaseParent {
	
	public void testMember() throws Exception {
		
		PropertyDefinition explain = createPropertyDefinition("explain", Property.Type.STRING);
		NewNodeType robotTypeTemplate = createNewNodeType(objectType, "robot", new PropertyDefinition[] { explain });
		robotTypeTemplate.setConstraint("explain", new RequireConstraint());
		robotTypeTemplate.setSelfMemberType() ;
		
		NodeType robotType = registerNodeType(robotTypeTemplate) ;
		
		Node r1 = saveNode(robotType, "1") ;
		Node r2 = saveNode(robotType, "2") ;
		Node r3 = saveNode(robotType, "3") ;
		Node r4 = saveNode(robotType, "4") ;
		Node r5 = saveNode(robotType, "5") ;
		
		r1.addLink(r2, LinkTypeImpl.MEMBER, Position.BEFORE) ;
		r1.addLink(r3, LinkTypeImpl.MEMBER, Position.BEFORE) ;
		r1.addLink(r4, LinkTypeImpl.MEMBER, Position.BEFORE) ;
		r1.addLink(r5, LinkTypeImpl.AGREE, Position.BEFORE) ;
		
		session.save() ;
		assertEquals(3, r1.getLinks(LinkTypeImpl.MEMBER, Page.ALL).size()) ;

		assertEquals(2, r1.getLinks(LinkTypeImpl.MEMBER, Page.create(2, 1)).size()) ;
		assertEquals(1, r1.getLinks(LinkTypeImpl.MEMBER, Page.create(2, 2)).size()) ;
		assertEquals(0, r1.getLinks(LinkTypeImpl.MEMBER, Page.create(2, 3)).size()) ;

		assertEquals(2, r1.getLinks(Page.create(2, 1)).size()) ;
		assertEquals(2, r1.getLinks(Page.create(2, 2)).size()) ;
		assertEquals(0, r1.getLinks(Page.create(2, 3)).size()) ;


		List<Link> members = r1.getLinks(LinkTypeImpl.MEMBER, Page.create(3, 1)).toList() ;

		// test Order
		assertEquals(r4.getIdentifier(), members.get(0).getTo().getIdentifier()) ;
		assertEquals(r3.getIdentifier(), members.get(1).getTo().getIdentifier()) ;
		assertEquals(r2.getIdentifier(), members.get(2).getTo().getIdentifier()) ;

	}
	
	
	public void testDefaultAction() throws Exception {

		String name = "Hate";
		LinkType hateType = session.createLinkType(name);
		assertNotNull(hateType);
		assertEquals("hate", hateType.getString());

		Node mbNode = saveNode(objectType, "myeongbak");
		Node ghNode = saveNode(objectType, "geunhae");
		mbNode.addLink(ghNode, hateType, Position.AFTER) ;

		session.save();
		GenericIterator<Link> links = mbNode.getLinks(Page.ALL);
		assertEquals(1, links.size());
		assertEquals(hateType, links.next().getType());

		GenericIterator<Node> nodes = mbNode.getNodes(hateType, Page.ALL);
		assertEquals(1, nodes.size());

		Node nextNode = nodes.next() ;
		assertTrue(ghNode.isSame(nextNode));

		// Link is simplex transmission
		nodes = ghNode.getNodes(hateType, Page.ALL);
		assertEquals(false, nodes.hasNext());

		// setLink via Node
		LinkType sangsengType = session.createLinkType("sangseng");
		mbNode.addLink(ghNode, sangsengType, Position.AFTER);

		session.save();
		
		nodes = mbNode.getNodes(sangsengType, Page.ALL);
		Node nNode = nodes.next() ;
		assertEquals(ghNode, nNode);
		links = mbNode.getLinks(Page.ALL);
		assertEquals(2, links.size());

		// left right
		links = mbNode.getLinks(Page.ALL);
		assertEquals(2, links.size());
	}
	
	public void testAfterSave() throws Exception {
		LinkType hateType = session.createLinkType("hate");
		LinkType sangsengType = session.createLinkType("sangseng");
		
		Node mbNode = saveNode(objectType, "myeongbak");
		Node ghNode = saveNode(objectType, "geunhae");
		mbNode.addLink(ghNode, hateType, Position.AFTER) ;

		session.save();
		
		Node loadNode = session.getNode(mbNode.getUUIDString()) ;
		GenericIterator<Link> links = loadNode.getLinks(Page.ALL);
		assertEquals(1, links.size());

		loadNode.addLink(ghNode, sangsengType, Position.AFTER) ;

		
		session.save();
		GenericIterator<Link> links2 = loadNode.getLinks(Page.ALL) ;

		assertEquals(2, links2.size());
		
		GenericIterator<Link> links3 = loadNode.getLinks(Page.ALL) ;
		assertEquals(2, links3.size());
		
	}
	
	public void xtestReferenceProperty() throws Exception {
		Node testFirstNode = createTestNode() ;
		PropertyDefinition ref = createPropertyDefinition("ref", Property.Type.REFERENCE) ;
		NodeType newtype = registerNodeType(objectType, "newtype", new PropertyDefinition[]{ref});
		
		Node newNode = createNodeSave(testFirstNode, newtype, "new") ;
		Node target = createNodeSave(testFirstNode, objectType, "target") ;
		
		newNode.setProperty("ref", testFirstNode.getIdentifier()) ;
		session.save() ;
		
		GenericIterator<Link> links = newNode.getLinks(LinkTypeImpl.REFERENCE.getString()) ;
		assertEquals(1, links.size()) ;
		assertEquals(LinkTypeImpl.REFERENCE, links.next().getType()) ;
		
	}
}
