package com.bleujin.lore.core.mapping;

import java.util.ArrayList;
import java.util.List;

import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.value.Value;

public class AndConstraint implements IValueConstraint{

	private static final long serialVersionUID = -1770007612351649283L;
	private IValueConstraint[] constraints ;
	
	public AndConstraint(IValueConstraint one, IValueConstraint two){
		this(new IValueConstraint[]{one, two}) ;
	}	
	
	public AndConstraint(IValueConstraint one, IValueConstraint two, IValueConstraint three){
		this(new IValueConstraint[]{one, two, three}) ;
	}
	
	public AndConstraint(IValueConstraint one, IValueConstraint two, IValueConstraint three,  IValueConstraint four){
		this(new IValueConstraint[]{one, two, three, four}) ;
	}
	
	public AndConstraint(IValueConstraint one, IValueConstraint two, IValueConstraint three,  IValueConstraint four,  IValueConstraint five, String message){
		this(new IValueConstraint[]{one, two, three, four, five}) ;
	}
	
	AndConstraint(IValueConstraint[] constraints){
		this.constraints = constraints ;
	}
	
	
	public boolean isValid(Value value) {
		for (int i = 0, last = constraints.length; i < last; i++) {
	        if (! constraints[i].isValid(value)) return false ;
        }
	    return true;
    }

	public ValidMessage getValidMessage(IDString propId, Value value){
		List<String> messages = new ArrayList<String>() ;
		boolean result = true ;
		for (IValueConstraint constraint : constraints) {
			ValidMessage inner = constraint.getValidMessage(propId, value) ;
	        if (! inner.isValid()) {
	        	result = false ;
	    		messages.add(inner.getViolationMessage()) ;
	        }
        }
		return ValidMessage.create(result, messages.toString()) ;
	}

	public String getConfigurationExpression(){
		String str = "" +
		"<configured-object>\n" + 
        "<class-name>" + getClass().getName()+ "</class-name>\n" +
        "<constructor>\n" ; 
	    for(IValueConstraint vc : constraints){
		    str += 
		    "    <constructor-param>\n" +
		    "        <description>constraint</description>\n" +
		    "        <type>com.bleujin.lore.core.mapping.IValueConstraint</type>\n" +
		    "        <value>" ;
	    	str += vc.getConfigurationExpression() + 
		    "		</value>\n" +
		    "    </constructor-param>\n" ; 
	    }
	    str += 
        "</constructor>\n" +
        "</configured-object>" ;
		return str ;
	}	
}
