package com.bleujin.lore.core.mapping;

import java.text.MessageFormat;

import org.apache.commons.validator.GenericValidator;

import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.value.Value;

public class LongRangeConstraint implements IValueConstraint {
 
	private static final long serialVersionUID = -6628516686400590489L;
	private long min ;
	private long max ;
	public LongRangeConstraint(long min, long max){
		this.min = min ;
		this.max = max ;
		if (min > max) throw new IllegalArgumentException() ;		
	}
	
	public ValidMessage getValidMessage(IDString propId, Value value) {
		return ValidMessage.create(isValid(value), MessageFormat.format("{0} should be equal or larger than {1} and equal or smaller than {2} ", new Object[]{propId, new Long(min), new Long(max)})) ;
	}

	public boolean isValid(Value value) {
		if (!GenericValidator.isLong(value.getString())) return false ;
		try {
			return GenericValidator.isInRange(value.getLong(), min, max) ;
		} catch (ValueFormatException e) {
			return false ;
		}
	}
	
	public String getConfigurationExpression(){
		String str = "" +
		"<configured-object>\n" + 
        "<class-name>" + getClass().getName()+ "</class-name>\n" +
        "<constructor>\n" +
        "    <constructor-param>\n" +
        "        <description>min Length</description>\n" +
        "        <type>long</type>\n" +
        "        <value>" + min + "</value>\n" +
        "    </constructor-param>\n" +
        "    <constructor-param>\n" +
        "        <description>max Length</description>\n" +
        "        <type>long</type>\n" +
        "        <value>" + max + "</value>\n" +
        "    </constructor-param>\n" +
        "</constructor>\n" +
        "</configured-object>" ;
		return str ;
	}
}
