package com.bleujin.lore.core.mapping;

import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.value.Value;

public class OrConstraint implements IValueConstraint {
 
	private static final long serialVersionUID = 2057029837804066193L;
	private IValueConstraint[] constraints;
	private String message;

	public OrConstraint(IValueConstraint one, IValueConstraint two, String message){
		this(new IValueConstraint[]{one, two}, message) ;
	}	
	
	public OrConstraint(IValueConstraint one, IValueConstraint two, IValueConstraint three, String message){
		this(new IValueConstraint[]{one, two, three}, message) ;
	}
	
	public OrConstraint(IValueConstraint one, IValueConstraint two, IValueConstraint three,  IValueConstraint four, String message){
		this(new IValueConstraint[]{one, two, three, four}, message) ;
	}
	
	public OrConstraint(IValueConstraint one, IValueConstraint two, IValueConstraint three,  IValueConstraint four,  IValueConstraint five, String message){
		this(new IValueConstraint[]{one, two, three, four, five}, message) ;
	}
	
	OrConstraint(IValueConstraint[] constraints, String message){
		this.constraints = constraints ;
		this.message = message ;
	}

	public boolean isValid(Value value) {
		for (IValueConstraint constraint : constraints) {
			if (constraint.isValid(value)) return true ; 
		}
		return false;
	}

	public ValidMessage getValidMessage(IDString propId, Value value) {
		return ValidMessage.create(isValid(value), message) ;
	}

	
	public String getConfigurationExpression(){
		String str = "" +
		"<configured-object>\n" + 
        "<class-name>" + getClass().getName()+ "</class-name>\n" +
        "<constructor>\n" ; 
	    for(IValueConstraint vc : constraints){
		    str += 
		    "    <constructor-param>\n" +
		    "        <description>constraint</description>\n" +
		    "        <type>com.bleujin.lore.core.mapping.IValueConstraint</type>\n" +
		    "        <value>" ;
	    	str += vc.getConfigurationExpression() + 
		    "		</value>\n" +
		    "    </constructor-param>\n" ; 
	    }
	    str += 
        "</constructor>\n" +
        "<constructor>\n" + 
	    "    <constructor-param>\n" +
	    "        <description>constraint</description>\n" +
	    "        <type>java.lang.String</type>\n" +
	    "        <value>"  + this.message + "</value>\n" +
	    "    </constructor-param>\n" + 
        "</constructor>\n" +
        "</configured-object>" ;
		return str ;
	}
}
