package com.bleujin.lore.core.mapping;

import java.io.UnsupportedEncodingException;
import java.text.MessageFormat;

import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.value.Value;


public class OverByteLengthConstraint implements IValueConstraint{
 
	private static final long serialVersionUID = -6321881774888573205L;
	private int minLength;
	public OverByteLengthConstraint(int minLength){
		this.minLength = minLength ;
	}
	
	public boolean isValid(Value value) {
	    try {
	        return (!value.isEmpty()) && value.getString().getBytes("UTF-8").length >= minLength ;
        } catch (UnsupportedEncodingException e) {
	        throw new IllegalArgumentException(e.getMessage()) ;
        }
    }

	public ValidMessage getValidMessage(IDString propId, Value value){
		return ValidMessage.create(isValid(value), MessageFormat.format("{0} input at least {1} byte.", new Object[]{propId, new Integer(minLength)})) ;
	}
	public String getConfigurationExpression(){
		String str = "" +
		"<configured-object>" + "\n" + 
        "<class-name>" + getClass().getName()+ "</class-name>" + "\n" +
        "<constructor>" + "\n" +
        "    <constructor-param>" + "\n" +
        "        <description>min Length</description>" + "\n" +
        "        <type>int</type>" + "\n" +
        "        <value>" + minLength + "</value>" + "\n" +
        "    </constructor-param>" + "\n" +
        "</constructor>" + "\n" +
        "</configured-object>" ;
		return str ;
	}
}
