package com.bleujin.lore.core.mapping;

import java.io.UnsupportedEncodingException;
import java.text.MessageFormat;

import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.value.Value;


public class RangeByteLengthConstraint implements IValueConstraint{
 
	private static final long serialVersionUID = -8253195433368381332L;
	private int minLength ;
	private int maxLength ;
	public RangeByteLengthConstraint(int minLength, int maxLength){
		this.minLength = minLength ;
		this.maxLength = maxLength ;
		if ((minLength > maxLength) || minLength < 0) throw new IllegalArgumentException() ;
	}
	
	public boolean isValid(Value value) {
	    try {
	    	if (!value.isEmpty()){
	    		return value.getString().getBytes("UTF-8").length <= maxLength && value.getString().getBytes("UTF-8").length >= minLength ;
	    	} else {
	    		return minLength <= 0 ;
	    	}
        } catch (UnsupportedEncodingException e) {
	        throw new IllegalArgumentException(e.getMessage()) ;
        }
    }

	public ValidMessage getValidMessage(IDString propId, Value value){
		return ValidMessage.create(isValid(value), MessageFormat.format("{0} is possible to input {1}-{2} byte.", new Object[]{propId, new Integer(minLength), new Integer(maxLength)})) ;
	}

	
	public String getConfigurationExpression(){
		String str = "" +
		"<configured-object>\n" + 
        "<class-name>" + getClass().getName()+ "</class-name>\n" +
        "<constructor>\n" +
        "    <constructor-param>\n" +
        "        <description>min Length</description>" + "\n" +
        "        <type>int</type>\n" +
        "        <value>" + minLength + "</value>\n" +
        "    </constructor-param>\n" +
        "    <constructor-param>\n" +
        "        <description>max Length</description>\n" +
        "        <type>int</type>\n" +
        "        <value>" + maxLength + "</value>\n" +
        "    </constructor-param>\n" +
        "</constructor>\n" +
        "</configured-object>" ;
		return str ;
	}	
}
