package com.bleujin.lore.core.mapping;

import java.text.MessageFormat;

import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.value.Value;

public class RangeNumberConstraint implements IValueConstraint {
 
	private static final long serialVersionUID = -8115291197951835770L;
	private long min;
	private long max;

	public RangeNumberConstraint(long minLength, long maxLength) {
		this.min = minLength;
		this.max = maxLength;
		if (minLength > maxLength) throw new IllegalArgumentException() ;
	}

	public boolean isValid(Value value) {
		try {
			return (!value.isEmpty()) && value.getLong() <= max && value.getLong() >= min;
		} catch (ValueFormatException e) {
			return false ;
		}
	}

	public ValidMessage getValidMessage(IDString propId, Value value) {
		return ValidMessage.create(isValid(value), MessageFormat.format("{0} is possible to input {1}-{2} byte.", new Object[] { propId, new Long(min), new Long(max) })) ;
	}
	
	public String getConfigurationExpression(){
		String str = "" +
		"<configured-object>\n" + 
        "<class-name>" + getClass().getName()+ "</class-name>\n" +
        "</configured-object>" ;
		return str ;
	}
}