package com.bleujin.lore.core.mapping;


import junit.framework.TestCase;

import com.bleujin.lore.core.value.Value;

public class TestConstraint extends TestCase {

	public void testRangeByte() throws Exception {
		IValueConstraint c = new RangeByteLengthConstraint(10, 20);
 		assertEquals(false, c.isValid(Value.stringValue("123456789")));
 		assertEquals(true, c.isValid(Value.stringValue("1234567890")));
 		assertEquals(true, c.isValid(Value.stringValue("12345678901234567890")));
 		assertEquals(false, c.isValid(Value.stringValue("123456789012345678901")));
		
 		assertEquals(false, c.isValid(Value.stringValue("가나다")));
 		assertEquals(true, c.isValid(Value.stringValue("가나다라")));
 		assertEquals(true, c.isValid(Value.stringValue("가나다라마바")));
 		assertEquals(false, c.isValid(Value.stringValue("가나다라마바사")));
		
		
 		c = new RangeByteLengthConstraint(0, 10);
 		assertEquals(true, c.isValid(Value.stringValue("")));
 		assertEquals(true, c.isValid(Value.stringValue("          ")));
 		assertEquals(false, c.isValid(Value.stringValue("           ")));
 		
	}
	
	public void testNumberic() throws Exception {
		IValueConstraint c = new NumberConstraint();
 		assertTrue(c.isValid(Value.stringValue("-1")));
 		assertFalse(c.isValid(Value.stringValue("--1")));
 		assertFalse(c.isValid(Value.stringValue("1245a")));
 		assertTrue(c.isValid(Value.stringValue("0")));
 		assertTrue(c.isValid(Value.stringValue("1")));
 		assertTrue(c.isValid(Value.stringValue("999999999999")));
	}
	
	public void testDate() throws Exception {
		IValueConstraint c = new DateConstraint();
		assertFalse(c.isValid(Value.stringValue("")));
		assertTrue(c.isValid(Value.stringValue("19991211-124514")));
		assertFalse(c.isValid(Value.stringValue("19991211")));
		assertTrue(c.isValid(Value.stringValue("99991231-235959")));
		assertTrue(c.isValid(Value.stringValue("10000101-000000")));
		assertTrue(c.isValid(Value.stringValue("12340132-000000"))); // -> Feb. 1
    }
	
	public void testDouble() throws Exception {
		IValueConstraint c = new DoubleConstraint();
		assertTrue(c.isValid(Value.stringValue("")));
		assertTrue(c.isValid(Value.stringValue("122.0")));
		assertTrue(c.isValid(Value.stringValue("122.00")));
		assertTrue(c.isValid(Value.stringValue("122.")));
		assertFalse(c.isValid(Value.stringValue("122..")));
    }
	
	public void testEmail() {
		IValueConstraint c = new EmailConstraint();
		assertTrue(c.isValid(Value.stringValue("")));
		assertTrue(c.isValid(Value.stringValue("bleujin@hitel.net")));
		assertFalse(c.isValid(Value.stringValue("bleujin")));
		assertFalse(c.isValid(Value.stringValue("bleujin@hitel.")));
    }
	
	public void testLongRange() throws Exception {
		IValueConstraint c = new LongRangeConstraint(0, 3);
		assertFalse(c.isValid(Value.stringValue("")));
		assertTrue(c.isValid(Value.stringValue("0")));
		assertTrue(c.isValid(Value.stringValue("1")));
		assertTrue(c.isValid(Value.stringValue("3")));
		assertFalse(c.isValid(Value.stringValue("4")));
    }
	
	public void testAnd() throws Exception {
		IValueConstraint c = new AndConstraint(new IValueConstraint[]{new RequireConstraint(), new AlphaNumConstraint()});
		assertFalse(c.isValid(Value.stringValue("")));
		assertTrue(c.isValid(Value.stringValue("1")));
		assertFalse(c.isValid(Value.stringValue("_")));

		assertTrue(c.isValid(Value.stringValue("a")));
		assertTrue(c.isValid(Value.stringValue("aA")));
    }
	
	public void testOr() throws Exception {
		IValueConstraint c = new OrConstraint(new NumberConstraint(), new AlphaNumConstraint(), "invalid");
		assertTrue(c.isValid(Value.stringValue("")));

		assertTrue(c.isValid(Value.stringValue("a")));
		assertTrue(c.isValid(Value.stringValue("111")));
		assertTrue(c.isValid(Value.stringValue("aA")));

		assertFalse(c.isValid(Value.stringValue("_")));
		assertTrue(c.isValid(Value.stringValue("aa1")));
		
	}
}
