package com.bleujin.lore.core.mapping;

import java.lang.reflect.Array;

import junit.framework.TestCase;

import com.bleujin.framework.configuration.ConfigurationFactory;
import com.bleujin.framework.configuration.DefaultConfigurationFactory;
import com.bleujin.framework.util.InstanceCreator;
import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.io.StringInputStream;
import com.bleujin.lore.core.value.Value;

public class TestConstrationReflection extends TestCase{
	
	public void testMakeDefaultValue() throws Exception {
		IDefaultValue defaultValue = getDefaultValue(IDefaultValue.NONE);

		defaultValue = getDefaultValue(new LongDefaultValue(3));
		defaultValue = getDefaultValue(new StringDefaultValue("3"));
		assertEquals("3", defaultValue.getValue().getString()) ;
		
		
		defaultValue = getDefaultValue(new CurrentDateValue());
	}
	
	
	
	
	private IDefaultValue getDefaultValue(IDefaultValue defaultValue) throws ALRepositoryException {
		return ConstraintHelper.makeDefaultValue(defaultValue.getConfigurationExpression()) ;
	}




	public void testMakeConfiguration() throws Exception {
		IValueConstraint constraint = getConstraint(new AlphaNumConstraint());

		constraint = getConstraint(new AlphaNumUnderbarConstraint());

		constraint = getConstraint(new ContainSpecialCharConstraint());

		constraint = getConstraint(new DoubleConstraint());

		constraint = getConstraint(new EmailConstraint());

		constraint = getConstraint(new IDConstraint());

		constraint = getConstraint(new LongRangeConstraint(0, 10));
		assertEquals(true, constraint.isValid(Value.longValue(5)));
		assertEquals(false, constraint.isValid(Value.longValue(11)));
		
		constraint = getConstraint(new NumberConstraint());
		
		constraint = getConstraint(new OverByteLengthConstraint(10));
		
		constraint = getConstraint(new OverLengthConstraint(10));
		
		constraint = getConstraint(new PositiveNumberConstraint());
		
		constraint = getConstraint(new RangeByteLengthConstraint(0, 10));
		
		constraint = getConstraint(new RequireConstraint());
		
		constraint = getConstraint(new SmallAlphaNumUnderBarConstraint());
		
		constraint = getConstraint(new UnderByteLengthConstraint(10));
		
		
		
		OrConstraint or = new OrConstraint(new AlphaNumConstraint(), new AlphaNumUnderbarConstraint(), "mmmm");
		AndConstraint and = new AndConstraint(new AlphaNumConstraint(), new AlphaNumUnderbarConstraint());
		// Debug.debug(this, and.getConfigurationString());
		// config = getConfiguration(and);
		// c = (IValueConstraint)InstanceCreator.createConfiguredInstance(config.getConfiguration("configured-object")) ;
	
	}


	private IValueConstraint getConstraint(IValueConstraint constraint) throws ALRepositoryException  {
		return ConstraintHelper.makeConstraint(constraint.getConfigurationExpression()) ;
	}
	
	

	public void testConfiguration() throws Exception {

		String reflectionStr = "<?xml version=\"1.0\" encoding=\"UTF-16\"?>\n\n" +
		"<configured-object>" + "\n" + 
        "<class-name>com.bleujin.lore.core.mapping.RangeNumberConstraint</class-name>" + "\n" +
        "<constructor>" + "\n" +
        "    <constructor-param>" + "\n" +
        "        <description>min Byte Length</description>" + "\n" +
        "        <type>long</type>" + "\n" +
        "        <value>10</value>" + "\n" +
        "    </constructor-param>" + "\n" +
        "    <constructor-param>" + "\n" +
        "        <description>max Byte Length</description>" + "\n" +
        "        <type>long</type>" + "\n" +
        "        <value>400</value>" + "\n" +
        "    </constructor-param>" + "\n" +
        "</constructor>" + "\n" +
        "</configured-object>" ;
		
		ConfigurationFactory config = DefaultConfigurationFactory.getInstance("default") ;
		config.build(new StringInputStream(reflectionStr)) ;
		
		IValueConstraint c = (IValueConstraint)InstanceCreator.createConfiguredInstance(config.getConfiguration("configured-object")) ;
		

		assertEquals(false, c.isValid(Value.longValue(5))) ;
		assertEquals(true, c.isValid(Value.longValue(10))) ;
		assertEquals(true, c.isValid(Value.longValue(200))) ;
		assertEquals(true, c.isValid(Value.longValue(400))) ;
		assertEquals(false, c.isValid(Value.longValue(500))) ;
	}
	
	public void testArray() throws Exception {

		String reflectionStr = "" +
		"<configured-object>\n" + 
		"<class-name>com.bleujin.lore.core.mapping.AndConstraint</class-name>\n" +
		"<constructor>\n" +
		"    <constructor-param>\n" +
		"        <description>constraint</description>\n" +
		"        <type>com.bleujin.lore.core.mapping.IValueConstraint</type>\n" +
		"        <value>\n" +
		"    		<configured-object>\n" +
		"				<class-name>com.bleujin.lore.core.mapping.AlphaNumConstraint</class-name>\n" +
		"			</configured-object>\n" +
		"		</value>\n" +
		"    </constructor-param>\n" +
		"    <constructor-param>\n" +
		"        <description>constraint</description>\n" +
		"        <type>com.bleujin.lore.core.mapping.IValueConstraint</type>\n" +
		"        <value>\n" +
		"			<configured-object>\n" +
		"				<class-name>com.bleujin.lore.core.mapping.AlphaNumUnderbarConstraint</class-name>\n" +
		"			</configured-object>" +
		"		</value>\n" +
		"    </constructor-param>\n" +
		"</constructor>\n" +
		"</configured-object>\n" ;
		
		IValueConstraint c = ConstraintHelper.makeConstraint(reflectionStr) ;
		assertNotNull(c) ;
		
		String saved = c.getConfigurationExpression();
		
		assertEquals(StringUtil.deleteWhitespace(reflectionStr), StringUtil.deleteWhitespace(saved)) ;
		
		assertEquals(false, c.isValid(Value.stringValue("_abc@")));
		String vm = c.getValidMessage(IDString.propId("abc"), Value.stringValue("_abc@")).getViolationMessage() ;
		assertEquals(true, StringUtil.contains(vm, ',')) ;
		
		
		
		IValueConstraint or = new OrConstraint(new NumberConstraint(), new PositiveNumberConstraint(), "N or P") ;
		String or_expr = or.getConfigurationExpression();
		
		
		
		
	}
	
	
	public void xtestStringArray() throws Exception {
		
		Class stringArrayClass = Class.forName("[Ljava.lang.String;");
		assertEquals(true, stringArrayClass.isArray()) ;
		
		Class stringClass = Class.forName("java.lang.String") ;
		String sc = (String)stringClass.newInstance() ;
		
		Class o = Array.newInstance(stringArrayClass, 0).getClass() ;
		Debug.debug(this, o);
		
		String str = "<configured-object>" + "\n" + 
        "			<class-name>java.lang.String</class-name>" + "\n" +
        "			<constructor>" + "\n" +
        "    			<constructor-param>" + "\n" +
        "        			<description>min Byte Length</description>" + "\n" +
        "        			<type>java.lang.String</type>" + "\n" +
        "        			<value>abc</value>" +
        "    			</constructor-param>" + "\n" +
        "			</constructor>" + "\n" +
		"		</configured-object>" ; 
		
		String reflectionStr = "" +
		"<configured-object>" + "\n" + 
        "<class-name>com.bleujin.lore.core.constraint.ArrayTestClass</class-name>" + "\n" +
        "<constructor>" + "\n" +
        "    <constructor-param>" + "\n" +
        "        <description>min Byte Length</description>" + "\n" +
        "        <type>[Ljava.lang.String;</type>" + "\n" +
        "        <value>" +
		"		 <configured-object>" + "\n" + 
        "			<class-name>java.lang.String</class-name>" + "\n" +
        "			<constructor>" + "\n" +
        "    			<constructor-param>" + "\n" +
        "        			<description>min Byte Length</description>" + "\n" +
        "        			<type>[Ljava.lang.String;</type>" + "\n" +
        "        			<value>abc</value>" +
        "    			</constructor-param>" + "\n" +
        "			</constructor>" + "\n" +
		"		</configured-object>" + "\n" + 
        "		</value>" + "\n" +
        "    </constructor-param>" + "\n" +
        "</constructor>" + "\n" +
        "</configured-object>" ;
		
		ConfigurationFactory config = DefaultConfigurationFactory.getInstance("string_array") ;
		config.build(new StringInputStream("<?xml version=\"1.0\" encoding=\"UTF-16\"?>\n\n" +reflectionStr)) ;
		
		Object c = InstanceCreator.createConfiguredInstance(config.getConfiguration("configured-object")) ;
		Debug.debug(this, c);
	}

}
