package com.bleujin.lore.core.migration.sample;

import java.io.IOException;
import java.sql.SQLException;

import com.bleujin.framework.db.Rows;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.dimpl.NodeTypeManager;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.mapping.IValueConstraint;
import com.bleujin.lore.core.mapping.PositiveNumberConstraint;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.migration.Importer;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class DeptImporter implements Importer {

	private UserSession session;
	private Rows rows;

	public DeptImporter(UserSession session, Rows rows) {
		this.session = session;
		this.rows = rows;
	}

	public void imp() throws IOException, ALRepositoryException, SQLException {

		String DEPTNO = "deptNo";
		String DNAME = "dname";
		String LOC = "loc";
		NodeTypeManager ntManager = session.getNodeTypeManager() ;

		PropertyDefinition pd_deptno = createPropertyDefinition(DEPTNO, Property.Type.LONG);
		PropertyDefinition pd_dname = createPropertyDefinition(DNAME, Property.Type.STRING);
		PropertyDefinition pd_loc = createPropertyDefinition(LOC, Property.Type.STRING);
		PropertyDefinition[] pd = new PropertyDefinition[] { pd_deptno, pd_dname, pd_loc };

		NodeType objectType = ntManager.getNodeType("_object");
		NewNodeType newType = ntManager.createNodeTypeTemplate(objectType, "dept", pd);
		newType.setConstraint(pd_deptno, new PositiveNumberConstraint());
		newType.setConstraint(pd_dname, new RequireConstraint());
		newType.setConstraint(pd_loc, IValueConstraint.NONE);
		NodeType deptType = ntManager.register(newType, false) ; 
		Node deptParentNode = session.getNode("/scott_dept");

		// create emp node
		while (rows.next()) {
			int deptNo = rows.getInt(DEPTNO);
			String dname = rows.getString(DNAME);
			String loc = rows.getString(LOC);

			INode node = session.createNode(deptParentNode, deptType, dname);
			node.setProperty(DEPTNO, deptNo);
			node.setProperty(DNAME, dname);
			node.setProperty(LOC, loc);
		}

		// set empNode link
		GenericIterator<Node> employees = session.getNode("/scott_emp").getChildNode();
		LinkType belongToLink = session.createLinkType("belongTo");
		while (employees.hasNext()) {
			Node employee = employees.next();
			addLink(employee, belongToLink, DEPTNO, employee.getProperty(DEPTNO).getValue().getLong());
		}

		session.save();
	}

	private PropertyDefinition createPropertyDefinition(String pid, Property.Type requireType) throws ALRepositoryException {
		return session.getNodeTypeManager().createPropertyDefinition(pid, requireType);
	}

	private void addLink(Node employee, LinkType linkType, String propId, long value) throws ALRepositoryException {
		Node deptParentNode = session.getNode("/scott_dept");
		GenericIterator<Node> depts = deptParentNode.getChildNode();
		while (depts.hasNext()) {
			Node dept = depts.next();
			if (dept.getProperty(propId).getValue().getLong() == value) {
				employee.addLink(dept, linkType, Position.AFTER);
				break;
			}
			;
		}
	}
}
