package com.bleujin.lore.core.migration.sample;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Date;

import com.bleujin.framework.db.Rows;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.dimpl.NodeTypeManager;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.mapping.PositiveNumberConstraint;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.migration.Importer;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class EmpImporter implements Importer{


	private UserSession session ;
	private Rows rows ;
	public EmpImporter(UserSession session, Rows rows) {
		this.session = session ;
		this.rows = rows ;
    }
	
	public void imp() throws IOException, ALRepositoryException, SQLException {

		String EMPNO = "empNo";
		String ENAME = "ename";
		String JOB = "job";
		String MGR = "mgr";
		String HIREDATE = "hireDate";
		String COMM = "comm";
		String DEPTNO = "deptno";
		NodeTypeManager ntManager = session.getNodeTypeManager() ;
		
		PropertyDefinition pd_empno = createPropertyDefinition(EMPNO, Property.Type.LONG);
		PropertyDefinition pd_ename = createPropertyDefinition(ENAME, Property.Type.STRING);
		PropertyDefinition pd_job = createPropertyDefinition(JOB, Property.Type.STRING);
		PropertyDefinition pd_mgr = createPropertyDefinition(MGR, Property.Type.LONG);
		PropertyDefinition pd_hiredate = createPropertyDefinition(HIREDATE, Property.Type.DATE);
		PropertyDefinition pd_comm = createPropertyDefinition(COMM, Property.Type.LONG);
		PropertyDefinition pd_deptno = createPropertyDefinition(DEPTNO, Property.Type.LONG);
		PropertyDefinition[] pd = new PropertyDefinition[]{
				pd_empno, 
				pd_ename, 
				pd_job, 
				pd_mgr, 
				pd_hiredate, 
				pd_comm, 
				pd_deptno, 
		} ;
		
		NodeType objectType = ntManager.getNodeType("_object") ;
		NewNodeType newType = ntManager.createNodeTypeTemplate(objectType, "emp", pd) ;
		newType.setConstraint(pd_empno, new PositiveNumberConstraint()) ;
		newType.setConstraint(pd_ename, new RequireConstraint()) ;
		newType.setConstraint(pd_mgr, new PositiveNumberConstraint()) ;
		newType.setConstraint(pd_deptno, new PositiveNumberConstraint()) ;
		NodeType empType = ntManager.register(newType, false) ; 
		
		Node empParentNode = session.getNode("/scott_emp") ;
		
		// create emp node 
		while (rows.next()) {
	        int empNo = rows.getInt(EMPNO) ;
	        String ename = rows.getString(ENAME) ;
	        String job = rows.getString(JOB) ;
	        int mgr = rows.getInt(MGR) ;
	        Date hireDate = rows.getDate(HIREDATE) ;
	        int comm = rows.getInt(COMM) ;
	        int deptno = rows.getInt(DEPTNO) ;
	        
	        INode node = session.createNode(empParentNode, empType, ename) ;
	        node.setProperty(EMPNO, empNo) ;
	        node.setProperty(ENAME, ename) ;
	        node.setProperty(JOB, job);
	        node.setProperty(MGR, mgr) ;
	        node.setProperty(HIREDATE, hireDate) ;
	        node.setProperty(COMM, comm) ;
	        node.setProperty(DEPTNO, deptno) ;
        }
		
		// set manager link 
		GenericIterator<Node> employeeNode = empParentNode.getChildNode() ;
		LinkType managerLink = session.createLinkType("manager") ;
		while (employeeNode.hasNext()) {
	        Node node = employeeNode.next() ;
	        addLink(node, employeeNode, managerLink, EMPNO, node.getProperty(MGR).getValue().getLong()) ;
        }
		
		session.save() ;
    }
	
	private PropertyDefinition createPropertyDefinition(String pid, Property.Type requireType) throws ALRepositoryException {
	    return session.getNodeTypeManager().createPropertyDefinition(pid, requireType);
    }
	private void addLink(Node oneEmployee, GenericIterator<Node> employeeNode, LinkType linkType, String propId, long value) throws ALRepositoryException {
	    while (employeeNode.hasNext()) {
	        Node enode = employeeNode.next();
			if (enode.getProperty(propId).getValue().getLong() == value){
	        	oneEmployee.addLink(enode, linkType, Position.AFTER) ;
				break ;
	        } ;
        }
    }

}
