package com.bleujin.lore.core.node;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.Serializable;
import java.io.StringReader;

import org.apache.commons.digester.Digester;
import org.apache.ecs.xml.XML;
import org.xml.sax.SAXException;

import com.bleujin.framework.db.procedure.IStringObject;
import com.bleujin.lore.core.common.Publishable;

public abstract class BinaryPropertyValue implements IStringObject, Publishable, Serializable {

	private static final long serialVersionUID = -6065407182228406351L;

	public XML toXML() {
		XML result = new XML("propertyValue");
		result.addAttribute("name", getName());
		result.addAttribute("path", getPath());
		result.addAttribute("size", getSize());
		result.addAttribute("contentType", getContentType());

		return result;
	}

	public final static BinaryPropertyValue toBinaryValue(String xmlValue) {
		try {
			if (xmlValue == null) return BinaryDummyValue.EMPTY ;
			
			Digester digester = getDigester();
			Reader reader = new StringReader(xmlValue);

			return (BinaryDummyValue) digester.parse(reader);
		} catch (IOException e) {
			throw new IllegalArgumentException(e.getMessage());
		} catch (SAXException e) {
			throw new IllegalArgumentException(e.getMessage());
		}
	}

	private static Digester getDigester() {
		Digester digester = new Digester();
		digester.setValidating(false);

		digester.addObjectCreate("propertyValue", BinaryDummyValue.class);
		digester.addSetProperties("propertyValue");

		return digester;
	}
	
	public String getString(){
		return toXML().toString() ;
	}

	public abstract String getPath();

	public abstract String getName();

	public abstract String getContentType();

	public abstract long getSize();

	public abstract InputStream getInputStream() throws FileNotFoundException, IOException;

}


