package com.bleujin.lore.core.node;

import com.bleujin.lore.core.bean.Register;
import com.bleujin.lore.core.common.Acceptable;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.IConstant;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.observation.Event.Mode;

public interface Node extends INode, IDialogue, Acceptable, Cloneable, IConstant {

	public final static String NAME = "name" ;
	public final static String WORKSPACE = "workspace" ;
	public final static String PATH = "path" ;
	public final static String UUID = "uuid" ;
	public final static String STATUS = "status" ;
	public final static String NODETYPE_ID = "ntid" ;
	public final static String PARENT = "parent" ;
	public final static int INIT_VERSION = 0 ;

	public boolean isSame(Node n2);
	public boolean isSameVersion(Node n2);

	public Mode getMode();
	public void setMode(Mode mode) ;
	public int getVersionNum() ;

	public void setRegister(Register register) ;
	public Node newClone(UserSession session) ;

	public INode createChild(NodeType nodetype, String name) throws ALRepositoryException;
	public void addMemberNode(Node member) throws ALRepositoryException;
	public UserSession getSession();

	public void delete() throws ALRepositoryException;
	public void move(Node parent) throws ALRepositoryException ;
	public void copy(Node parent) throws ALRepositoryException ;
	public GenericIterator<Node> searchNode(String searchText) throws ALRepositoryException;
	public GenericIterator<Node> searchNode(String searchText, Page page) throws ALRepositoryException;


	public Lock lock(boolean isDeep) throws ALRepositoryException;
	public boolean isLocked() throws ALRepositoryException;
	public Lock getLock()  throws ALRepositoryException;

	public NodeVersion checkIn() throws ALRepositoryException;
	public VersionHistory getVersionHistory() throws ALRepositoryException;
	
	// getNode & getLink..

	public GenericIterator<Node> getDescentNode(int maxDepth) throws ALRepositoryException ;
	public GenericIterator<Node> getChildNode(Page page) throws ALRepositoryException;
	public GenericIterator<Node> getChildNode() throws ALRepositoryException;

	public GenericIterator<Node> getMemberNode() throws ALRepositoryException ;
	public GenericIterator<Node> getMemberNode(Page page) throws ALRepositoryException ;
	public GenericIterator<Node> getMemberNode(int listNum, int pageNo) throws ALRepositoryException ;

	public GenericIterator<Node> getNodes(LinkType type, Page page) throws ALRepositoryException;
	public GenericIterator<Node> getNodes(String linkTypeName, int listNum, int pageNo) throws ALRepositoryException;
	
	public GenericIterator<Link> getLinks() throws ALRepositoryException ;
	public GenericIterator<Link> getLinks(String linkTypeName) throws ALRepositoryException  ;
	public GenericIterator<Link> getLinks(Page page) throws ALRepositoryException;
	public GenericIterator<Link> getLinks(LinkType linkType, Page page) throws ALRepositoryException  ;

}