package com.bleujin.lore.core.node;

import java.io.Serializable;
import java.util.Map;

import com.bleujin.lore.core.common.Acceptable;
import com.bleujin.lore.core.common.IConstant;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.Publishable;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.value.Value;

public interface Property extends Cloneable, Serializable, IConstant, Publishable, Acceptable {

	public static enum Type {
		STRING, BINARY, LONG, DOUBLE, DATE, BOOLEAN, NAME, PATH, REFERENCE, TEXT, DECIMAL, VIRTUAL_TEXT, UNDEFINED;
		
		public boolean isString(){
			return this.equals(STRING) ;
		}
		public boolean isBinary(){
			return this.equals(BINARY) ;
		}
		public boolean isLong(){
			return this.equals(LONG) ;
		}
		public boolean isDouble(){
			return this.equals(DOUBLE) ;
		}
		public boolean isDate(){
			return this.equals(DATE) ;
		}
		public boolean isBoolean(){
			return this.equals(BOOLEAN) ;
		}
		public boolean isName(){
			return this.equals(NAME) ;
		}
		public boolean isPath(){
			return this.equals(PATH) ;
		}
		public boolean isReference(){
			return this.equals(REFERENCE) ;
		}
		public boolean isText(){
			return this.equals(TEXT) ;
		}
		public boolean isDecimal(){
			return this.equals(DECIMAL) ;
		}
		public boolean isVirtualText(){
			return this.equals(VIRTUAL_TEXT) ;
		}

		public boolean isUndefined(){
			return this.equals(UNDEFINED) ;
		}

		public String toString(){
			return this.name() ;
		}
		
		public static Type fromName(String typeCd) {
			return Property.Type.valueOf(typeCd) ;
		}
	}
	
	public IDString getId();

	public Property newClone() ;
	
	public Value getValue();

	public Property.Type getType() ;
	
	public String getTypeName();

	public String getString();
	
	public Node getNode() ;

	public void setValue(Value value) throws ALRepositoryException;

	public boolean equalValue(Property property);

	public Object getObject() throws ALRepositoryException ;
	
	public Object getObject(Map param) throws ALRepositoryException;

	public Value[] getValues() throws ALRepositoryException;

	public boolean hasArrayValue();

}
