package com.bleujin.lore.core.node;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.observation.Event.Mode;
import com.bleujin.lore.core.workspace.TestCredential;

public class TestOtherSession extends TestCaseParent {

	private String explain = "explain";

	public void testSave() throws Exception {
		Node robot = saveNode(getRobotType(), "abcd") ;
		robot.setProperty(explain, "init") ;
		robot.save() ;
		
		
		// .. not save
		robot.setProperty(explain, "modified") ;
		
		//
		UserSession other = anotherLore.login(new TestCredential("bleujin")) ;
		Node robotByOther = other.getNode("/abcd") ;
		assertEquals("init", robotByOther.getString(explain)) ;
		
		
		
		robot.save() ;
		
	}
	
	
	

	public void testOtherSession() throws Exception {
		createTestNode() ;
		
		UserSession session1 = anotherLore.login(new TestCredential("bleujin")) ;
		Node rootNode = session1.getRootNode();
		assertEquals("", rootNode.getPath()) ;
		Node testFirstNode = session1.getNode("/" + getTestNodeName()) ;
		INode childBySession1 = session1.createNode(testFirstNode, getRobotType(), "one") ;


		assertEquals(0, testFirstNode.getChildNode().size());

		assertEquals(false, session1.existByPath("/" + getTestNodeName() + "/one"));
		assertEquals(childBySession1.getParent(), testFirstNode) ;
		


		UserSession session2 = anotherLore.login(new TestCredential("bleujin"));

		assertEquals("", session2.getRootNode().getPath()) ;
		assertTrue(session1.getLocalStore().equals(session2.getLocalStore())) ;

		assertEquals(0, session2.getNode("/" + getTestNodeName()).getChildNode().size()) ;


		session1.save();

		childBySession1.setProperty(explain, "abc") ;

		Node bySecond = session2.getNode("/" + getTestNodeName() + "/one") ;
		
		assertTrue(childBySession1.getSession() != bySecond.getSession()) ;
		assertTrue(childBySession1.getSession().getLocalStore() == bySecond.getSession().getLocalStore()) ;

		assertEquals("abc", childBySession1.getString(explain)) ;
		assertEquals(null, bySecond.getString(explain)) ;
		
		session1.save() ;

		// save 되었지만 인지하지 못한다..
		assertEquals(null, bySecond.getString(explain)) ;
    }

	
	public void testProperty() throws Exception {
		UserSession firstSession = anotherLore.login(new TestCredential("bleujin")) ;
		Node rootNode = firstSession.getRootNode() ;
		Node oneNodeByFirst = firstSession.createNode(rootNode, getRobotType(), "one").save() ;
		oneNodeByFirst.setProperty(explain, "abc") ;
		firstSession.save() ;
		
		
		Property explainByFirst = oneNodeByFirst.getProperties().next();
		assertEquals("abc", explainByFirst.getString());
		oneNodeByFirst.setProperty(explain, "zzz") ;

		// explainByFirst = oneNodeByFirst.getProperties().next();
		assertEquals("zzz", oneNodeByFirst.getString(explain));

		assertTrue(explainByFirst == oneNodeByFirst.getProperty(explain));
		assertEquals("zzz", explainByFirst.getString());
		
		firstSession.save() ;
		
		UserSession secondSession = anotherLore.login(new TestCredential("bleujin"));
		Node oneNodeBySecond = secondSession.getNode("/one") ;
		
		assertEquals("zzz", oneNodeBySecond.getProperties().next().getString()) ;
	    
    }

	public void testExistSession() throws Exception {
		Node rootNode = session.getRootNode() ;
		Node childNodeByFirst = session.createNode(rootNode, getRobotType(), "child").save() ;
		//Node gchildNodeByFirst = firstSession.createNode(childNodeByFirst, getRobotType(), "gchild") ;
		assertTrue(rootNode.getSession() == session) ;

		session.save() ;
		assertEquals(1, rootNode.getChildNode().size()) ;
	}
	
	public void testNewSession() throws Exception {
		UserSession firstSession = anotherLore.login(new TestCredential("b")) ;
		
		Node firstNode = firstSession.getNode("");
		Node childNodeByFirst = firstSession.createNode(firstNode, getRobotType(), "child").save() ;
		
		assertTrue(firstNode.getSession() == firstSession) ;
		
		firstSession.save() ;
		assertEquals(1, firstNode.getChildNode().size()) ;
		firstSession.logout() ;
	}
	
	public void xtestLink2() throws Exception {
		Node testFirstNode = createTestNode() ;

		UserSession firstSession = anotherLore.login(new TestCredential("bleujin")) ;
		
		Node rootNode = firstSession.getRootNode();
		Node childNodeByFirst = firstSession.createNode(testFirstNode, getRobotType(), "child").save() ;
		Node gchildNodeByFirst = firstSession.createNode(childNodeByFirst, getRobotType(), "gchild").save() ;

		firstSession.save() ;
		
		assertEquals(1, rootNode.getChildNode().size()) ;
		assertEquals(1, rootNode.getChildNode().next().getChildNode().size()) ;
		
		
		UserSession secondSession = anotherLore.login(new TestCredential("bleujin")) ; 
		assertEquals(1, secondSession.getRootNode().getChildNode().size()) ;
		secondSession.save(); 

		assertEquals(1, secondSession.getRootNode().getChildNode().size()) ;


		// delete node 
		childNodeByFirst.delete() ;
		assertEquals(Mode.DELETED, childNodeByFirst.getMode()) ; 
		assertEquals(1, rootNode.getChildNode().next().getChildNode().size()) ;
		
		assertEquals(1, rootNode.getChildNode().size()) ;
    }
	
}
