package com.bleujin.lore.core.nodetype;

import java.util.ArrayList;
import java.util.List;

import org.apache.ecs.xml.XML;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.Publishable;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.mapping.IDefaultValue;
import com.bleujin.lore.core.mapping.IValueConstraint;
import com.bleujin.lore.core.mapping.ValidMessage;
import com.bleujin.lore.core.node.INodeCommon;
import com.bleujin.lore.core.node.Property.Type;
import com.bleujin.lore.core.persitence.GlobalStore;
import com.bleujin.lore.core.value.Value;

public abstract class AbstractNodeType implements NodeType, Publishable {

	
	public String getName(){
		return getId().getString() ;
	}
	
	public MappedDefinition getMappedDefinition(String pdId) throws ALRepositoryException {
		return getMappedDefinition(IDString.propId(pdId)) ;
	}
	public abstract GlobalStore getGlobalStore() ;


	@Override
	public boolean equals(Object obj){
		if (obj == null || ! (obj instanceof NodeType)) return false ;
		NodeType that = (NodeType)obj ;
		
		return getId().equals(that.getId()) ;
	}

	@Override
	public int hashCode(){
		return getId().hashCode() ;
	}
	
	public String toString() {
		return getClass().getName() + "[id:"  + getName() + "]";
	}
	
	public boolean isNodeType(NodeType thatType) throws ALRepositoryException {
		return isNodeType(thatType.getId());
	}
	
	public boolean containsPropId(String propId)  throws ALRepositoryException {
		return containsPropId(IDString.propId(propId)) ;
	}
	public abstract boolean containsPropId(IDString propId)  throws ALRepositoryException ;

	
	public boolean isAllowedMemberType(NodeType nodeType) throws ALRepositoryException {
		return getMemberTypeId() == NodeTypeId.NO_SUPERTYPE_ID ||  nodeType.isNodeType(getMemberTypeId().getString()) ;
    }

	public boolean isNodeType(String ntName) throws ALRepositoryException {
		return isNodeType(getGlobalStore().toNodeTypeId(ntName)) ;
	}
	
	
	private boolean isNodeType(NodeTypeId thatId) throws ALRepositoryException {
		NodeType thisType = this;
		while (thisType != NodeType.NO_SUPERTYPE) {
			if (thisType.getId().equals(thatId)) {
				return true;
			}
			thisType = thisType.getSuperType();
		}

		return false;
	}
	
	public NodeType setConstraint(String propId, IValueConstraint constraint)  throws ALRepositoryException {
		return setConstraint(getMappedDefinition(propId), constraint) ;
	}

	public NodeType setDefaultValue(String propId, IDefaultValue defaultValue)  throws ALRepositoryException {
		return setDefaultValue(getMappedDefinition(propId), defaultValue) ;
	}


	public NodeType setConstraint(PropertyDefinition prop, IValueConstraint constraint) throws ALRepositoryException {
		getMappedDefinition(prop.getName()).setConstraint(constraint) ;
		getGlobalStore().setPropertyConstraint(this, prop, constraint) ;
		return this ;
	}
	
	
	public NodeType setDefaultValue(PropertyDefinition prop, IDefaultValue defaultValue)  throws ALRepositoryException {
		Type requiredType = getMappedDefinition(prop.getName()).requiredType();
		if (! defaultValue.isAcceptableType(requiredType)) throw new ALRepositoryException("cannt acceptable type:" + requiredType + "-" + defaultValue.getValue());
		getMappedDefinition(prop.getName()).setDefaultValue(defaultValue) ;
		getGlobalStore().setPropertyDefaultValue(this, prop, defaultValue) ;
		return this ;
	}
	

	public XML toXML() throws ALRepositoryException {
		XML result = new XML(XML_TAG_ROOT) ;
		XML typeinfo = new XML(XML_TAG_TYPEINFO) ;
		typeinfo.addAttribute(XML_TAG_ID, getName()) ;
		typeinfo.addAttribute(XML_TAG_SUPERTYPE, getSuperType().getName()) ;
		typeinfo.addAttribute(XML_TAG_MEMBERTYPE, getMemberTypeId().getString()) ;
		
		result.addElement(typeinfo) ;
		
		GenericIterator<MappedDefinition> mds = getMappedDefinitions();
		for(MappedDefinition md : mds){
			result.addElement(md.toXML()) ;
		}		

		return result ;
	}
	
	public GenericIterator<ValidMessage> getViolateConstraintMessage(INodeCommon node) throws ALRepositoryException{
		
		GenericIterator<MappedDefinition> mds = getGlobalStore().getMappedDefinition(this) ;
		
		List<ValidMessage> message = new ArrayList<ValidMessage>() ;
		for(MappedDefinition md : mds){
			if (md.getConstraint() == null) continue ;
			Value value = node.getProperty(md.getName()).getValue();
			ValidMessage vm = md.getConstraint().getValidMessage(md.getId(), value) ;
			if (! vm.isValid()) {
				message.add(vm) ;
			} 
		}

		return new GenericIteratorHelper<ValidMessage>(message) ;
	}


}
