package com.bleujin.lore.core.nodetype;

import org.apache.ecs.xml.XML;

import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.Publishable;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.mapping.IDefaultValue;
import com.bleujin.lore.core.mapping.IValueConstraint;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.Property.Type;
import com.bleujin.lore.core.value.Value;

public class MappedDefinition implements PropertyDefinition, Publishable {

	private static final long serialVersionUID = 8100483610014104112L;
	private PropertyDefinition prop;
	private IValueConstraint constraint;
	private IDefaultValue defaultValue;

	private MappedDefinition(PropertyDefinition prop, IValueConstraint constraint, IDefaultValue defaultValue) {
		this.prop = prop;
		this.constraint = constraint;
		this.defaultValue = defaultValue;
	}

	public final static MappedDefinition create(PropertyDefinition prop) {
		return new MappedDefinition(prop, IValueConstraint.NONE, IDefaultValue.NONE);
	}

	public final static MappedDefinition load(PropertyDefinition prop, IValueConstraint constraint, IDefaultValue defaultValue) {
		return new MappedDefinition(prop, constraint, defaultValue);
	}

	public Value getDefaultValue() {
		if (defaultValue == null || defaultValue == IDefaultValue.NONE) {
			if (requiredType() == Property.Type.DATE) {
				return Value.dateValue(null);
			} else if (requiredType() == Property.Type.BINARY) {
				return Value.binaryValue(null);
			}
			return Value.newValue(requiredType(), null);
		} else {
			return defaultValue.getValue() ;
		}
	}

	public IDefaultValue getDefaultConstraint(){
		return defaultValue ;
	}
	
	public IDString getId() {
		return prop.getId();
	}
	
	public String getName() {
		return getId().getString() ;
	}

	public boolean isSameProperty(IDString propId) {
		return prop.isSameProperty(propId);
	}

	public Type requiredType() {
		return prop.requiredType();
	}

	public void setConstraint(IValueConstraint constraint) {
//		if (this.constraint != null && (this.constraint != IValueConstraint.NONE)){
//			throw new ALRepositoryException(this.constraint + " constraint already exist at " + getName() ) ;
//		}
		
		this.constraint = constraint;
	}

	public void setDefaultValue(IDefaultValue defaultValue) {
		this.defaultValue = defaultValue;
	}

	public IValueConstraint getConstraint() {
		return constraint;
	}

	public PropertyDefinition getPropertyDefinition() {
		return prop;
	}

	public XML toXML() throws ALRepositoryException {
		XML result = new XML(getName()) ;
		XML constraintXML = new XML("constraint") ;
		constraintXML.addElement(constraint == null ? "" : constraint.getConfigurationExpression());
		XML defaultXML = new XML("default") ;
		defaultXML.addElement(defaultValue == null ? "" : defaultValue.getConfigurationExpression());
		
		result.addAttribute("id", getName()) ;
		result.addAttribute("required", requiredType().toString()) ;
		result.addElement(defaultXML) ;
		result.addElement(constraintXML) ;

		return result ;
	}

}
