package com.bleujin.lore.core.nodetype;

import java.io.Serializable;
import java.util.ArrayList;

import org.apache.ecs.xml.XML;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.common.Publishable;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ALRepositoryRuntimeException;
import com.bleujin.lore.core.exception.NoSuchPropertyDefinitionException;
import com.bleujin.lore.core.mapping.ValidMessage;
import com.bleujin.lore.core.node.INodeCommon;
import com.bleujin.lore.core.persitence.GlobalStore;

public interface NodeType extends Serializable, Publishable, NodeTypeDefinition {

	public final static String XML_TAG_ROOT = "nodetype" ; 
	public final static String XML_TAG_TYPEINFO = "type-info" ; 
	public final static String XML_TAG_ID = "id" ; 
	public final static String XML_TAG_SUPERTYPE = "supertype" ; 
	public final static String XML_TAG_MEMBERTYPE = "memgertype" ; 
	
	public NodeTypeId getId() ;

	public boolean isNodeType(NodeType thatType)  throws ALRepositoryException ;
	public boolean isNodeType(String thatTypeName)  throws ALRepositoryException ;
	public boolean isAllowedMemberType(NodeType memberType)  throws ALRepositoryException ;
	public GenericIterator<ValidMessage> getViolateConstraintMessage(INodeCommon abstractShareNode) throws ALRepositoryException;

	public NewNodeType updateFor() throws ALRepositoryException ;
	
	
//	public GenericIterator<NodeType> getSubTypes() throws ALRepositoryException ;
//	public GenericIterator<Node> getNodes() throws ALRepositoryException ;

	public final static NodeType NO_SUPERTYPE = new AbstractNodeType(){

		private static final long serialVersionUID = -2842054943160457646L;
		private final static String EXCEPTION_MESSAGE = "this is GHOST ROOT NODE" ;
		
		public boolean containsPropId(IDString propId) throws ALRepositoryException {
			return false;
		}

		public NodeTypeId getId() {
			return NodeTypeId.NO_SUPERTYPE_ID;
		}

		public NodeTypeId getMemberTypeId() throws ALRepositoryException {
			return NodeTypeId.NO_SUPERTYPE_ID;
		}

		public GenericIterator<MappedDefinition> getMappedDefinitions() throws ALRepositoryException {
			return new GenericIteratorHelper<MappedDefinition>(new ArrayList<MappedDefinition>());
		}

		public MappedDefinition getMappedDefinition(IDString propId) throws ALRepositoryException {
			throw NoSuchPropertyDefinitionException.create(propId) ;
		}

		public NodeType getSuperType() throws ALRepositoryException {
			throw new ALRepositoryRuntimeException(EXCEPTION_MESSAGE) ;
		}

		public boolean isAllowedMemberType(NodeType memberType) throws ALRepositoryException {
			return true ;
		}

		public boolean isNodeType(NodeType thatType) throws ALRepositoryException {
			return this == thatType;
		}

		public boolean isNodeType(String thatTypeId) throws ALRepositoryException {
			return "*".equals(thatTypeId);
		}

		public void setMemberType(NodeType memberType) throws ALRepositoryException {
			throw new ALRepositoryRuntimeException(EXCEPTION_MESSAGE) ;
		}

		public GenericIterator<ValidMessage> getViolateConstraintMessage(INodeCommon node) throws ALRepositoryException {
			return new GenericIteratorHelper(new ArrayList<ValidMessage>()) ;
		}

		public XML toXML() throws ALRepositoryException {
			throw new ALRepositoryRuntimeException(EXCEPTION_MESSAGE) ;
		}
		
		public void setPrimary(PropertyDefinition pd) throws NoSuchPropertyDefinitionException{
			// ignore ;
		}

		@Override
		public GlobalStore getGlobalStore() {
			throw new ALRepositoryRuntimeException(EXCEPTION_MESSAGE) ;
		}
		public PropertyDefinition getPrimaryProperty() throws NoSuchPropertyDefinitionException, ALRepositoryException {
			throw new ALRepositoryRuntimeException(EXCEPTION_MESSAGE) ;
		}

		public NewNodeType updateFor() {
			throw new ALRepositoryRuntimeException(EXCEPTION_MESSAGE) ;
		}

		public String getPrimaryPropertyId() {
			throw new ALRepositoryRuntimeException(EXCEPTION_MESSAGE) ;
		}
		
		public String toString(){
			return "*" ;
		}
	} ;



	
}
