package com.bleujin.lore.core.nodetype;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.exception.ALRepositoryException;



/**
 * The <code>NodeTypeDefinition</code> interface provides methods for
 * discovering the static definition of a node type. These are accessible both
 * before and after the node type is registered. Its subclass
 * <code>NodeType</code> adds methods that are relevant only when the node type
 * is "live"; that is, after it has been registered. Note that the separate
 * <code>NodeDefinition</code> interface only plays a significant role in
 * implementations that support node type registration. In those cases it serves
 * as the superclass of both <code>NodeType</code> and
 * <code>NodeTypeTemplate</code>. In implementations that do not support node
 * type registration, only objects implementing the subinterface
 * <code>NodeType</code> will be encountered.
 *
 * @since JCR 2.0
 */
public interface NodeTypeDefinition {
	
	public NodeTypeId getId() ;
	public String getName() ;
	public NodeType getSuperType()  throws ALRepositoryException ;

	public GenericIterator<MappedDefinition> getMappedDefinitions() throws ALRepositoryException;
	public MappedDefinition getMappedDefinition(IDString propId) throws ALRepositoryException ;
	public MappedDefinition getMappedDefinition(String propId) throws ALRepositoryException ;

	public NodeTypeId getMemberTypeId() throws ALRepositoryException ;

	public boolean containsPropId(String propName)  throws ALRepositoryException ;
	public String getPrimaryPropertyId() ;
	
}
