package com.bleujin.lore.core.observation;

import java.util.Calendar;


public abstract class Event {
	
	public enum Mode {
		CREATED, UPDATED, MOVED, CHECKOUT, DELETED, NORMAL ;
		
		public boolean isMoved(){
			return this.equals(MOVED) ;
		}
		
		public boolean isDeleted(){
			return this.equals(DELETED) ;
		}
		
		public boolean isNormal(){
			return this.equals(NORMAL) ;
		}
		
		public boolean isUpdated(){
			return this.equals(UPDATED) ;
		}
		
		public boolean isCreated(){
			return this.equals(CREATED) ;
		}

		public boolean isCheckOut() {
			return this.equals(CHECKOUT) ;
		}
		
	}
	
	public final static EventType[] NODE_EVENT = new EventType[]{EventType.NODE_ADDED, EventType.NODE_REMOVED} ;
	public final static EventType[] PROPERTY_EVENT = new EventType[]{EventType.PROPERTY_ADDED, EventType.PROPERTY_CHANGED, EventType.PROPERTY_VERSION_CHANGED} ;
	public final static EventType[] LINK_EVENT = new EventType[]{EventType.LINK_ADDED, EventType.LINK_REMOVED} ;
	
	public enum EventType {
		NODE_ADDED, NODE_UPDATED, NODE_REMOVED, NODE_MOVED, 
		PROPERTY_ADDED, PROPERTY_CHANGED, PROPERTY_VERSION_CHANGED,  
		LINK_ADDED, LINK_REMOVED,
		NODE_CHECKOUTED;

		public String toString() {
			return this.name().toLowerCase();
		}
	}

	private UserBean triggerUser ;
	private EventType eventType ;
	private Calendar current ; 
	protected Event(UserBean triggerUser, EventType eventType) {
		this.triggerUser = triggerUser ;
		this.eventType = eventType ;
		this.current = Calendar.getInstance() ;
		
    }

	public EventType getType() {
	    return eventType;
    }

	public UserBean getTriggerUser() {
	    return triggerUser;
    }
	
	public long getDate(){
		return current.getTimeInMillis();
	}

}
