package com.bleujin.lore.core.observation;

import java.util.TimeZone;

import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.link.LinkType.Position;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.observation.Event.EventType;

public class TestObserver extends TestCaseParent{

	private NodeType newType ;
	
	public void setUp() throws Exception {
		super.setUp() ;
		newType = registerNodeType(objectType, "person", new PropertyDefinition[]{createPropertyDefinition("firstName", Property.Type.STRING)});
	}
	
	public void xtestGregorian() throws Exception {
		String[] timezone = TimeZone.getAvailableIDs() ;
		Debug.debug(this, timezone);
	}
	
	public void testEnum() throws Exception {
		assertEquals("node_added", EventType.NODE_ADDED.toString()) ;
    }
	
	public void testObserver() throws Exception {
		Node testFirstNode = createTestNode() ;
		EventStackingListener listener = new EventStackingListener();
		workspace.getObservationManager().addEventListener(listener) ;
		
		Node node = saveNode(newType, "abc") ;
		
		assertEquals(2, listener.getCount())  ;

		assertEquals(EventType.PROPERTY_ADDED, listener.popEvent().getType()) ; // because of default value

		NodeEvent event = (NodeEvent)listener.popEvent();
		assertEquals(EventType.NODE_ADDED, event.getType()) ;
		assertEquals(node, event.getNode()) ;
		
		
		node.setProperty("firstname", "bleujin") ;
		assertEquals(EventType.PROPERTY_CHANGED, listener.popEvent().getType()) ;

		
		LinkType diffLink = workspace.getGlobalStore().createLinkType("diff");
		node.addLink(testFirstNode, diffLink, Position.AFTER) ;
		assertEquals(EventType.LINK_ADDED, listener.popEvent().getType()) ;

		
		session.save() ;
		assertEquals(1, node.getNodes(diffLink, Page.ALL).size()) ;
		
		NodeVersion v1 = node.checkIn() ;
		v1.checkOut() ;
		assertEquals(1, listener.getCount())  ;
		assertEquals(EventType.NODE_CHECKOUTED, listener.popEvent().getType());
		
		node.delete() ;
		assertEquals(1, listener.getCount())  ;
		assertEquals(EventType.NODE_REMOVED, listener.popEvent().getType()) ;
	}

	
	public void testMemeber() throws Exception {
		EventStackingListener listener = new EventStackingListener();
		workspace.getObservationManager().addEventListener(listener) ;

		Node a = saveNode(objectType, "a") ;
		Node b = saveNode(objectType, "b") ;
		Node c = saveNode(objectType, "c") ;
		Node d = saveNode(objectType, "d") ;
		
		assertEquals(4, listener.getCount()) ; // node_create + link_create(child)
		listener.clearStack() ;
		
		a.addMemberNode(b) ;
		a.addMemberNode(c) ;
		a.addMemberNode(d) ;
		assertEquals(3, listener.getCount()) ;
		
    }
	
	public void testFileName() throws Exception {
		assertEquals(1, nextInt()) ;
		assertEquals(2, nextInt()) ;
		assertEquals(3, nextInt()) ;
		assertEquals(4, nextInt()) ;
		assertEquals(0, nextInt()) ;
		assertEquals(1, nextInt()) ;
	}
	
	private int index = 0 ;
	private int nextInt(){
		index = (++index >= 5) ? index % 5 : index ;
		return index ;
	}
	
	public void testMultipleObservers() throws Exception {
		EventStackingListener listener1 = new EventStackingListener();
		EventStackingListener listener2 = new EventStackingListener();

		workspace.getObservationManager().addEventListener(listener1) ;
		workspace.getObservationManager().addEventListener(listener2) ;

		
		saveNode(objectType, "abc") ;
		assertEquals(1, listener1.getCount())  ;
		assertEquals(listener1.getCount(), listener2.getCount())  ;

	}
}
