package com.bleujin.lore.core.persitence;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.dimpl.ShareInfo;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.NamespaceException;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.mapping.IDefaultValue;
import com.bleujin.lore.core.mapping.IValueConstraint;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property.Type;
import com.bleujin.lore.core.nodetype.MappedDefinition;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NodeTypeId;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.extend.template.Template;
import com.bleujin.lore.extend.template.TemplateId;


public abstract class GlobalStore {

	public abstract NewNodeType createNodeType(NodeType superType, String ntName, PropertyDefinition[] definitions) throws ALRepositoryException ;
	
	
	public abstract boolean includeNodeType(String ntName) throws ALRepositoryException ;
	public abstract GenericIterator<NodeType> getAllNodeTypes(String prefixId) throws ALRepositoryException ;
	// public abstract GenericIterator<LinkType> getAllLinkTypes() throws RepositoryException ;

	
	
	public abstract NodeType getNodeType(String ntName) throws NamespaceException, ALRepositoryException ;
	public abstract GenericIterator<NodeType> register(NewNodeType[] newNodeTypes, boolean allowUpdate) throws ALRepositoryException ;
	
	public abstract void removeNodeType(String ntName) throws ALRepositoryException  ;
	public abstract GenericIterator<MappedDefinition> getMappedDefinition(NodeType nodetype) throws ALRepositoryException;
	// public abstract PropertyDefinition getPropertyDefinition(String pid) throws RepositoryException;
	public abstract void setPropertyConstraint(NodeType nodetype, PropertyDefinition prop, IValueConstraint constraint) throws ALRepositoryException;
	public abstract void setPropertyDefaultValue(NodeType nodetype, PropertyDefinition prop, IDefaultValue defaultValue) throws ALRepositoryException ;
	public abstract PropertyDefinition createPropertyDefinition(String propName, Type type) throws ALRepositoryException ;

	public abstract LinkType getLinkType(String linkTypeName) throws ALRepositoryException ;
	public abstract LinkType createLinkType(String linkTypeName) throws ALRepositoryException ;

	
	public abstract Sequence getSequence() ;
	
	public abstract NamespaceRegistry getNamespace() ;

	public abstract void setMemberType(NodeType nodetype, NodeType membertype) throws ALRepositoryException;
	
	public abstract ShareInfo getShareInfo(String shareName) throws ALRepositoryException ;
	public abstract GenericIterator<ShareInfo> getShareInfos() throws ALRepositoryException ;
	public abstract void shareNode(String shareName, String workspaceName, String nodePath, boolean isDeep)  ;


	public abstract Template saveTemplate(TemplateId templateId, Template Template)  throws ALRepositoryException;
	public abstract void bindTemplate(Node node, TemplateId tplId)  throws ALRepositoryException;
	public abstract void bindTemplate(NodeType nodeType, TemplateId tplId)  throws ALRepositoryException;
	public abstract Template getBoundTemplate(Node node)  throws ALRepositoryException;
	public abstract Template getTemplate(UserSession session, TemplateId tplId)  throws ALRepositoryException;
	public abstract GenericIterator<TemplateId> getTemplates(UserSession session)  throws ALRepositoryException;

	public abstract void close()  ;

	
	
	public abstract void clearStore() throws ALRepositoryException;
	public abstract void confirm() ;

	protected abstract int addURIAddress(String prefix, String address)  ;
	protected abstract void removeURIAddress(String prefix) ;
	protected abstract boolean existURIAddress(String prefix) throws NamespaceException ;
	protected abstract URI getURIAddress(String prefix) throws NamespaceException;
	protected abstract GenericIterator<URI> listURIAddress() ;
	
	
	public abstract NodeTypeId toNodeTypeId(String nodeTypeName) throws NamespaceException ;
	public abstract void setPrimaryProperty(NodeType nodetype, PropertyDefinition pd) throws ALRepositoryException;
}
