package com.bleujin.lore.core.persitence;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.dimpl.ShareInfo;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.SearchException;
import com.bleujin.lore.core.link.Link;
import com.bleujin.lore.core.link.LinkType;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeName;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.VersionHistory;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.observation.Event;
import com.bleujin.lore.core.search.indexer.NodeIndexer;
import com.bleujin.lore.core.search.searcher.NodeSearcher;
import com.bleujin.lore.core.value.Value;

public interface ILocalStore {

	
	public NodeIndexer getIndexer() ;
	public NodeSearcher getSearcher(UserSession session, Node startNode) ; 
	
	
	public INode createNode(UserSession session, Node parent, NodeType nodetype, NodeName name, String explain) ;
	
	public GenericIterator<Property> getProperties(Node node) throws ALRepositoryException;

	public GenericIterator<Node> getChildNodes(Node node, Page page) throws ALRepositoryException;
	
	public GenericIterator<Node> getNodes(Node node, LinkType type, Page page) throws ALRepositoryException;

	public GenericIterator<Node> getNodes(UserSession session, NodeType nodetype, boolean includeSub, Page page) throws ALRepositoryException;

	public Node getNodeByPath(UserSession session, String path) throws ALRepositoryException;

	public Node getNodeByUUID(UserSession session, String uuid) throws ALRepositoryException;

	public boolean exist(UserSession session, String path) throws ALRepositoryException;

	public GenericIterator<Link> getLinks(Node node, Page page) throws ALRepositoryException;

	public GenericIterator<Link> getLinks(Node node, LinkType linkType, Page page) throws ALRepositoryException;

	public VersionHistory getVersionHistory(Node node) throws ALRepositoryException;

	public void move(Node src, Node destParent) throws ALRepositoryException;

	public void copy(Node src, Node destParent) throws ALRepositoryException;

	public void indexNode(List<Event> pendingEvent, Node node) throws ALRepositoryException;

	public void indexNode(List<Event> pendingEvent, Collection<Node> thisNodes, Map<String, Link> thisLinks) throws ALRepositoryException;

	public GenericIterator<Node> searchNode(UserSession session, Node startNode, String searchText, Page page) throws SearchException, ALRepositoryException ;

	public void clear()  ;

	public Node getIternalShareNode(UserSession session, String workspaceName, ShareInfo shareInfo)  throws ALRepositoryException;

	public Value[] getValues(Node node, Property property) throws ALRepositoryException ;

	public void close();
	
	public void save(List<Event> pendingEvent) throws ALRepositoryException;


	public void save(List<Event> pendingEvent, Node thisNode) throws ALRepositoryException;

	// for test
	public void confirm(UserSession session) throws ALRepositoryException;


}
