package com.bleujin.lore.core.search.indexer;

import java.io.File;
import java.io.IOException;

import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.IndexInput;
import org.apache.lucene.store.IndexOutput;
import org.apache.lucene.store.Lock;
import org.apache.lucene.store.RAMDirectory;

import com.bleujin.lore.core.search.ISearcher;

public class IndexDirectory extends Directory{

	private String path ;
	private Directory inner ;
	
	private IndexDirectory(String path, Directory dir){
		File file = new File(path) ;
		if(! file.exists()) file.mkdirs() ;
		this.path = path ;
		this.inner = dir ;
	}
	
	private IndexDirectory() {
		this("/anotherlore/data/index", new RAMDirectory()) ;
	}
	
	public final static IndexDirectory createRamIndexDir(){
		return new IndexDirectory() ;
	}
	
	private IndexDirectory(String path) throws IOException {
        this(path, FSDirectory.getDirectory(path, ISearcher.DIRECTORY_APPEND)) ;
        if (inner.list().length == 0) { // if index file not exist..
        	inner.close();
        	inner = FSDirectory.getDirectory(path, ISearcher.DIRECTORY_CREATE);
        }
    }
	public final static IndexDirectory createIndexDirectory(String path) throws IOException{
		return new IndexDirectory(path) ;
	}
	
	public final static IndexDirectory createIndexDirectory(String path, String indexName) throws IOException{
		return new IndexDirectory(path +"/" + indexName) ;
	} 
	
	public String getIndexPath(){
		return this.path ;
	}
	
    public void close() throws IOException {
    	inner.close() ;
    }

    public void deleteFile(String file) throws IOException {
	    inner.deleteFile(file) ;
    }

    public boolean fileExists(String file) throws IOException {
	    return inner.fileExists(file) ;
    }

    public long fileLength(String file) throws IOException {
    	return inner.fileLength(file) ;
    }

    public long fileModified(String file) throws IOException {
    	return inner.fileModified(file) ;
    }

    public String[] list() throws IOException {
    	return inner.list() ;
    }

    public Lock makeLock(String file) {
	    return inner.makeLock(file);
    }

    public void renameFile(String source, String target) throws IOException {
    	inner.renameFile(source, target) ;
    }

    public void touchFile(String file) throws IOException {
		inner.touchFile(file) ;
    }

    
	public IndexOutput createFile(String name) throws IOException {
		return inner.createOutput(name) ;
	}

	public IndexOutput createOutput(String name) throws IOException {
		return inner.createOutput(name);
	}

	public IndexInput openFile(String name) throws IOException {
		return inner.openInput(name);
	}

	public IndexInput openInput(String name) throws IOException {
		return inner.openInput(name);
	}
}
