package com.bleujin.lore.core.search.limo;

import java.io.File;

public class IndexFileType {

    public static IndexFileType getIndexFileType(File file) {
        for (int index = 0; index < ALL.length; index++) {
            IndexFileType type = ALL[index];
            if (type.isType(file))
                return type;
        }

        return UNKNOWN;
    }

    private IndexFileType(String name, String endsWith) {
        this.name = null;
        this.endsWith = null;
        this.name = name;
        this.endsWith = endsWith;
    }

    public String getName() {
        return name;
    }

    public boolean isType(File file) {
        if (equals(NORMALIZATION_FACTORS)) {
            int extIndex = file.getName().lastIndexOf(".");
            if (extIndex == -1)
                return false;
            String ext = file.getName().substring(extIndex + 1);
            if (!ext.startsWith("f") || ext.length() < 2)
                return false;
            for (int index = 1; index < ext.length(); index++) {
                char extChar = ext.charAt(index);
                if (extChar != '0' && extChar != '1' && extChar != '2' && extChar != '3' && extChar != '4' && extChar != '5' && extChar != '6' &&
                    extChar != '7' && extChar != '8' && extChar != '9')
                    return false;
            }

            return true;
        } else {
            return file.getName().endsWith(endsWith);
        }
    }

    public String toString() {
        return getName();
    }

    public static final IndexFileType SEGMENTS;
    public static final IndexFileType DELETABLE;
    public static final IndexFileType FIELD_INFO;
    public static final IndexFileType STORED_FIELD_INDEX;
    public static final IndexFileType STORED_FIELD_DATA;
    public static final IndexFileType TERM_INFO;
    public static final IndexFileType TERM_INFO_INDEX;
    public static final IndexFileType TERM_FREQUENCIES;
    public static final IndexFileType TERM_POSITIONS;
    public static final IndexFileType NORMALIZATION_FACTORS;
    public static final IndexFileType TERM_VECT_DOC_INDEX;
    public static final IndexFileType TERM_VECT_DOCS;
    public static final IndexFileType TERM_VECT_FIELDS;
    public static final IndexFileType DELETIONS;
    public static final IndexFileType UNKNOWN = new IndexFileType("Unknown", "");
    public static final IndexFileType ALL[];
    private String name;
    private String endsWith;

    static {
        SEGMENTS = new IndexFileType("Segments List", "segments");
        DELETABLE = new IndexFileType("Deletable List", "deletable");
        FIELD_INFO = new IndexFileType("Field Info", ".fnm");
        STORED_FIELD_INDEX = new IndexFileType("Stored Field Index", ".fdx");
        STORED_FIELD_DATA = new IndexFileType("Stored Field Data", ".fdt");
        TERM_INFO = new IndexFileType("Term Info", ".tis");
        TERM_INFO_INDEX = new IndexFileType("Term Info Index", ".tii");
        TERM_FREQUENCIES = new IndexFileType("Term Frequencies", ".frq");
        TERM_POSITIONS = new IndexFileType("Term Positions", ".prx");
        NORMALIZATION_FACTORS = new IndexFileType("Normalization Factors", "");
        TERM_VECT_DOC_INDEX = new IndexFileType("Term Vector Document Index", ".tvx");
        TERM_VECT_DOCS = new IndexFileType("Term Vector Documents", ".tvd");
        TERM_VECT_FIELDS = new IndexFileType("Term Vector Fields", ".tvf");
        DELETIONS = new IndexFileType("Deletions", ".del");
        ALL = (new IndexFileType[] {
               SEGMENTS, DELETABLE, FIELD_INFO, STORED_FIELD_INDEX, STORED_FIELD_DATA, TERM_INFO, TERM_INFO_INDEX, TERM_FREQUENCIES, TERM_POSITIONS,
               NORMALIZATION_FACTORS,
               TERM_VECT_DOC_INDEX, TERM_VECT_DOCS, TERM_VECT_FIELDS, DELETIONS
        });
    }
}
