package com.bleujin.lore.core.search.limo;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.PhraseQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.highlight.Highlighter;
import org.apache.lucene.search.highlight.QueryScorer;
import org.apache.lucene.search.highlight.SimpleHTMLFormatter;

// Referenced classes of package net.sourceforge.limo:
//            ReconstructedDocument, IndexFile

public final class LimoUtils {

    public LimoUtils() {
    }

    public static final int estimateMemory(Query query, int numFields, int numDocs) {
        int memory = 0;
        if (query instanceof BooleanQuery) {
            BooleanQuery boolQuery = (BooleanQuery) query;
            BooleanClause clauses[] = boolQuery.getClauses();
            for (int index = 0; index < clauses.length; index++) {
                BooleanClause clause = clauses[index];
                memory += estimateMemory(clause.getQuery(), numFields, numDocs);
            }

        } else
        if (query instanceof TermQuery)
            memory = numFields * numDocs + 1024;
        else
        if (query instanceof PhraseQuery) {
            int numPhraseTerms = ( (PhraseQuery) query).getTerms().length;
            memory = numFields * numDocs + 1024 * numPhraseTerms;
        } else {
            throw new IllegalArgumentException("Your query is of type " + query.getClass().getName() + " which shouldn't be here. " +
                                               "Make sure your query is rewriten before using this " + "method.");
        }
        return memory;
    }

    public static final int getTermCount(Query query) {
        int termCount = 0;
        if (query instanceof BooleanQuery) {
            BooleanQuery boolQuery = (BooleanQuery) query;
            BooleanClause clauses[] = boolQuery.getClauses();
            for (int index = 0; index < clauses.length; index++)
                termCount += getTermCount(clauses[index].getQuery());

        } else
        if (query instanceof TermQuery)
            termCount++;
        else
        if (query instanceof PhraseQuery)
            termCount += ( (PhraseQuery) query).getTerms().length;
        else
            throw new IllegalArgumentException("Your query is of type " + query.getClass().getName() + " which shouldn't be here. " +
                                               "Make sure your query is rewriten before using this " + "method.");
        return termCount;
    }

    public static final String highlightTerms(Query query, String text, String fieldname, Analyzer analyzer) throws IOException {
        String matches;
        matches = "";
        if (text == null || text.equals(""))
            return "";
        org.apache.lucene.analysis.TokenStream tokens = analyzer.tokenStream(fieldname, new StringReader(text));
        org.apache.lucene.search.highlight.Formatter formatter = new SimpleHTMLFormatter("<span class=\"highlightedText\">", "</span>");
        org.apache.lucene.search.highlight.Scorer scorer = new QueryScorer(query);
        Highlighter highlighter = new Highlighter(formatter, scorer);
        matches = highlighter.getBestFragments(tokens, text, 1000, "...");
        IOException e;
        if (matches == null || matches.equals(""))
            return text;
        else
            return matches;
    }
//
//    public static final ReconstructedDocument reconstructDocument(int docId, IndexReader reader) throws IOException {
//        ReconstructedDocument proxy = new ReconstructedDocument();
//        Document doc = reader.document(docId);
//        addStoredFields(doc, proxy);
//        TermEnum termEnum = reader.terms();
//        TermPositions termPos = reader.termPositions();
//        do {
//            if (!termEnum.next())
//                break;
//            String fieldName = termEnum.term().field();
//            if (fieldName != null && !proxy.isStored(fieldName)) {
//                termPos.seek(termEnum.term());
//                if (termPos.skipTo(docId) && termPos.doc() == docId) {
//                    String termText = termEnum.term().text();
//                    int index = 0;
//                    while (index < termPos.freq()) {
//                        int pos = termPos.nextPosition();
//                        proxy.add(fieldName, termText, pos, false);
//                        index++;
//                    }
//                }
//            }
//        }
//        while (true);
//        return proxy;
//    }
//
//    private static final void addStoredFields(Document doc, ReconstructedDocument proxy) {
//        Enumeration fields = doc.fields();
//        do {
//            if (!fields.hasMoreElements())
//                break;
//            Field field = (Field) fields.nextElement();
//            if (field.isStored())
//                proxy.add(field.name(), field.stringValue());
//        }
//        while (true);
//    }

    public static final String getIndexSize(String indexDirPath) {
        long size = 0L;
        File indexDir = new File(indexDirPath);
        if (!indexDir.exists() || !indexDir.isDirectory())
            throw new IllegalArgumentException("'" + indexDirPath + "' is not a " + "Lucene index directory.");
        File files[] = indexDir.listFiles();
        for (int index = 0; index < files.length; index++)
            size += files[index].length();

        return bytesToHumanReadable(size);
    }

    public static final List getIndexFiles(String indexDir) {
        return getIndexFiles(new File(indexDir));
    }

    public static final List getIndexFiles(File indexDir) {
        if (!indexDir.isDirectory())
            throw new IllegalArgumentException("'" + indexDir + "' " + "is not a directory.");
        File indexFiles[] = indexDir.listFiles();
        List indexFileList = new ArrayList();
        for (int index = 0; index < indexFiles.length; index++) {
            IndexFile indexFile = new IndexFile(indexFiles[index]);
            indexFileList.add(indexFile);
        }

        return indexFileList;
    }

    public static final String bytesToHumanReadable(long bytes) {
        if (bytes < 1024L)
            return bytes + " bytes";
        long kilobytes = bytes / 1024L;
        if (kilobytes < 1024L)
            return kilobytes + " K";
        long megabytes = kilobytes / 1024L;
        if (megabytes < 1024L) {
            return megabytes + " M";
        } else {
            long gigabytes = megabytes / 1024L;
            return gigabytes + " G";
        }
    }
}
