package com.bleujin.lore.core.search.searcher;

import org.apache.commons.lang.StringUtils;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Query;

import com.bleujin.lore.core.common.CaseInsensitiveHashMap;
import com.bleujin.lore.core.search.ISearcher;
import com.bleujin.lore.core.search.NewField;

public class ALQueryParser extends QueryParser implements ISearcher
{

    private static CaseInsensitiveHashMap typeMap = new CaseInsensitiveHashMap();

    private static Integer INT_TYPE = new Integer(1);
    private static Integer DATE_TYPE = new Integer(2);
    private static Integer ETC_TYPE = new Integer(3);

    static {
//        typeMap.put(ArticleIndexer.DOC_ID, ETC_TYPE);
//        typeMap.put(DArticle.PArtId, INT_TYPE);
//
//        typeMap.put(DArticle.OperDay, DATE_TYPE);
//        typeMap.put(DArticle.ExpireDay, DATE_TYPE);
//        typeMap.put(DArticle.CreDate, DATE_TYPE);
//        typeMap.put(DArticle.ModDate, DATE_TYPE);
//        typeMap.put(DArticle.Status, ETC_TYPE);
//
//        typeMap.put(DArticle.ArtId, INT_TYPE);
//        typeMap.put(DArticle.ModSerNo, INT_TYPE);
//        typeMap.put(DArticle.RegUserId, ETC_TYPE);
//        typeMap.put(DArticle.RegDay, DATE_TYPE);
//        typeMap.put(DArticle.ArtSubject, ETC_TYPE);
    }

    public ALQueryParser(Analyzer analyzer) {
        super(ALL_FIELD, analyzer);
        this.setDefaultOperator(QueryParser.AND_OPERATOR);
    }

    public Query parse(String query) throws ParseException{
    	return super.parse(adjustQuery(query)) ;
    }
    
	private String adjustQuery(String squery) {
        String tmpSQL = ' ' + squery.toLowerCase().replace('\n', ' ').replace('\t', ' ');

        // boolean operator
        tmpSQL = StringUtils.replace(tmpSQL, "=", ":") ;
        tmpSQL = StringUtils.replace(tmpSQL, "'", "") ;

        // upper case
        tmpSQL = StringUtils.replace(tmpSQL, " and ", " AND ");
        tmpSQL = StringUtils.replace(tmpSQL, " or ", " OR ");
        tmpSQL = StringUtils.replace(tmpSQL, " to ", " TO ");
        tmpSQL = StringUtils.replace(tmpSQL, " not ", " - ");
        return tmpSQL;
    }
    
    protected Query getFieldQuery(String field, String queryText) throws ParseException {
        String _field = field.toLowerCase() ;
        if(isNumberType(field)) {
            return super.getFieldQuery(_field, getPadValue(queryText));
        }
        return super.getFieldQuery(_field, queryText);
    }

    protected Query getRangeQuery(String field, String start, String end, boolean inclusive) throws ParseException {
        String _field = field.toLowerCase();
        if(isNumberType(field)) {
            return super.getRangeQuery(_field, getPadValue(start), getPadValue(end), inclusive);
        } else if(isDateType(field)) {
            return super.getRangeQuery(_field, start, end, inclusive);
        } else {
            return super.getRangeQuery(_field, start, end, inclusive);
        }
    }

    private boolean isNumberType(String field) throws ParseException {
        return isContainsInMap(field) && INT_TYPE.equals(typeMap.get(field));
    }

    private boolean isDateType(String field) throws ParseException {
        return isContainsInMap(field) && DATE_TYPE.equals(typeMap.get(field));
    }

    private boolean isContainsInMap(String field) throws ParseException {
        boolean result = typeMap.containsKey(field);

        if(result) {
            return true ;
        } else { // confirm afield..
            fillTypeMap();
            return typeMap.containsKey(field) ;
        }
    }

    private void fillTypeMap() throws ParseException {
//        try {
//            UserProcedure upt = dbm.getRepositoryService().createUserProcedure("article@afieldTypeBy()") ;
//            Rows rows = upt.execQuery(dbm) ;
//            while(rows.next()){
//                typeMap.put(rows.getString(DAfield.AfieldId), new Integer(rows.getInt("type"))) ;
//            }
//        } catch(SQLException ex) {
//            throw new ParseException(ex.getMessage());
//        }
    }

    private String getPadValue(String value) {
        return NewField.getValue(Integer.parseInt(value));
    }


}
