package com.bleujin.lore.core.search.searcher;

import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.document.Document;

import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.dimpl.AbstractNode;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.SearchException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.search.ISearcher;

public class SearchResult implements ISearcher {
	
	private UserSession session ;
	private List<Document> finds ;
	SearchResult(UserSession session, Node startNode, List<Document> finds){
		this.session = session ;
		this.finds = finds ;
	}
	
	List<Document> toDocument(Page page) throws SearchException {
        int limit = Math.min(finds.size(), page.getEndLoc()) ;
        
        if (finds.size() <= page.getStartLoc()) return new ArrayList<Document>() ;
        
        List<Document> result = new ArrayList<Document>() ;
        for(int i = page.getStartLoc(), last = limit ; i < last ; i++){
        	result.add(finds.get(i)) ;
        }
        
        return result ;
	}

	public int totalLength() {
	    return finds.size();
    }

	public List<Node> toNode(Page page) throws ALRepositoryException {
		List<Document> docs = toDocument(page) ;
		List<Node> result = new ArrayList<Node>() ;
		for(Document doc : docs) {
			// TODO : after speed up.. 
			
			Node node = session.getNode(AbstractNode.ID_BEGIN_MARK + doc.get(DOC_ID) + AbstractNode.ID_END_MARK) ;
			result.add(node) ;
		}

	    return result;
    }
	
	public GenericIterator<Node> toNodeIterator(Page page) throws ALRepositoryException {
		return new GenericIteratorHelper<Node>(toNode(page)) ;
	}

}
