package com.bleujin.lore.core.search.searcher;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.search.Filter;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.Sort;
import org.apache.lucene.search.TopDocCollector;
import org.apache.lucene.search.TopFieldDocs;
import org.apache.lucene.store.Directory;

import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.SearchException;

public class SearcherHelper {

	private Directory dir;

	SearcherHelper(Directory dir) {
		this.dir = dir ;
	}
	
	List<Document> search(Query query, Page page) throws SearchException {
	    try {
	        IndexSearcher searcher = new IndexSearcher(dir);
	        TopDocCollector collector = new TopDocCollector(page.getMaxCount());
	        searcher.search(query, collector);
	        
	        List<Document> result = toList(searcher, collector);

	        searcher.close() ;
	        return result ;
        } catch (IOException e) {
	        throw new SearchException(e) ;
        }
    }

	List<Document> search(Query query, Filter filter, Sort sort, int maxCount) throws SearchException {
		try {
	        IndexSearcher searcher = new IndexSearcher(dir);
	        Debug.happy(getClass(), "Search query[" + query + "], filter[" + filter + "] sort[" + sort + "] maxCount[" + maxCount+ "]") ;
	        TopFieldDocs collector = searcher.search(query, filter, maxCount, sort);

	        Debug.happy(this.getClass(), query + " " + filter + " " + maxCount + " " + sort);
	        
	        List<Document> result = toList(searcher, collector);
	        searcher.close() ;
	        return result ;
        } catch (IOException e) {
	        throw new SearchException(e) ;
        }
	}

	private List<Document> toList(IndexSearcher searcher, TopDocCollector collector) throws CorruptIndexException, IOException {
		ScoreDoc[] hits = collector.topDocs().scoreDocs;
		List<Document> result = new ArrayList<Document>() ;
		for (int i = 0; i < hits.length; i++) {
			int docId = hits[i].doc;
		    Document d = searcher.doc(docId);
		    result.add(d) ;
		}
		return result;
	}

	private List<Document> toList(IndexSearcher searcher, TopFieldDocs topFieldDoc) throws CorruptIndexException, IOException {
		ScoreDoc[] hits = topFieldDoc.scoreDocs;
		List<Document> result = new ArrayList<Document>() ;
		for (int i = 0; i < hits.length; i++) {
			int docId = hits[i].doc;
		    Document d = searcher.doc(docId);
		    result.add(d) ;
		}
		return result;
	}

}
