package com.bleujin.lore.core.search.searcher;

import org.apache.lucene.analysis.WhitespaceAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.QueryFilter;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.store.Directory;

import com.bleujin.lore.core.search.indexer.IndexDirectory;
import com.bleujin.lore.core.search.indexer.TestCaseSearchParent;

public class TestFilter extends TestCaseSearchParent{

	private Directory directory ;
	public void setUp() throws Exception {
		super.setUp() ;
		directory = IndexDirectory.createIndexDirectory(TEST_INDEX_DIR) ; 
			// new RAMDirectory() ;
		IndexWriter writer = new IndexWriter(directory, new WhitespaceAnalyzer(), true) ;
		
		Document document = new Document() ;
		document.add(new Field("owner", "elwood", Field.Store.YES, Field.Index.NOT_ANALYZED)) ;
		document.add(new Field("keywords", "elwoods sensitive info", Field.Store.NO, Field.Index.ANALYZED)) ;
		writer.addDocument(document) ;
		
		document = new Document() ;
		document.add(new Field("owner", "jake", Field.Store.YES, Field.Index.NOT_ANALYZED)) ;
		document.add(new Field("keywords", "jakes sensitive info", Field.Store.NO, Field.Index.ANALYZED)) ;
		writer.addDocument(document) ;

		document = new Document() ;
		document.add(new Field("owner", "bleujin", Field.Store.YES, Field.Index.NOT_ANALYZED)) ;
		document.add(new Field("keywords", "bleujin sensitive info", Field.Store.NO, Field.Index.ANALYZED)) ;
		writer.addDocument(document) ;

		
		writer.close() ;
	}
	
	public void testFilter() throws Exception {
	    TermQuery query = new TermQuery(new Term("keywords", "info")) ;
	    IndexSearcher searcher = new IndexSearcher(directory) ;
	    Hits hits = searcher.search(query) ;
	    assertEquals(3, hits.length()) ;
	    
	    QueryFilter filter = new QueryFilter(new TermQuery(new Term("owner", "jake"))) ;
	    hits = searcher.search(query, filter) ;
	    assertEquals(1, hits.length()) ;
	    assertEquals("jake", hits.doc(0).get("owner")) ;
	    
	    
//	    TopDocs docs = searcher.search(query, new QueryFilter(new TermQuery(new Term("keywords", "info"))), 2) ;
//	    Debug.debug(this, " : " + docs.scoreDocs[0].) ;

    }
}
