package com.bleujin.lore.core.search.searcher;

import java.io.IOException;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;

public class TestSearcher extends TestCaseParent {

	public void setUp() throws Exception {
		super.setUp();
		indexingNode();
	}

	private String explain = "explain";
	private void indexingNode() throws ALRepositoryException, IOException, InterruptedException {

		Node robot = saveNode(getRobotType(), "r3");
		robot.setProperty(explain, "before");

		Node asimo = saveNode(getRobotType(), "asimo");
		asimo.setProperty(explain, "asimo Type");

		session.save() ;
	}

	public void testSearch() throws Exception {
		assertEquals(2, session.getLocalStore().getIndexer().getIndexInfo().get("numdoc"));

		// NodeSearcher searcher = new NodeSearcher(session.getLocalPersistence(), workspace.getIndexDirectory()) ;
		NodeSearcher searcher = session.getSearcher() ;
		// search property
		
		SearchResult sr = searcher.search("explain:before");
		assertEquals(1, sr.totalLength());
		String uuid = sr.toNode(Page.TEN).get(0).getIdentifier();

		// search name
		sr = searcher.search("name:r3");
		assertEquals(1, sr.totalLength());

		sr = searcher.search("robot");
		assertEquals(2, sr.totalLength());

		// search field : uuid
		sr = searcher.searchField(Node.UUID, uuid, Page.ALL);
		assertEquals(1, sr.totalLength());

		// search field : path
		sr = searcher.searchField(Node.PATH, "/r3", Page.ALL);
		assertEquals(1, sr.totalLength());

		sr = searcher.searchField(new String[] { Node.PATH, Node.UUID }, new String[] { "/r3", uuid }, true, Page.ALL);
		assertEquals(1, sr.totalLength());
		assertEquals("before", sr.toNode(Page.TEN).get(0).getString(explain));
		
	}

	// delete
	public void testDelete() throws Exception {
		NodeSearcher searcher = session.getSearcher() ;
		Node r3 = session.getNode("/r3") ;
		r3.delete() ;
		session.save() ;
		
		SearchResult sr = searcher.search("explain:before");
		assertEquals(0, sr.totalLength());
	}

	// new version search... 
	public void testVersion() throws Exception{
		NodeSearcher searcher = session.getSearcher() ;
		
		Node r3 = session.getNode("/r3") ;
		NodeVersion r3_1 = r3.checkIn();
		r3_1.setProperty(explain, "after") ;
		r3_1.checkOut() ;
		
		session.save() ;
		
		SearchResult sr = searcher.searchField(new String[] { Node.PATH, Node.UUID }, new String[] { "/r3", r3.getIdentifier() }, true, Page.ALL);
		assertEquals("after", sr.toNode(Page.TEN).get(0).getString(explain));

		searcher = session.getSearcher() ;

		assertEquals(1, searcher.search("explain:after").totalLength());

		assertEquals(0, searcher.search("explain:before").totalLength());

		
		
	}

	
	public void testAboutChildOnly() throws Exception{
		Node asimo = session.getNode("/asimo");
		Node child = createNodeSave(asimo, getRobotType(), "asimo") ;
		child.setProperty(explain, "asimo child") ;
		session.save() ;
		
		Node root = session.getRootNode() ;
		GenericIterator<Node> result = root.searchNode("asimo") ;
		assertEquals(2, result.size()) ;
		
		Node r3 = session.getNode("/r3") ;
		GenericIterator<Node> r3_result = r3.searchNode("asimo") ;
		assertEquals(0, r3_result.size()) ;

	}
	
	
	public void testNode() throws Exception {
		Node testFirstNode = createTestNode() ;
		GenericIterator<Node> nodes = testFirstNode.searchNode("abc");
		assertEquals(0, nodes.size());

		nodes = session.getRootNode().searchNode("explain:before");
		assertEquals(1, nodes.size());
		
		nodes = testFirstNode.searchNode("explain:before");
		assertEquals(0, nodes.size());
		
	}
	
	public void testFilter() throws Exception {
		Node testFirstNode = createTestNode() ;
		
		GenericIterator<Node> nodes = session.getRootNode().searchNode("explain:before");
		assertEquals(1, nodes.size());
		
		NodeSearcher s  = session.getSearcher() ;
		s.addFilter(Node.NODETYPE_ID, ntManager.getNodeType("robot").getName()) ; // "default:robot"
		GenericIterator<Node> snodes = s.search("explain:before").toNodeIterator(Page.ALL) ;
		assertEquals(1, snodes.size()) ;
		
	}

}