package com.bleujin.lore.core.security;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.bleujin.lore.addon.security.Action;
import com.bleujin.lore.addon.security.Authority;
import com.bleujin.lore.addon.security.UserAuthority;

public class AuthoritySetting {
	private Map<String, Authority> mapping = new HashMap<String, Authority>();

	private static Authority managerAuthority = new Authority(null, "node@manager");
	private static Authority readAuthority = new Authority(managerAuthority, "node@read");
	private static Authority writeAuthority = new Authority(managerAuthority, "node@write");
	private static Authority updateAuthority = new Authority(managerAuthority, "node@update");
	private static Authority deleteAuthority = new Authority(managerAuthority, "node@delete");

	private List<UserAuthority> store = new ArrayList<UserAuthority>() ;

	public AuthoritySetting() {
	}

	public Authority requireAuthority(Action action){
		return mapping.get(action.getKey());
	}

	public void addMapping(Action action, Authority readAuthority) {
	    mapping.put(action.getKey(), readAuthority) ;
    }
	
	public Authority readAuthority(){
		return readAuthority ;
	}
	public Authority writeAuthority(){
		return writeAuthority ;
	}
	public Authority updateAuthority(){
		return updateAuthority ;
	}
	public Authority deleteAuthority(){
		return deleteAuthority ;
	}
	public Authority managerAuthority() {
		return managerAuthority;
	}
	
	
	
	public void add(UserAuthority userAuthority){
		store.add(userAuthority) ;
	}

	public void remove(UserAuthority authority) {
		store.remove(authority) ;
    }	
	
	public List<UserAuthority> getStore(){
		return Collections.unmodifiableList(store) ;
	}
	

	public static AuthoritySetting createDefault(){
		AuthoritySetting as = new AuthoritySetting() ;
		
		as.addMapping(NodeAction.create("read"), readAuthority);
		as.addMapping(NodeAction.create("write"), writeAuthority);
		as.addMapping(NodeAction.create("update"), updateAuthority);
		as.addMapping(NodeAction.create("delete"), deleteAuthority);

		return as ;
	}


}
