package com.bleujin.lore.core.security;

import java.util.ArrayList;
import java.util.List;

import com.bleujin.lore.addon.security.Authority;
import com.bleujin.lore.addon.security.IResource;
import com.bleujin.lore.addon.security.IUser;
import com.bleujin.lore.addon.security.UserAuthority;
import com.bleujin.lore.addon.security.UserAuthority.Range;
import com.bleujin.lore.addon.security.UserAuthority.Type;
import com.bleujin.lore.core.exception.ALRepositoryException;

public class TransientSecurityFilter extends WorkspaceSecurityFilter {

	private final static boolean DEFAULT_ALLOW = false ;
	public TransientSecurityFilter(AuthoritySetting setting) {
		super(setting) ;
    }

	
	public boolean isAllowed(IResource targetResource, IUser loginUser, Authority requireAuth) throws ALRepositoryException{

		if (requireAuth == null) return DEFAULT_ALLOW ;

		boolean result = DEFAULT_ALLOW ;
		for(IUser user : loginUser.getUpperTree()){
			Authority auth = requireAuth ;
			while(auth != null){
				IResource resource = targetResource ;
				while (! resource.isRoot()){
					List<UserAuthority> findUserAuthority = findUserAuthority(resource, user, auth) ;
					
					for (UserAuthority userAuthority : findUserAuthority) {
						if (userAuthority.getRange() == Range.ONLY_THIS_RESOURCE && ! resource.equals(targetResource)){
							continue ;
						}

						if (userAuthority.getType() == Type.REVOKE) return false ;
						result = true ;
					}
					resource = resource.getParent() ;
				}
				auth = auth.getParent() ;
			}
		}
		
		return result ;
	}

	
	private List<UserAuthority> findUserAuthority(IResource resource, IUser user, Authority authority) throws ALRepositoryException{
		String searchKey = resource.getKey() + ":" + user.getUserId() + ":" + authority.getKey() ;
		List<UserAuthority> result = new ArrayList<UserAuthority>() ;
		for(UserAuthority ua : setting.getStore()){
			String uaKey = ua.getResource().getKey() + ":" + ua.getUser().getUserId() + ":" + ua.getAuthority().getKey();
			if (uaKey.equals(searchKey)) {
				result.add(ua) ;
			}
		}
		
		return result ;
	}

	
}
