package com.bleujin.lore.core.util;

import net.sf.json.JSONObject;

import org.apache.ecs.xml.XML;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.JSONExeption;
import com.bleujin.lore.core.mapping.RequireConstraint;
import com.bleujin.lore.core.mapping.StringDefaultValue;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.NewNodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;
import com.bleujin.lore.core.persitence.NodeBean;

public class TestJsonNode extends TestCaseParent{
	
	public void testNode() throws Exception {
		NodeType robot = getRobotType();
		Node node = saveNode(robot, "p2_robot");
		node.setProperty("explain", "first robot");

		JSONObject json = toJSON(node);

		assertEquals("explain", json.getJSONObject("explain").get("id")) ;
		assertEquals("first robot", json.getJSONObject("explain").get("value")) ;
		assertEquals("p2_robot", json.getJSONObject(NodeBean.XML_TAG_ROOT).get(NodeBean.XML_TAG_NAME)) ;
	}

	private JSONObject toJSON(Node node) throws ALRepositoryException, JSONExeption {
		XML xml = node.toXML();
		JSONObject json = JSONUtil.toJSON(xml);
		return json;
	}

	public void testNode2() throws Exception {
		String name = "name";
		String explain = "explain";
		NewNodeType newType = createNewNodeType(objectType, "person", new PropertyDefinition[]{createPropertyDefinition(name, Property.Type.STRING), createPropertyDefinition(explain, Property.Type.STRING)}) ;
		newType.setConstraint(name, new RequireConstraint()) ;
		newType.setConstraint(explain, new RequireConstraint()) ;
		newType.setDefaultValue(name, new StringDefaultValue("noname"));
		newType.setDefaultValue(explain, new StringDefaultValue("un'known\""));
		NodeType person = super.registerNodeType(newType) ;
		
		Node node = saveNode(person, "bleujin") ;
		
		JSONObject json = toJSON(node);
		
		assertEquals("noname", json.getJSONObject("name").get("value")) ;
		assertEquals("bleujin", json.getJSONObject(NodeBean.XML_TAG_ROOT).get(NodeBean.XML_TAG_NAME)) ;
		assertEquals("un'known\"", json.getJSONObject("explain").get("value")) ;
	}
	
	
	public void testNodeType() throws Exception {
		String name = "name";
		String explain = "explain";
		NewNodeType newType = createNewNodeType(objectType, "person", new PropertyDefinition[]{createPropertyDefinition(name, Property.Type.STRING), createPropertyDefinition(explain, Property.Type.STRING)}) ;
		newType.setConstraint(name, new RequireConstraint()) ;
		newType.setConstraint(explain, new RequireConstraint()) ;
		newType.setDefaultValue(name, new StringDefaultValue("noname"));
		newType.setDefaultValue(explain, new StringDefaultValue("unknown"));
		NodeType person = registerNodeType(newType) ;
		
		XML xml = person.toXML();

		JSONObject json = JSONUtil.toJSON(xml);
		assertEquals("default:_object", json.getJSONObject(NodeType.XML_TAG_TYPEINFO).get(NodeType.XML_TAG_SUPERTYPE)) ;

		assertEquals("name", json.getJSONObject("name").get("id")) ;
		assertEquals("STRING", json.getJSONObject("name").get("required")) ;
		assertEquals("explain", json.getJSONObject("explain").get("id")) ;
		assertEquals("STRING", json.getJSONObject("explain").get("required")) ;

	}
	
}
