package com.bleujin.lore.core.value;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.node.Property.Type;

public class ArrayValue extends Value{

	private static final long serialVersionUID = -5493646637657908695L;
	
	private final Property.Type propertyType ;
	private final Serializable[] values ;
	
	ArrayValue(Property.Type propertyType, Serializable... values) {
		this.propertyType = propertyType ;
		this.values = values ;
	}
	

	@Override
	public Value newClone()  {
		// TODO Auto-generated method stub
		return new ArrayValue(this.propertyType, this.values.clone());
	}

	@Override
	public boolean getBoolean() throws ValueFormatException {
		return firstValue().getBoolean();
	}

	@Override
	public Calendar getDate() throws ValueFormatException {
		return firstValue().getDate();
	}

	@Override
	public BigDecimal getDecimal() throws ValueFormatException {
		return firstValue().getDecimal();
	}

	@Override
	public double getDouble() throws ValueFormatException {
		return firstValue().getDouble() ;
	}

	@Override
	public long getLong() throws ValueFormatException {
		return firstValue().getLong() ;
	}

	@Override
	public Object getObject(Map param, Node node) throws ALRepositoryException {
		return firstValue().getObject(param, node);
	}

	@Override
	public Type getPropertyType() {
		return firstValue().getPropertyType();
	}

	@Override
	public InputStream getStream() throws IOException, ValueFormatException {
		return firstValue().getStream();
	}

	@Override
	public String getString() {
		return firstValue().getString();
	}

	@Override
	public String getValueString() {
		return firstValue().getValueString();
	}

	private Value firstValue(){
		return Value.newValue(propertyType, values.length >0 ? values[0] : null);
	}

	public long getLength() {
		long result = 0L ;
		for (Serializable value : values) {
			result += Value.newValue(propertyType, value).getLength() ;
		}
		return result ;
	}

	public List<Value> toList(){
		List<Value> result = new ArrayList<Value>();
		for (Serializable value : values) {
			result.add(Value.newValue(propertyType, value)) ;
		}
		return result ;
	}


}
