package com.bleujin.lore.core.value;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Map;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;

public class DateValue extends Value {
 
	private static final long serialVersionUID = -6041016945815015422L;
	private Calendar value ;
	public DateValue(Calendar value) {
		this.value = value ;
	}



	public String getString() {
		return getValueString(); 
	}
	
	public String getValueString() {
		if (value == null) return null ;
		return DateUtil.calendarToGMTString(value) ; 
	}

	public Object getObject(Map param, Node node) throws ALRepositoryException {
		return value ;
	}
	
	public InputStream getStream() throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.writeObject(getString().getBytes("UTF-8"));
		oos.flush();
		return new ByteArrayInputStream(baos.toByteArray());
	}

	public long getLong() {
		return value.getTimeInMillis();
	}

	public BigDecimal getDecimal() throws ValueFormatException {
		return new BigDecimal(DateUtil.calendarToLong(value)) ;
	}

	public double getDouble() throws ValueFormatException {
		if (value.getTimeInMillis() > Double.MAX_VALUE) throw new ValueFormatException() ; 
		else return value.getTimeInMillis();
	}

	public Calendar getDate() {
		return value ;
	}

	public boolean getBoolean() throws ValueFormatException {
		throw new ValueFormatException() ;
	}

	
	
	public Property.Type getPropertyType() {
		return Property.Type.DATE;
	}
	
	public Value newClone() {
		return value == null ? new DateValue(null) : new DateValue((Calendar)value.clone());
	}

}
