package com.bleujin.lore.core.value;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Map;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;

public class DecimalValue extends Value {
 
	private static final long serialVersionUID = 497703923518750305L;
	private BigDecimal value ;
	public DecimalValue(BigDecimal value) {
		this.value = value ;
	}
	


	public String getString() {
		return getValueString(); 
	}
	
	public String getValueString() {
		return value.toString() ; 
	}

	public Object getObject(Map param, Node node) throws ALRepositoryException {
		return value ;
	}
	
	public InputStream getStream() throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.writeObject(getString().getBytes("UTF-8"));
		oos.flush();
		return new ByteArrayInputStream(baos.toByteArray());
	}

	public long getLong() throws ValueFormatException {
		return value.longValue() ;
	}

	public BigDecimal getDecimal() throws ValueFormatException {
		return value ;
	}

	public double getDouble() throws ValueFormatException {
		return value.doubleValue() ;
	}

	public Calendar getDate()  {
		return DateUtil.longToCalendar(value.longValue())  ;
	}

	public boolean getBoolean() throws ValueFormatException {
		throw new ValueFormatException() ;
	}

	
	
	public Property.Type getPropertyType() {
		return Property.Type.DECIMAL;
	}
	
	public Value newClone() {
		return new DecimalValue(value);
	}
}
