package com.bleujin.lore.core.value;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Map;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;

public class LongValue extends Value {
 
	private static final long serialVersionUID = 3790741553920803272L;
	private long value ;
	public LongValue(long value) {
		this.value = value ;
	}


	public String getString() {
		return getValueString(); 
	}
	
	public String getValueString() {
		return new Long(value).toString() ; 
	}

	public Object getObject(Map param, Node node) throws ALRepositoryException {
		return value ;
	}
	
	public InputStream getStream() throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.writeObject(getString().getBytes("UTF-8"));
		oos.flush();
		return new ByteArrayInputStream(baos.toByteArray());
	}

	public long getLong() throws ValueFormatException {
		return value ;
	}

	public BigDecimal getDecimal() throws ValueFormatException {
		return new BigDecimal(value) ;
	}

	public double getDouble() throws ValueFormatException {
		return new Double(getString()).doubleValue() ;
	}

	public Calendar getDate() throws ValueFormatException {
		return DateUtil.longToCalendar(value) ;
	}

	public boolean getBoolean() throws ValueFormatException {
		throw new ValueFormatException() ;
	}

	

	public Property.Type getPropertyType() {
		return Property.Type.LONG;
	}
	
	public Value newClone(){
		return new LongValue(value);
	}
}
