package com.bleujin.lore.core.value;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Map;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;

public class StringValue extends Value {
 
	private static final long serialVersionUID = -2559301476167686815L;
	private String value;

	StringValue(String value) {
		this.value = value;
	}

	public String getString() {
		return getValueString();
	}

	public String getValueString() {
		return value;
	}

	public Object getObject(Map param, Node node) throws ALRepositoryException {
		return getString();
	}

	public InputStream getStream() throws IOException, ValueFormatException {
		if (getString() == null) throw new ValueFormatException() ;
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.write(getString().getBytes("UTF-8"));
		oos.flush();
		return new ByteArrayInputStream(baos.toByteArray());
	}

	public long getLong() throws ValueFormatException {
		try {
			return Long.valueOf(value);
		} catch (NumberFormatException e) {
			throw new ValueFormatException(e);
		}
	}

	public BigDecimal getDecimal() throws ValueFormatException {
		try {
			return new BigDecimal(value);
		} catch (NumberFormatException e) {
			throw new ValueFormatException(e);
		}

	}

	public double getDouble() throws ValueFormatException {
		try {
			return Double.valueOf(value);
		} catch (NullPointerException e) {
			throw new ValueFormatException(e);
		} catch (NumberFormatException e) {
			throw new ValueFormatException(e);
		}
	}

	public Calendar getDate() throws ValueFormatException {
		try {
			return DateUtil.stringToCalendar(getString()) ;
		} catch (IllegalArgumentException e) {
			throw new ValueFormatException(e);
		}
	}

	public boolean getBoolean() {
		return Boolean.valueOf(value);
	}

	public Property.Type getPropertyType() {
		return Property.Type.STRING;
	}

	public Value newClone()  {
		return new StringValue(value);
	}

}
