package com.bleujin.lore.core.value;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NodeType;

public class TestArrayValue extends TestCaseParent{

	public void testArray() throws Exception {
		
		Value array = Value.arrayValue(Property.Type.STRING, "abc1", "abc2", "abc3") ;
		assertEquals("abc1", array.getString());
		assertEquals(12, array.getLength());
	}

	public void testSet() throws Exception {
		NodeType robotType = getRobotType() ;
		Node robot = saveNode(robotType, "robot") ;
		
		robot.setProperty("explain", Value.arrayValue(Property.Type.STRING, "abc1", "abc2", "abc3")) ;
		
		Property explain = robot.getProperty("explain") ;
		
		assertEquals(true, explain.hasArrayValue()) ;
		
		assertEquals("abc1", explain.getString());
		assertEquals("abc1", explain.getValue().getValueString());
		
		
		robot.setProperty("explain", Value.arrayValue(Property.Type.STRING)) ;
		assertEquals(null, robot.getString("explain")) ;

		
	}
	
	public void testGet() throws Exception {
		NodeType robotType = getRobotType() ;
		Node robot = saveNode(robotType, "robot") ;
		
		robot.setProperty("explain", Value.arrayValue(Property.Type.STRING, "abc1", "abc2", "abc3")) ;
		session.save() ;
		
		Node load = session.getNode(robot.getIdentifier()) ;
		Property explain = load.getProperty("explain") ;

		assertEquals(true, explain.hasArrayValue()) ;
		
		Value[] values = explain.getValues();
		assertEquals("abc1", values[0].getString()) ;
		assertEquals("abc2", values[1].getString()) ;
		assertEquals("abc3", values[2].getString()) ;

	}
	
	public void testVersion() throws Exception {
		NodeType robotType = getRobotType() ;
		Node robot = saveNode(robotType, "robot") ;
		robot.setProperty("explain", Value.arrayValue(Property.Type.STRING, "abc1", "abc2", "abc3")) ;
		session.save() ;
		
		NodeVersion robotv1 = robot.checkIn() ;
		robotv1.setProperty("explain", Value.arrayValue(Property.Type.STRING, "cba1", "cba2")) ;
		robotv1.checkOut() ;
		
		Property explain = robotv1.getProperty("explain") ;
		assertEquals(true, explain.hasArrayValue()) ;
		
		session.save() ;

		Node load = session.getNode(robot.getIdentifier()) ;
		GenericIterator<NodeVersion> versions =  load.getVersionHistory().getAllVersions() ;

		NodeVersion v1 = versions.next() ;
		
		assertEquals("abc1", v1.getProperty("explain").getString()) ;
		Value[] values = v1.getProperty("explain").getValues() ;
		assertEquals(3, values.length) ;
		assertEquals("abc1", values[0].getString()) ;
		assertEquals("abc2", values[1].getString()) ;
		assertEquals("abc3", values[2].getString()) ;

	}
	
	

}
