package com.bleujin.lore.core.value;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.Calendar;

import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.BinaryPropertyValue;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.NodeVersion;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class TestValue extends TestCaseParent {

	public final static BinaryPropertyValue binaryValue = new BinaryPropertyValue() {

		public String getContentType() {
			return "jpg";
		}

		public InputStream getInputStream() throws FileNotFoundException, IOException {
			return null;
		}

		public long getSize() {
			return 0;
		}

		public String getName() {
			return "111.jpg";
		}

		public String getPath() {
			return "c:/111.jpg";
		}
	};

	public void testLong() throws Exception {
		String st = "000012";
		assertEquals(12L, Long.parseLong(st));

		assertEquals("00000000000000012.12", StringUtil.leftPad(String.valueOf(12.12D), 20, "0"));
	}

	public void testNull() throws Exception {
		Value v = Value.undefinedValue((String) null);
		assertEquals(Property.Type.UNDEFINED, v.getPropertyType());

		v = Value.stringValue((String) null);
		assertEquals(Property.Type.STRING, v.getPropertyType());

		assertNull(v.getString());
		assertFalse(v.getBoolean());

		try {
			assertEquals(0L, v.getLong());
			fail();
		} catch (ValueFormatException ignore) {
		}

		try {
			assertEquals(0.0, v.getDouble());
			fail();
		} catch (ValueFormatException ignore) {
		}

		try {
			assertNull(v.getDate());
			fail();
		} catch (ValueFormatException ignore) {
		}

		try {
			assertNull(v.getStream());
			fail();
		} catch (ValueFormatException ignore) {
		}
	}

	public void testEqual() throws Exception {

		Value v = Value.stringValue("123");
		Value v2 = Value.stringValue("123");

		assertEquals(v, v2);
		Value v3 = Value.decimalValue(new BigDecimal(3));
		assertEquals("3", v3.getString());
	}

	public void testBinary() throws Exception {

		Value v = Value.binaryValue(binaryValue);
		assertEquals("111.jpg", v.getString());

		String explain = "explain";
		String img = "img";
		NodeType imgNodeType = registerNodeType(objectType, "imgType", new PropertyDefinition[] { createPropertyDefinition(img, Property.Type.BINARY),
				createPropertyDefinition(explain, Property.Type.STRING) });
		INode imgNode = session.createNode(session.getRootNode(), imgNodeType, "img");
		imgNode.setProperty(explain, "1");
		imgNode.setProperty(img, binaryValue);
		imgNode.save() ;

		Node theNode = session.getNode("/img");
		assertEquals("111.jpg", theNode.getString(img));

		Node savedNode = imgNode.save();

		NodeVersion v1 = savedNode.checkIn();
		v1.checkOut();

		theNode = session.getNode("/img");
		assertEquals("111.jpg", theNode.getString(img));

		assertEquals("<propertyValue contentType='jpg' size='0' name='111.jpg' path='c:/111.jpg'></propertyValue>", binaryValue.getString());
	}

	public void testLength() throws Exception {
		Value v = Value.stringValue("가") ;
		assertEquals(3, v.getLength()) ;
		
		assertEquals(1, Value.longValue(3).getLength()) ;
		assertEquals(2, Value.longValue(12).getLength()) ;
		assertEquals(3, Value.longValue(123).getLength()) ;
		assertEquals(4, Value.longValue(1234).getLength()) ;

		assertEquals(4, Value.booleanValue(true).getLength()) ;
		assertEquals(10, Value.textValue("abcd가123").getLength()) ;

		assertEquals(3, Value.doubleValue(.3).getLength()) ;
		assertEquals(4, Value.doubleValue(0.35).getLength()) ;
		
		assertEquals(5, Value.virtualTextValue("#node").getLength()) ;

		assertEquals(15, Value.dateValue(Calendar.getInstance()).getLength()) ;
	}
	
}
