package com.bleujin.lore.core.value;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Map;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;

public class TextValue extends Value {
 
	private static final long serialVersionUID = 3971444882652997974L;
	private String value ;
	public TextValue(String value) {
		this.value = value ;
	}
	

	public String getString() {
		return getValueString();
	}
	
	public String getValueString() {
		return value ;
	}

	public Object getObject(Map param, Node node) throws ALRepositoryException {
		return new StringBuffer(value);
	}
	
	public InputStream getStream() throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.write(getString().getBytes("UTF-8"));
		oos.flush();
		return new ByteArrayInputStream(baos.toByteArray());
	}

	public long getLong() throws ValueFormatException {
		throw new ValueFormatException();
	}

	public BigDecimal getDecimal() throws ValueFormatException {
		throw new ValueFormatException();
	}

	public double getDouble() throws ValueFormatException {
		throw new ValueFormatException();
	}

	public Calendar getDate() throws ValueFormatException {
		Calendar cal;
		try {
			cal = Calendar.getInstance();
			cal.setTime(DateUtil.stringToDate(value));
		} catch (IllegalArgumentException e) {
			throw new ValueFormatException(e) ;
		}
		return cal;
	}

	public boolean getBoolean() throws ValueFormatException {
		throw new ValueFormatException();
	}
	
	public Property.Type getPropertyType() {
		return Property.Type.TEXT;
	}
	
	public Value newClone() {
		return new TextValue(value);
	}
}
