package com.bleujin.lore.core.value;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Map;

import com.bleujin.framework.util.DateUtil;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.ValueFormatException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;

public class UndefinedValue extends Value{

 
	private static final long serialVersionUID = 7485492426106941081L;
	private String value ;
	UndefinedValue(String value) {
		this.value = value ;
	}

	

	public String getString() {
		return getValueString(); 
	}
	
	public String getValueString() {
		return value ; 
	}

	public Object getObject(Map param, Node node) throws ALRepositoryException {
		return value ;
	}
	
	public InputStream getStream() throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.writeObject(getString().getBytes("UTF-8"));
		oos.flush();
		return new ByteArrayInputStream(baos.toByteArray());
	}

	public long getLong() throws ValueFormatException {
		return new Long(getString()).longValue();
	}

	public BigDecimal getDecimal() throws ValueFormatException {
		return new BigDecimal(getString()) ;
	}

	public double getDouble() throws ValueFormatException {
		return new Double(getString()).doubleValue() ;
	}

	public Calendar getDate() throws ValueFormatException {
		return DateUtil.stringToCalendar(getString()) ;
	}

	public boolean getBoolean() throws ValueFormatException {
		return new Boolean(getString()) ;
	}

	
	
	public Property.Type getPropertyType() {
		return Property.Type.UNDEFINED;
	}
	
	public Value newClone(){
		return new UndefinedValue(value);
	}
}
