package com.bleujin.lore.core.workspace;

import java.io.File;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Logger;

import com.bleujin.framework.configuration.Configuration;
import com.bleujin.framework.configuration.ConfigurationException;
import com.bleujin.framework.logging.LogBroker;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.dimpl.ShareInfo;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.AccessDeniedException;
import com.bleujin.lore.core.node.ShareNode;
import com.bleujin.lore.core.observation.FileListener;
import com.bleujin.lore.core.observation.UserBean;
import com.bleujin.lore.core.persitence.GlobalStore;
import com.bleujin.lore.core.session.SessionContext;
import com.bleujin.lore.core.session.SessionManager;
import com.bleujin.lore.core.session.SimpleSessionManagerFactory;
import com.bleujin.lore.core.valid.ValidChain;
import com.bleujin.lore.core.workspace.config.FileRepositoryInfo;
import com.bleujin.lore.core.workspace.config.GlobalStoreInfo;
import com.bleujin.lore.core.workspace.config.LicenseInfo;
import com.bleujin.lore.core.workspace.config.StoragesInfo;
import com.bleujin.lore.core.workspace.config.WorkspacesInfo;

public class AnotherLore {

	private SessionManager sessionManager;

	public final static String DEFAULT_WORKSPACE = "_default_workspace" ;

	private LicenseInfo license;
	private FileRepositoryInfo fileRepository ; 
	
	private ValidChain vchain;
	private ObservationManager observationManager;
	private GlobalStore globalStore;
	private String indexParentDir;

	public AnotherLore() {

	}

	public GlobalStore getGlobalStore() {
		return globalStore;
	}

	public ObservationManager getObservationManager() {
		return observationManager;
	}

	public ValidChain getValidChain() {
		return vchain;
	}

	public FileRepositoryInfo getFileRepository() {
		return this.fileRepository;
	}
	
	
	public UserSession login(Credential credential) throws AccessDeniedException {
		return login(credential, DEFAULT_WORKSPACE);
	}

	public UserSession login(Credential credential, String workspaceName) throws AccessDeniedException {
		if (credential.isAuthorizedUser()) {
			
			UserBean user = new UserBean(credential.getUserId(), credential.getUserIp(), credential.getLangCd());
			SessionContext context = sessionManager.getSessionContext();
			UserSession session = UserSession.create(getWorkspace(workspaceName), context, sessionManager.generateSessionId(), user);
			context.add(session);
			
			Debug.happy(credential.getUserId() + " login at " + workspaceName) ;
			return session;
		} else {
			throw new IllegalArgumentException("not permitted user : " + credential);
		}
	}
	
	
	public void initTest(LicenseInfo license, FileRepositoryInfo fileRepository, GlobalStore globalStore, String indexParentDir, Map<String, Workspace> created){
		this.license = license;
		this.fileRepository = fileRepository;

		this.observationManager = new ObservationManager();
		observationManager.addEventListener(new FileListener(new File("d:/anotherlore/event/trace.txt")));
		this.vchain = ValidChain.EMPTY;
		this.sessionManager = SimpleSessionManagerFactory.createFactory().createManager("anotherLore-session-manager");
		sessionManager.begin() ;

		this.indexParentDir = indexParentDir;
		this.globalStore = globalStore ;
		this.workspaces.putAll(created);
	}
	

	public void init(Configuration config) throws ConfigurationException {

		// logging
		// String logsConfigFileName = icsConfigBase + config.getChild("logs-config-file").getValue();
		// configFactory = ConfigurationFactory.getInstance("logs-configuration");
		// configFactory.build(logsConfigFileName);
		// LogBroker.initialize(configFactory.getConfiguration("logs"));

		
		Logger log = LogBroker.getLogger(this) ;
		
		if (!config.getTagName().equals("root")) {
			throw new IllegalArgumentException("incorrect 'root' configuration.");
		}

		this.license = new LicenseInfo(config.getChild("license"));
		log.info(license.toString()) ;
		this.fileRepository = new FileRepositoryInfo(config.getChild("file-repository-path"));
		log.info(fileRepository.toString()) ;

		Locale locale = new Locale(config.getChild("default-locale").getValue());
		this.observationManager = new ObservationManager();
//		observationManager.addEventListener(new FileListener(new File("d:/anotherlore/event/trace.txt")));
		this.vchain = ValidChain.EMPTY;
		this.sessionManager = SimpleSessionManagerFactory.createFactory().createManager("anotherLore-session-manager");
		sessionManager.begin() ;
		sessionManager.setCheckInterval(config.getChild("session-manager.check-interval-sec").getValueAsInt());
		sessionManager.setMaxInactiveInterval(config.getChild("session-manager.max-inactive-interval-sec").getValueAsInt()) ;

		GlobalStoreInfo globalInfo = GlobalStoreInfo.create(config.getChild("global-store"));
		WorkspacesInfo workspacesInfo = WorkspacesInfo.create(config.getChild("workspaces"));
		StoragesInfo storagesInfo = StoragesInfo.create(config.getChildren("storage"));

		this.indexParentDir = fileRepository.getIndexRoot() ;

		this.globalStore = storagesInfo.getGlobalStore(globalInfo) ;
		workspaces.putAll(storagesInfo.getWorkspaces(this, workspacesInfo, this.indexParentDir));
	}

	// must name mapping singleton
	private Map<String, Workspace> workspaces = new HashMap<String, Workspace>();
	private Workspace getWorkspace(String workspaceName) {
		if (!workspaces.containsKey(workspaceName)) {
			throw new IllegalArgumentException("not found workspace : " + workspaceName);
		}
		return workspaces.get(workspaceName);
	}

	
	public GenericIterator<String> getWorkspaces(){
		return new GenericIteratorHelper<String>(workspaces.keySet()) ; 
	}
	
	// for test
	public void addNewWorkspace(Workspace workspace){
		workspaces.put(workspace.getName(), workspace) ;
	}
	
	public void destroyWorld() {
		for (Workspace workspace : workspaces.values()) {
			workspace.close() ;
		}
		getGlobalStore().close() ;
		workspaces.clear();
		sessionManager.end();
	}

	public ShareNode getNode(UserSession session, ShareInfo shareInfo) throws ALRepositoryException {
		String workspaceName = shareInfo.getWorkspaceName();
		Workspace workspace = getWorkspace(workspaceName);
		return workspace.getShareNode(session, shareInfo) ;
	}
	
	public LicenseInfo getLicenseInfo(){
		return this.license ;
	}
}
