package com.bleujin.lore.core.workspace;

import java.util.ArrayList;
import java.util.List;

import com.bleujin.lore.core.common.lock.DefaultReadsWriteLock;
import com.bleujin.lore.core.common.lock.ReadsWriteLock;
import com.bleujin.lore.core.dimpl.LockImpl;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.LockException;
import com.bleujin.lore.core.node.Lock;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.observation.UserBean;

public class LockManager {
	private static List<Lock> locks =  new ArrayList<Lock>();
	private ReadsWriteLock rwLock = new DefaultReadsWriteLock() ;
	
	public LockManager() {
	}
	
	private void removeChildLock(Node node) throws ALRepositoryException{
		try {
			for (int i = locks.size() - 1; i >= 0; i--) {
				Lock lock = locks.get(i);
				if (lock.getNode().getPath().startsWith(node.getPath() + "/")) {
					locks.remove(lock);
				}
			}
		} catch(ALRepositoryException e){
			throw e ;
		}
	}
	
	public Lock addLock(Node target, boolean isDeep) throws ALRepositoryException, InterruptedException {
		Lock lock = getLock(target);
		rwLock.writeLock() ;
		try {
			if (lock != null && !lock.isOwner(getUserBean(target)))
				throw new LockException("already locked by " + lock.getNode().getSession().getUserId() + " - " + getUserBean(target).getUserId());
			
			// lock escalation
			if (isDeep) {
				removeChildLock(target);
			}
			
			if (null == lock) {
				lock = LockImpl.create(this, target, isDeep);
				locks.add(lock);
			} else if (isDeep && (lock.getNode() != target || !lock.isDeep())) {
				throw new LockException("already locked");
			}
			
			return lock;
		} catch(ALRepositoryException e){
			throw e ;
		} finally{
			rwLock.writeUnLock() ;
		}
	}

	public boolean isLocked(Node node) throws ALRepositoryException, InterruptedException {
		return getLock(node) != null;
	}

	public Lock getLock(final Node node) throws ALRepositoryException, InterruptedException {
		rwLock.readLock() ;
		try {
			for (final Lock lock : locks) {
				if ((!lock.isDeep() && lock.getNode().getPath().equals(node.getPath())) || (lock.isDeep() && node.getPath().startsWith(lock.getNode().getPath()))) {
					if (lock.isOwner(getUserBean(node))) {
						return lock;
					} else {
						return new Lock(){
							public Node getNode() throws ALRepositoryException {
								return node ;
								// return lock.getNode();
							}
							public boolean isDeep() {
								return lock.isDeep();
							}
							public boolean isOwner(UserBean user) {
								return false;
							}
							public void unlock() throws ALRepositoryException {
								throw new LockException("other owner") ;
							}
						} ;
					}
				}
			}
			return null;
		} catch(ALRepositoryException e){
			throw e ;
		} finally{
			rwLock.readUnLock() ;
		}		
	}

	public void release(Lock lock) throws InterruptedException {
		rwLock.writeLock() ;
		try {
			locks.remove(lock);
		} finally{
			rwLock.writeUnLock() ;
		}		
	}

	public void releaseOwnerLock(UserBean userbean) throws InterruptedException {
		rwLock.writeLock() ;
		try {
			for (int i = locks.size() - 1; i >= 0; i--) {
				Lock lock = locks.get(i);
				if (lock.isOwner(userbean)) {
					locks.remove(lock);
				}
			}
		} finally{
			rwLock.writeUnLock() ;
		}		
	}
	
	private UserBean getUserBean(Node node) {
		return node.getSession().getUserBean();
	}


}
