package com.bleujin.lore.core.workspace;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.Page;
import com.bleujin.lore.core.dimpl.LinkTypeImpl;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.node.Node;

public class TestUserSession extends TestCaseParent{
	
	public void xtestSession() throws Exception {
		Node testFirstNode = createTestNode() ;

		Node rootNode = session.getRootNode() ;
		assertEquals(rootNode, testFirstNode) ;
		
		Node childNode = session.createNode(testFirstNode, objectType, "child").save() ;
		Node retrievedChildNode = session.getNode("/child") ;
		
		assertEquals(childNode, retrievedChildNode) ;
		assertEquals(true, session.existByPath("/child"))  ;
		
		Node grandChild = session.createNode(childNode, objectType, "grandchild").save() ;
		Node retrievedGrandChildNode = session.getNode("/child/grandchild") ;
		
		
		assertEquals(grandChild, retrievedGrandChildNode) ;
		
		childNode.save();
		assertEquals(1, childNode.getLinks(Page.ALL).size()) ;
		
		session.save() ;
		GenericIterator<Node> iter = testFirstNode.getChildNode() ;

		assertEquals(1, iter.size()) ;
		
		Node grandChild2 = session.createNode(childNode, objectType, "grandchild2").save() ;
		session.save() ;
		assertEquals(2, iter.next().getChildNode().size()) ;
		
		assertEquals(grandChild, session.getNode(grandChild.getIdentifier())) ;
    }

	public void testMove() throws Exception {
		Node parent = createTestNode() ;

		Node child1 = createNodeSave(parent, objectType, "n1") ;
		Node child2 = createNodeSave(parent, objectType, "n2") ;
		
		assertEquals(0, child1.getChildNode().size()) ;
		child2.move(child1) ;

		session.save() ;
		
		assertEquals(1, child1.getChildNode().size()) ;
		assertEquals(1, getTestFirstNode().getChildNode().size()) ;
		
		Node n1Child = createNodeSave(child1, objectType, "n1-child") ;
		try {
			child1.move(n1Child) ;
			fail() ;
		} catch(ALRepositoryException ignore){
			
		}
    }
	
	public void testMoveToRoot() throws Exception {
		Node parent = createTestNode() ;

		Node child1 = createNodeSave(parent, objectType, "n1").save() ;
		Node child2 = createNodeSave(parent, objectType, "n2").save() ;
		
		assertEquals(0, child1.getChildNode().size()) ;
		child2.move(session.getRootNode()) ;  // TO Root

		session.save() ;
		
		assertEquals(1, parent.getChildNode().size()) ;
		assertEquals(0, child1.getChildNode().size()) ;
		
		assertEquals(2, session.getRootNode().getChildNode().size()) ;
	}
	
	public void xtestCopy() throws Exception {
		Node testFirstNode = createTestNode() ;
		Node n1 = createNodeSave(testFirstNode, objectType, "n1") ;
		Node n2 = createNodeSave(testFirstNode, getRobotType(), "n2") ;
		Node n2child = createNodeSave(n2, objectType, "n2-child") ;
		Node n2child2 = createNodeSave(n2, objectType, "n2-child2") ;
		Node n2gchild = createNodeSave(n2child, objectType, "n2-gchild") ;
	    
		assertEquals(0, n1.getChildNode().size()) ;
		n2.copy(n1) ;
		
		session.save() ;
		assertEquals(1, n1.getChildNode().size()) ;
		assertEquals("n2", n1.getChildNode().next().getName()) ;
		
		String explain = "explain";
		n2.setProperty(explain, "n2 explain") ;
		
		assertEquals("n2 explain", n2.getProperty(explain).getValue().getString()) ;
		assertEquals(null, n1.getChildNode().next().getProperty(explain).getValue().getString()) ;

		assertEquals(2, n2.getChildNode().size()) ;
		assertEquals(2,session.getLocalStore().getNodes(n2, LinkTypeImpl.CHILD, Page.ALL).size()) ;
		assertEquals(1, n2child.getChildNode().size()) ;

		assertEquals(3, n2.getDescentNode(100).size()) ;
    }
	
	

}
