package com.bleujin.lore.core.workspace;

import com.bleujin.lore.core.TestCaseParent;
import com.bleujin.lore.core.dimpl.TransientLocalStore;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ItemNotFoundException;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.persitence.ILocalStore;

public class TestWorkspace extends TestCaseParent{
	
	private Workspace otherWorkspace ;
	private String otherWorkspaceName = "other_workspace";
	public void setUp() throws Exception {
		super.setUp() ;
		// for test
		ILocalStore newLocal = TransientLocalStore.create("d:/anotherlore/test2/" + otherWorkspaceName + ".dat", "D:/anotherlore/test2");
		
		otherWorkspace = Workspace.create(otherWorkspaceName, anotherLore, newLocal) ;
		
		anotherLore.addNewWorkspace(otherWorkspace) ;
	}
	
	
	
	public void testLoginNotExistWorkspace() throws Exception {
		try {
			UserSession bleuhero = anotherLore.login(new TestCredential("bleuhero"), "unknown_workspace") ;
			fail() ;
		} catch(IllegalArgumentException ignore){
			
		}
    }
	
	public void testOtherLoginAtSameWorkspace() throws Exception {
	    UserSession otherSession = anotherLore.login(new TestCredential("otherUser")) ;
	    assertEquals(session.getWorkspace(), otherSession.getWorkspace()) ;
	} 
	
	public void testRecognizeWorkspace() throws Exception {
		assertEquals(anotherLore.DEFAULT_WORKSPACE, session.getWorkspace().getName());

	}
	public void testTwoWorkspace() throws Exception {
		UserSession otherSession = anotherLore.login(new TestCredential("bleujin"), otherWorkspaceName) ;
		
		INode otherRootNode = otherSession.createNode(session.getRootNode(), objectType, "newbie") ;
		
	}
	
	public void testNodeHaveWorkspaceName() throws Exception {
		Node node = session.getRootNode() ;
		assertEquals(session.getWorkspace().getName(), node.getWorkspaceName());
		
		Node created = saveNode(objectType, "new") ;
		assertEquals(session.getWorkspace().getName(), created.getWorkspaceName());
	}
	
	public void testWorkspaceName() throws Exception {
		Node node = session.getRootNode() ;
		
		assertEquals(anotherLore.DEFAULT_WORKSPACE, node.getWorkspaceName()) ;
		Node bleujin = saveNode(objectType, "bleujin") ;
		
		assertEquals(anotherLore.DEFAULT_WORKSPACE, bleujin.getWorkspaceName()) ;
		session.save() ;
		
		UserSession otherSession = anotherLore.login(Credential.ALL_OK) ;
		Node bleujinOther = otherSession.getNode("/bleujin") ;
		assertEquals(anotherLore.DEFAULT_WORKSPACE, bleujinOther.getWorkspaceName()) ;
		
		
		UserSession sessionOfOtherWorkspace = anotherLore.login(Credential.ALL_OK, otherWorkspaceName) ;

		try {
			Node bnode = sessionOfOtherWorkspace.getNode(bleujin.getUUIDString()) ;
			fail() ;
		} catch(ItemNotFoundException ignore){
		}
		
		try {
			Node bnode = sessionOfOtherWorkspace.getNode("/bleujin") ;
			fail() ;
		} catch(ItemNotFoundException ignore){
		}
	
		Node bnode = otherSession.getNode(bleujin.getUUIDString()) ;
		assertEquals(anotherLore.DEFAULT_WORKSPACE, bnode.getWorkspaceName()) ;
		
		
	}
	
	

}
