package com.bleujin.lore.core.workspace;


import java.util.ArrayList;

import com.bleujin.framework.configuration.Configuration;
import com.bleujin.framework.configuration.ConfigurationException;
import com.bleujin.framework.configuration.NotBuildException;
import com.bleujin.framework.configuration.NotFoundXmlTagException;
import com.bleujin.framework.util.InstanceCreationException;
import com.bleujin.framework.util.InstanceCreator;
import com.bleujin.framework.util.StringUtil;
import com.bleujin.lore.addon.security.Member;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.common.GenericIterator;
import com.bleujin.lore.core.common.GenericIteratorHelper;
import com.bleujin.lore.core.dimpl.ShareInfo;
import com.bleujin.lore.core.dimpl.ShareNodeImpl;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.ShareNode;
import com.bleujin.lore.core.observation.UserBean;
import com.bleujin.lore.core.persitence.GlobalStore;
import com.bleujin.lore.core.persitence.ILocalStore;
import com.bleujin.lore.core.security.AuthoritySetting;
import com.bleujin.lore.core.security.DefaultSecurityFilter;
import com.bleujin.lore.core.security.NodeAction;
import com.bleujin.lore.core.security.NodeResource;
import com.bleujin.lore.core.security.WorkspaceSecurityFilter;
import com.bleujin.lore.core.valid.ValidChain;

public class Workspace {

	private final String name;

	private final AnotherLore lore ;
	// workspace storage..
	private final ILocalStore localStore ;
	private final LockManager lockManager ;
	private final WorkspaceSecurityFilter securityFilter ;
	
	private Workspace(String workspaceId, AnotherLore anotherLore, ILocalStore localStore){
		this.name = workspaceId ;
		this.lore = anotherLore ;
		this.localStore = localStore ;
		this.lockManager = new LockManager() ;
		this.securityFilter = new DefaultSecurityFilter(AuthoritySetting.createDefault()) ;
	}
	
	public static Workspace create(String workspaceName, AnotherLore lore, ILocalStore localStore){
		if (workspaceName.indexOf("/") > -1 || workspaceName.indexOf("-") > -1 || (! StringUtil.isAlphaUnderbarStartWith(workspaceName))) throw new IllegalArgumentException("not supported name : " + workspaceName);
			
		return new Workspace(workspaceName, lore, localStore) ;
	}
	
	static Workspace create(Configuration config) throws NotFoundXmlTagException, NotBuildException, InstanceCreationException, ConfigurationException {
		String workspaceName = config.getAttribute("name") ;
		ILocalStore localStore = (ILocalStore)InstanceCreator.createConfiguredInstance(config.getChild("local-storage.configured-object"));
		
		Debug.debug(Workspace.class, workspaceName);
		
		// TODO Auto-generated method stub
		return null;
	}
	

	

	public ILocalStore getLocalStore() {
		return localStore ;
	}

	public String getName() {
		return name;
	}

	public GlobalStore getGlobalStore() {
		return lore.getGlobalStore();
    }
	
	public ObservationManager getObservationManager(){
		return lore.getObservationManager() ;
	}


	public ValidChain getValidChain() {
	    return lore.getValidChain() ;
    }

	public void clearStore() {
		localStore.clear() ;
	}


	public LockManager getLockManager() {
		return lockManager;
	}

	public boolean isAllowed(UserBean userBean, Node node, String actions) throws ALRepositoryException {
		return securityFilter.isAllowed(new NodeResource(node), new Member(userBean.getUserId()), NodeAction.create(actions));
	};

	
	private AuthoritySetting as = null ;
	private AuthoritySetting getSetting(){
		if (as == null){
			as = new AuthoritySetting() ;
			
		}
		return as ;
	}
	
	
	// TODO
	public void copy() {};
	public void imp() {};
	public void exp() {}

	public void shareNode(String shareName, String nodePath, boolean enableWrite) {
		getGlobalStore().shareNode(shareName, getName(), nodePath, enableWrite) ;
		
	}

	public ShareNode getShareNode(UserSession session, String shareName) throws ALRepositoryException {
		ShareInfo info = getGlobalStore().getShareInfo(shareName) ;
		return lore.getNode(session, info) ;
	}

	public GenericIterator<ShareNode> getShareNodes(UserSession session) throws ALRepositoryException {
		GenericIterator<ShareInfo> infos = getGlobalStore().getShareInfos() ;
		
		ArrayList<ShareNode> result = new ArrayList<ShareNode>() ;
		for (ShareInfo info : infos) {
			result.add(lore.getNode(session, info));
		}
		return new GenericIteratorHelper<ShareNode>(result) ;
	}


	ShareNode getShareNode(UserSession session, ShareInfo shareInfo) throws ALRepositoryException {
		Node node = localStore.getIternalShareNode(session, getName(), shareInfo) ;
		return ShareNodeImpl.shareRoot(session, node, shareInfo);
	}

	public void close() {
		localStore.close() ;
	}

}



