package com.bleujin.lore.core.workspace.config;

import com.bleujin.framework.configuration.Configuration;
import com.bleujin.framework.configuration.ConfigurationException;
import com.bleujin.framework.configuration.NotBuildException;
import com.bleujin.framework.configuration.NotFoundXmlTagException;
import com.bleujin.framework.util.StringUtil;

public class FileRepositoryInfo {
	private static final String DEFAULT_RESOURCE_ROOT = "data/resource";
	private static final String DEFAULT_PROPERTY_FILE_ROOT = "data/property";
	private static final String DEFAULT_INDEX_ROOT = "data/index";
	private static final String DEFAULT_OBSERVER_ROOT = "data/event";

	private String baseDir = null;

	private String resourceRoot = null;
	private String propertyfileRoot = null;
	private String indexRoot = null;

	public FileRepositoryInfo(Configuration config) throws NotBuildException, ConfigurationException  {
		// webapp doc base

		this.baseDir = config.getAttribute("base");
		if (StringUtil.isBlank(baseDir)) {
			baseDir = "/anotherlore/" ;
		} else if (!(baseDir.endsWith("/") || baseDir.endsWith("\\"))) {
			baseDir += "/";
		}

		// set
		this.resourceRoot = parseRoot("resource-root", config, DEFAULT_RESOURCE_ROOT);
		this.propertyfileRoot = parseRoot("property-file-root", config, DEFAULT_PROPERTY_FILE_ROOT);
		this.indexRoot = parseRoot("index-root", config, DEFAULT_INDEX_ROOT);
	}
	
	public FileRepositoryInfo() {
		this.baseDir = "/anotherlore/" ;
		this.resourceRoot = baseDir + DEFAULT_RESOURCE_ROOT ;
		this.propertyfileRoot = baseDir + DEFAULT_PROPERTY_FILE_ROOT ;
		this.indexRoot = baseDir + DEFAULT_INDEX_ROOT ;
	}

	private String parseRoot(String configName, Configuration config, String defaultDir) throws NotBuildException, ConfigurationException {
		String root;
		try {
			root = config.getChild(configName).getValue(defaultDir);
		} catch (NotFoundXmlTagException e) {
			root = defaultDir ;
		}
		
		if (!(root.startsWith("/") || root.startsWith("\\") || root.indexOf(":") == 1)) {
			if (root.startsWith("./") || root.startsWith(".\\")) {
				root = root.substring("./".length());
			}
			root = baseDir + root;
		}
		return root;
	}


	public String baseDir() {
		return baseDir;
	}
	
	public String getPropertyfileRoot() {
		return propertyfileRoot;
	}

	public String getResourceRoot() {
		return resourceRoot;
	}

	public String getIndexRoot() {
		return indexRoot;
	}
	
	public String toString(){
		return "baseDir = " + this.baseDir + "\n" +
		"resource-root = " + this.resourceRoot + "\n" +  
		"property-file-root = " + this.propertyfileRoot + "\n" +
		"index-root = " + this.indexRoot + "\n" ;
		
	}
}
