package com.bleujin.lore.core.workspace.config;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import com.bleujin.framework.configuration.ConfigurationException;
import com.bleujin.framework.configuration.ConfigurationFactory;
import com.bleujin.framework.db.DBController;
import com.bleujin.framework.util.PathMaker;
import com.bleujin.lore.common.Debug;
import com.bleujin.lore.core.dimpl.CacheGlobalStore;
import com.bleujin.lore.core.dimpl.CacheLocalStore;
import com.bleujin.lore.core.dimpl.DatabaseGlobalStore;
import com.bleujin.lore.core.dimpl.DatabaseLocalStore;
import com.bleujin.lore.core.dimpl.TransientGlobalStore;
import com.bleujin.lore.core.dimpl.TransientLocalStore;
import com.bleujin.lore.core.persitence.GlobalStore;
import com.bleujin.lore.core.persitence.ILocalStore;
import com.bleujin.lore.core.search.indexer.IndexDirectory;
import com.bleujin.lore.core.search.indexer.NodeIndexer;
import com.bleujin.lore.core.workspace.AnotherLore;
import com.bleujin.lore.core.workspace.Workspace;

public class InitLore {

	public static AnotherLore init () throws ConfigurationException{
//		return initTest() ;
		return init("D:/eclipse/workspace/anotherLore/webapps/board/WEB-INF/anotherlore-config.xml") ;
	}

	public static AnotherLore initTest() throws ConfigurationException, IOException, ClassNotFoundException {
		String indexDir = "/anotherlore/test/index";
		String savePath = "/anotherlore/test";
		deleteIndexDir(indexDir);		
		return initTest(savePath, indexDir) ;
	}
	
	public static AnotherLore initTest(DBController dc) throws ConfigurationException, IOException {
		String indexDir = "/anotherlore/data/index";
		
		deleteIndexDir(indexDir);		
		return initTest(dc, indexDir) ;
	}
	
	private static AnotherLore initTest(DBController dc, String indexParentName) throws ConfigurationException, IOException{
		deleteIndexDir(indexParentName);
		AnotherLore newLore = new AnotherLore() ;
		
		GlobalStore gstore = new CacheGlobalStore(DatabaseGlobalStore.create(dc)) ;
		
		IndexDirectory indexDir = IndexDirectory.createIndexDirectory(indexParentName, "testcase") ;
		NodeIndexer indexer = new NodeIndexer(indexDir);
		ILocalStore lstore = new CacheLocalStore(DatabaseLocalStore.create(dc, indexer)) ;
		// ILocalStore lstore = new DatabaseLocalStore(dc, indexer) ;
		Workspace workspace = Workspace.create(AnotherLore.DEFAULT_WORKSPACE, newLore, lstore) ;
		
		Map<String, Workspace> workspaces = new HashMap<String, Workspace>() ;
		workspaces.put(AnotherLore.DEFAULT_WORKSPACE, workspace) ;
		
		newLore.initTest(new LicenseInfo(), new FileRepositoryInfo(), gstore, indexParentName, workspaces);
		
		return newLore;
	}

	private static void deleteIndexDir(String indexDir) {
		File file = new File(PathMaker.getFilePath(indexDir, "testcase"));
		if (file.exists() && file.isDirectory()) {
			File[] files = file.listFiles() ;
			for(File f : files){
				f.delete() ;
			}
		}
	}
	private static AnotherLore initTest(String savePath, String indexParentName) throws ConfigurationException, IOException, ClassNotFoundException{
		AnotherLore newLore = new AnotherLore() ;
		
		GlobalStore gstore = TransientGlobalStore.create(savePath+ "/global.dat") ;
		
		IndexDirectory indexDir = IndexDirectory.createIndexDirectory(indexParentName, "testcase") ;
		//NodeIndexer indexer = new NodeIndexer(indexDir);
		// ILocalStore lstore = new CacheLocalStore(TransientLocalStore.create(savePath + "/workspace.dat", indexParentName + "/testcase")) ;
		ILocalStore lstore = TransientLocalStore.create(savePath + "/testcase.dat", indexParentName + "/testcase") ;
		Workspace workspace = Workspace.create(AnotherLore.DEFAULT_WORKSPACE, newLore, lstore) ;
		
		Map<String, Workspace> workspaces = new HashMap<String, Workspace>() ;
		workspaces.put(AnotherLore.DEFAULT_WORKSPACE, workspace) ;
		
		newLore.initTest(new LicenseInfo(), new FileRepositoryInfo(), gstore, indexParentName, workspaces);
		
		return newLore;
	}	
	
	public static AnotherLore init(String configFileName) throws ConfigurationException{
		
		ConfigurationFactory defaultConfigFactory = null;
		defaultConfigFactory = ConfigurationFactory.getInstance("anotherlore-config");
		defaultConfigFactory.build(configFileName);

		final AnotherLore newLore = new AnotherLore() ;
		newLore.init(defaultConfigFactory.getConfiguration("root")) ;
		
		// hook;
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				Debug.debug(this, "******************* ");
				try {
					Debug.debug(this, "Shutdown..... ing.... ");
					newLore.destroyWorld();
					Debug.debug(this, "Shutdown..... Lore destroyed");
				} catch (Exception ex) {
					Debug.debug(this, "Shutdown..... But Exception Occured");
				}
			}
		});

		return newLore;
	}
}
