package com.bleujin.lore.core.workspace.config;

import java.io.IOException;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import com.bleujin.framework.configuration.Configuration;
import com.bleujin.framework.configuration.ConfigurationException;
import com.bleujin.framework.db.DBController;
import com.bleujin.framework.db.DBControllerInstantiationException;
import com.bleujin.framework.util.PathMaker;
import com.bleujin.lore.core.dimpl.CacheGlobalStore;
import com.bleujin.lore.core.dimpl.CacheLocalStore;
import com.bleujin.lore.core.dimpl.DatabaseGlobalStore;
import com.bleujin.lore.core.dimpl.DatabaseLocalStore;
import com.bleujin.lore.core.dimpl.TransientGlobalStore;
import com.bleujin.lore.core.dimpl.TransientLocalStore;
import com.bleujin.lore.core.persitence.GlobalStore;
import com.bleujin.lore.core.persitence.ILocalStore;
import com.bleujin.lore.core.search.indexer.IndexDirectory;
import com.bleujin.lore.core.search.indexer.NodeIndexer;
import com.bleujin.lore.core.workspace.AnotherLore;
import com.bleujin.lore.core.workspace.Workspace;

public class StoragesInfo {

	private Map<String, Configuration> store = new HashMap<String, Configuration>();

	private StoragesInfo(Configuration[] configs) {
		for (Configuration config : configs) {
			String storageId = config.getAttribute("id");
			String storageType = config.getAttribute("type");

			store.put(storageId, config);
		}
	}

	public GlobalStore getGlobalStore(GlobalStoreInfo ginfo) throws ConfigurationException {
		try {
			String refId = ginfo.getReferenceId();
			Configuration config = store.get(refId);
			String type = config.getAttribute("type");
			if ("db.oracle".equals(type)) {
				DBController dc = new DBController(config.getChild("database-controller"));
				dc.init();
				return new CacheGlobalStore(DatabaseGlobalStore.create(dc));
			} else {
				return TransientGlobalStore.create(config.getChild("file-path").getValue());
			}
		} catch (DBControllerInstantiationException ex) {
			throw new ConfigurationException(ex);
		} catch (SQLException ex) {
			throw new ConfigurationException(ex);
		} catch (IOException ex) {
			throw new ConfigurationException(ex);
		} catch (ClassNotFoundException ex) {
			throw new ConfigurationException(ex);
		}
	}

	public Map<String, Workspace> getWorkspaces(AnotherLore lore, WorkspacesInfo wsinfo, String indexParentDir) throws ConfigurationException {
		try {
			Map<String, Workspace> result = new HashMap<String, Workspace>();

			for (WorkspaceInfo info : wsinfo.getInfo()) {
				String refId = info.getReferenceId();
				
				Configuration config = store.get(refId);
				String type = config.getAttribute("type");
				ILocalStore localStore = null;
				
				String workspaceName = info.getName();
				IndexDirectory indexDir = IndexDirectory.createIndexDirectory(indexParentDir, workspaceName) ;
				NodeIndexer indexer = new NodeIndexer(indexDir) ;
				
				if ("db.oracle".equals(type)) {
					DBController dc = new DBController(config.getChild("database-controller"));
					dc.init();
					localStore = new CacheLocalStore(DatabaseLocalStore.create(dc, indexer));
				} else {
					String saveDir = config.getChild("file-path").getValue();
					localStore = TransientLocalStore.create(PathMaker.getFilePath(saveDir, workspaceName + ".dat"), PathMaker.getFilePath(indexParentDir, workspaceName));
				}
				result.put(workspaceName, Workspace.create(workspaceName, lore, localStore));

				if (info.isDefault()) {
					result.put(AnotherLore.DEFAULT_WORKSPACE, Workspace.create(workspaceName, lore, localStore));
				}
			}

			return result;
		} catch (IOException ex) {
			throw new ConfigurationException(ex);
		} catch (DBControllerInstantiationException ex) {
			throw new ConfigurationException(ex);
		} catch (SQLException ex) {
			throw new ConfigurationException(ex);
		}
	}

	public final static StoragesInfo create(Configuration[] config) throws ConfigurationException {
		return new StoragesInfo(config);
	}

}
