package com.bleujin.lore.extend.template;

import java.io.StringWriter;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.nodetype.NodeType;

public class BindTemplate implements Template {
 
	private static final long serialVersionUID = 4805005240643369221L;
	private transient Node node;
	private TemplateId tplId;
	private String content;
	private transient UserSession session;
	private transient TemplateManager templateManager ;

	private BindTemplate(Node node, TemplateId tplId, String content) {
		this.node = node;
		this.session = node.getSession();
		this.templateManager = session.getTemplateManager() ;
		this.tplId = tplId;
		this.content = content;
	}

	public static Template create(Node node, TemplateId tplId, String content) {
		return new BindTemplate(node, tplId, content);
	}

	public static Template createDefault(Node node) {
		return create(node, TemplateId.createId("/"), "$node.getName()");
	}

	public void bind(Node target) throws ALRepositoryException {
		templateManager.bindTemplate(target, tplId);
	}

	public void bind(NodeType target) throws ALRepositoryException {
		templateManager.bindTemplate(target, tplId);
	}

	public String applyTemplate() throws ALRepositoryException {
		try {
			VelocityEngine engine = new VelocityEngine();
			engine.init();

			VelocityContext context = new VelocityContext();
			context.put("node", this.node);
			context.put("session", this.session);

			StringWriter writer = new StringWriter();
			engine.evaluate(context, writer, "logName", getContent());
			return writer.toString();

		} catch (Exception e) {
			throw new ALRepositoryException(e.getMessage());
		}
	}

	public String getContent() throws ALRepositoryException {
		return content;
	}

}
