package com.bleujin.lore.extend.template;

import java.io.StringWriter;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.nodetype.NodeType;

public class TemplateImpl implements Template{
 
	private static final long serialVersionUID = -5918029081949445251L;
	private transient UserSession session ;
	private transient TemplateManager templateManager ;
	private TemplateId tplId ;
	private String content ;
	
	private TemplateImpl(UserSession session, TemplateId tplId, String content) {
		this.session = session ;
		this.templateManager = session.getTemplateManager() ;
		this.tplId = tplId ;
		this.content = content ;
	}
	
	public final static Template create(UserSession session, String tplId, String content) throws ALRepositoryException{
		TemplateImpl template = new TemplateImpl(session, TemplateId.createId(tplId), content) ;
		return session.getTemplateManager().saveTemplate(template.getId(), template) ;
	}

	public final static Template createTemporary(Node node, String tplId, String content) throws ALRepositoryException{
		Template template = BindTemplate.create(node, TemplateId.createId(tplId), content) ;
		return template ;
	}
	
	public final static Template load(UserSession session, TemplateId tplId, String content){
		return new TemplateImpl(session, tplId, content) ;
	}
	
	public TemplateId getId() {
		return this.tplId;
	}
	
	public String getContent() {
		return content;
	}
	
	public String applyTemplate() throws ALRepositoryException{
		try {
			VelocityEngine engine = new VelocityEngine();
			//engine.setProperty(RuntimeConstants.RUNTIME_LOG_LOGSYSTEM_CLASS, TemplateLogger.class.getName());
			engine.init();

			VelocityContext context = new VelocityContext();
			context.put("session", this.session);

			StringWriter writer = new StringWriter();
			engine.evaluate(context, writer, "anonyLog", this.content);
			return writer.toString();

		} catch (Exception e) {
			throw new ALRepositoryException(e.getMessage());
		}
	}

	public static Template createDefault(UserSession session) {
		return new TemplateImpl(session, TemplateId.createId("/"), "$node.getName()");
	}

	public void bind(NodeType nodetype) throws ALRepositoryException {
		templateManager.bindTemplate(nodetype, getId()) ;
		
	}

	public void bind(Node node) throws ALRepositoryException {
		templateManager.bindTemplate(node, getId()) ;
	}



}
