package com.bleujin.lore.extend.template;

import java.io.StringWriter;

import junit.framework.TestCase;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.runtime.RuntimeConstants;

public class TestVelocity extends TestCase{

	public void testInit() throws Exception {
		VelocityEngine engine = new VelocityEngine();
		engine.init();

		VelocityContext context = new VelocityContext();
		context.put("name", "bleujin");

		StringWriter writer = new StringWriter();
		engine.evaluate(context, writer, "logName", "Hello $name");
		
		assertEquals("Hello bleujin", writer.toString()) ;
	}
	
	public void testContextChain() throws Exception {

		VelocityEngine engine = new VelocityEngine();
		engine.init();

		VelocityContext context = new VelocityContext();
		context.put("name", "bleujin");

		VelocityContext inner = new VelocityContext(context) ;
		inner.put("name", "bleuhero") ;
		inner.put("bye", "ByeBye") ;
		
		StringWriter writer = new StringWriter();

		engine.evaluate(inner, writer, "logName", "Hello $name");
		assertEquals("Hello bleuhero", writer.toString()) ;

		writer = new StringWriter() ;
		
		engine.evaluate(inner, writer, "logName", "Hello $bye");
		assertEquals("Hello ByeBye", writer.toString()) ;
	}
	
	public void testHangul() throws Exception {
		VelocityEngine engine = new VelocityEngine();
		engine.init();

		VelocityContext context = new VelocityContext();
		context.put("name", "블루진");
		
		StringWriter writer = new StringWriter();
		engine.evaluate(context, writer, "logName", "안녕 $name");
		
		assertEquals("안녕 블루진", writer.toString()) ;
	}
	
	public void xtestLogger() throws Exception {
		VelocityEngine engine = new VelocityEngine();
		engine.setProperty(RuntimeConstants.RUNTIME_LOG_LOGSYSTEM_CLASS, TemplateLogger.class.getName());
		engine.init();

		VelocityContext context = new VelocityContext();
		context.put("name", "블루진");
		
		StringWriter writer = new StringWriter();
		engine.evaluate(context, writer, "logName", "안녕 $name");
		
		assertEquals("안녕 블루진", writer.toString()) ;
	}
	
	
}
