package com.bleujin.lore.impl.sample;

import com.bleujin.framework.util.RandomUtil;
import com.bleujin.lore.core.common.IDString;
import com.bleujin.lore.core.dimpl.NodeTypeManager;
import com.bleujin.lore.core.dimpl.UserSession;
import com.bleujin.lore.core.exception.ALRepositoryException;
import com.bleujin.lore.core.exception.NamespaceException;
import com.bleujin.lore.core.exception.UnsupportedRepositoryOperationException;
import com.bleujin.lore.core.node.INode;
import com.bleujin.lore.core.node.Node;
import com.bleujin.lore.core.node.Property;
import com.bleujin.lore.core.nodetype.NodeType;
import com.bleujin.lore.core.nodetype.PropertyDefinition;

public class RandomSampleNode {

	private int empCount = 100;
	private int jobCount = 5;
	private int locCount = 10;
	private int deptCount = 10;
	
	private UserSession session ;
	private NodeTypeManager ntManager ;
	RandomSampleNode(UserSession session){
		this(session, 1.0F) ;
	}
	
	RandomSampleNode(UserSession session, float weight){
		assert weight > 0 && weight < 10 ;
		
		this.session = session ;
		this.ntManager = session.getNodeTypeManager() ;
		this.empCount = (int)(empCount * weight) ;
		this.jobCount = (int)(jobCount * weight) ;
		this.locCount = (int)(locCount * weight) ;
		this.deptCount = (int)(deptCount * weight) ;
	}
	
	
	public void imp(NodeType rootType, Node rootNode) throws ALRepositoryException {
		createNodeType(rootType) ;
		createNode(rootNode) ;
		session.save() ;
	}


	private void createNode(Node parentNode) throws ALRepositoryException {
		NodeType objectType = ntManager.getNodeType("sample:object") ;
		NodeType emp = ntManager.getNodeType("sample:emp") ;

		String[] locIds = createLocNode(objectType, parentNode) ;
		String[] deptIds = createDeptNode(objectType, parentNode, locIds);
		String[] jobIds = createJobNode(objectType, parentNode) ;
		createEmpNode(objectType, parentNode, locIds, deptIds, jobIds) ;
	}


	
	private int[] createEmpNode(NodeType objectType, Node sampleRoot, String[] locIds, String[] deptIds, String[] jobIds) throws NamespaceException, ALRepositoryException {
		NodeType emp = ntManager.getNodeType("sample:emp") ; 
		Node empRoot = session.createNode(sampleRoot, objectType, "_emp").save() ;

		int[] empIds = RandomUtil.getRandomIntArray(10, 100000, empCount) ;
		for (int i = 0; i < empCount; i++) {
			INode newNode = session.createNode(empRoot, emp, String.valueOf(empIds[i])) ;
			newNode.setProperty("empId", String.valueOf(empIds[i])) ;
			newNode.setProperty("ename", RandomUtil.nextRandomString(15)) ;
			newNode.setProperty("email", RandomUtil.nextRandomString(25)) ;
			newNode.setProperty("jobId", IDString.unknownId(jobIds[RandomUtil.nextInt(jobIds.length)])) ;
			newNode.setProperty("hiredate", RandomUtil.nextCalendar(2000)) ;
			newNode.setProperty("salary", RandomUtil.nextInt(empCount)) ;
			newNode.setProperty("managerId", i == 0 ? "" : String.valueOf(empIds[0])) ;
			newNode.setProperty("deptId", IDString.unknownId(deptIds[RandomUtil.nextInt(deptIds.length)])) ;
		}

		return empIds ;	
	}


	private String[] createJobNode(NodeType objectType, Node sampleRoot) throws NamespaceException, ALRepositoryException {
		NodeType loc = ntManager.getNodeType("sample:job") ; 
		Node locRoot = session.createNode(sampleRoot, objectType, "_job").save() ;
		String[] jobIds = new String[jobCount] ;
		for (int i = 0; i < jobCount; i++) {
			jobIds[i] = RandomUtil.nextRandomString(10);
			INode newNode = session.createNode(locRoot, loc, jobIds[i]) ;
			newNode.setProperty("jobId", IDString.unknownId(jobIds[i])) ;
			newNode.setProperty("jobtitle", RandomUtil.nextRandomString(20)) ;
			newNode.setProperty("min_salary", RandomUtil.nextInt(100)) ;
			newNode.setProperty("max_salary", RandomUtil.nextInt(50) + 100) ;
		}

		return jobIds ;		
	}


	private String[] createLocNode(NodeType objectType, Node sampleRoot) throws NamespaceException, ALRepositoryException {
		NodeType loc = ntManager.getNodeType("sample:loc") ; 
		Node locRoot = session.createNode(sampleRoot, objectType, "_loc").save() ;
		
		String[] locIds = new String[locCount] ;
		for (int i = 0; i < locCount; i++) {
			locIds[i] = RandomUtil.nextRandomString(locCount, RandomUtil.DOWNCASE_CHAR_TYPE) ;
			INode newNode = session.createNode(locRoot, loc, locIds[i]) ;
			
			
			newNode.setProperty("locId", locIds[i]) ;
			newNode.setProperty("address", RandomUtil.nextRandomString(20)) ;
			newNode.setProperty("city", RandomUtil.nextRandomString(20)) ;
		}
		
		return locIds ;
	}


	private String[] createDeptNode(NodeType objectType, Node sampleRoot, String[] locIds) throws ALRepositoryException {
		NodeType dept = ntManager.getNodeType("sample:dept") ; 
		Node deptRoot = session.createNode(sampleRoot, objectType, "_dept").save() ;
		String[] deptIds = new String[deptCount] ;
		for (int i = 0; i < deptCount; i++) {
			deptIds[i] = String.valueOf(i*deptCount+deptCount) ;
			INode newNode = session.createNode(deptRoot, dept, deptIds[i]) ;
			newNode.setProperty("deptId", IDString.unknownId(deptIds[i])) ;
			newNode.setProperty("dName", RandomUtil.nextRandomString(20)) ;
			newNode.setProperty("locId", locIds[RandomUtil.nextInt(deptCount)]) ;
		}
		return deptIds ;
	}

	private void registerNodeType(NodeType superType, String nodeTypeName, PropertyDefinition[] definitions) throws UnsupportedRepositoryOperationException, ALRepositoryException {
		session.getNodeTypeManager().register(session.getNodeTypeManager().createNodeTypeTemplate(superType, nodeTypeName, definitions), false);
	}

	private PropertyDefinition createPropertyDefinition(String propId, Property.Type type) throws ALRepositoryException{
		return session.getNodeTypeManager().createPropertyDefinition(propId, type) ;
	}
	
	private void createNodeType(NodeType superType) throws ALRepositoryException {

		ntManager.unregister("sample:loc") ;
		PropertyDefinition locId = createPropertyDefinition("locId", Property.Type.STRING);
		PropertyDefinition address = createPropertyDefinition("address", Property.Type.STRING);
		PropertyDefinition city = createPropertyDefinition("city", Property.Type.STRING);
		registerNodeType(superType, "sample:loc", new PropertyDefinition[]{locId, address, city}) ;
		

		ntManager.unregister("sample:dept") ;
		PropertyDefinition deptId = createPropertyDefinition("deptId", Property.Type.NAME);
		PropertyDefinition dName = createPropertyDefinition("dName", Property.Type.STRING);
		registerNodeType(superType, "sample:dept", new PropertyDefinition[]{deptId, dName, locId}) ;

		ntManager.unregister("sample:job") ;
		PropertyDefinition jobId = createPropertyDefinition("jobid", Property.Type.NAME);
		PropertyDefinition jobtitle = createPropertyDefinition("jobtitle", Property.Type.STRING);
		PropertyDefinition minsalary = createPropertyDefinition("min_salary", Property.Type.LONG);
		PropertyDefinition maxsalary = createPropertyDefinition("max_salary", Property.Type.LONG);
		registerNodeType(superType, "sample:job", new PropertyDefinition[]{jobId, jobtitle, minsalary, maxsalary}) ;
		

		ntManager.unregister("sample:emp") ;
		PropertyDefinition empId = createPropertyDefinition("empId", Property.Type.STRING);
		PropertyDefinition ename = createPropertyDefinition("ename", Property.Type.STRING);
		PropertyDefinition email = createPropertyDefinition("email", Property.Type.STRING);
		PropertyDefinition hiredate = createPropertyDefinition("hiredate", Property.Type.DATE);
		PropertyDefinition salary = createPropertyDefinition("salary", Property.Type.LONG);
		PropertyDefinition managerId = createPropertyDefinition("managerId", Property.Type.STRING);
		registerNodeType(superType, "sample:emp", new PropertyDefinition[]{empId, ename, email, jobId, hiredate, salary, managerId, deptId}) ;

	}
	

}
