package com.bleujin.lore.install.fetch;

import java.io.File;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.SystemUtils;

import com.bleujin.framework.db.DBController;
import com.bleujin.framework.db.procedure.IUserProcedure;
import com.bleujin.lore.install.ISetupTask;
import com.bleujin.lore.install.InstallOuter;

public class CreatePackage implements ISetupTask {

	private DBController dc;
	private InstallOuter outer ;
	private String fileName;

	public CreatePackage(DBController dc, InstallOuter outer, String fileName) {
		this.dc = dc;
		this.outer = outer ;
		this.fileName = fileName;
	}

	public boolean execute() {
		try {
			String read = FileUtils.readFileToString(new File(fileName), SystemUtils.FILE_ENCODING);

			String innerReg = "\\ncreate.*package\\W*([0-9a-zA-Z_#]*)";
			Pattern pattern = Pattern.compile(innerReg, Pattern.CASE_INSENSITIVE);

			String[] packages = read.split("\n/\n") ;
			outer.info("PACKAGE=====================");
	        for(String pack : packages){
				dc.execUpdate(pack) ;
	        	
	        	Matcher innerMatcher = pattern.matcher(pack) ;
	        	if(innerMatcher.find()){
	        		outer.log(innerMatcher.group(1) + " CREATED");
	        	}
	        }

	        for (int i = 0; i < 5; i++) {
	            IUserProcedure recompile = dc.createUserProcedureBatch("OnlyAdmin@RecompileInvalid()") ;
	            dc.execUpdate(recompile) ;
	            outer.info("Package recompile :" + i);
			}

		} catch (IOException e) {
			outer.warning(e.getMessage()) ;
			return false ;
		} 
		return true ;
	}

}