/*
 * Copyright 2006 Day Management AG, Switzerland. All rights reserved.
 */
package javax.jcr;

import java.io.InputStream;
import java.io.IOException;

/**
 * The <code>Binary</code> interface and its related methods in
 * <code>Property</code>, <code>Value</code> and <code>ValueFactory</code>
 * replace the deprecated {@link Value#getStream} and {@link Property#getStream}
 * methods. The <code>Binary</code> interface allows repositories to provide
 * implementations that handle JCR BINARY values in the most efficient manner,
 * given the specifics of their internal handling of the binary data.
 *
 * @since JCR 2.0
 */
public interface Binary {

    /**
     * Returns an <code>InputStream</code> representation of this value.
     * Each call to <code>getStream()</code> returns a new stream.
     * The API consumer is responsible for calling <code>close()</code>
     * on the returned stream.
     *
     * @return A stream representation of this value.
     *
     * @throws RepositoryException if another error occurs.
     */
    InputStream getStream() throws RepositoryException;

    /**
     * Reads successive bytes from the specified <code>position</code> in this <code>Binary</code>
     * into the passed byte array until either the byte array is full or the end of the
     * Binary is encountered.
     *
     * @param b the buffer into which the data is read.
     * @param position the position in this Binary from which to start reading bytes.
     * @return the number of bytes read into the buffer, or -1 if there is no more
     *         data because the end of the Binary has been reached.
     * @throws IOException              if an I/O error occurs.
     * @throws NullPointerException     if b is null.
     * @throws IllegalArgumentException if offset is negative.
     * @throws RepositoryException      if another error occurs.
     */
    int read(byte[] b, long position) throws IOException, RepositoryException;

    /**
     * Returns the size of this <code>Binary</code> value in bytes.
     *
     * @return the size of this value in bytes.
     *
     * @throws RepositoryException if another error occurs.
     */
    long getSize() throws RepositoryException;
}
