/*
 * Copyright 2006 Day Management AG, Switzerland. All rights reserved.
 */
package javax.jcr.observation;

import javax.jcr.RangeIterator;
import java.util.Calendar;

/**
 * Allows easy iteration through a list of <code>Event</code>s
 * with <code>nextEvent</code> as well as a <code>skip</code> method inherited from
 * <code>RangeIterator</code>.
 */
public interface EventIterator extends RangeIterator {

     /**
      * Returns the next <code>Event</code> in the iteration.
      *
      * @return the next <code>Event</code> in the iteration.
      * @throws java.util.NoSuchElementException if iteration has no more <code>Event</code>s.
     */
    public Event nextEvent();

    /**
     * Returns the date associated with this event iterator, or <code>null</code>.
     * <p>
     * The date is required to be non-null for event iterators obtained through
     * an {@link EventJournal}.
     * @return the date associated with this event iterator, or <code>null</code>.
     * @since JCR 2.0
     */
    public Calendar getDate();
}